@@ -15,8 +15,6 @@
 #include <config.h>
 #endif
 
-#include <assert.h>
-
 /* utility */
 #include "fcintl.h"
 #include "log.h"
@@ -65,11 +63,12 @@ void generate_citydlg_dimensions(void)
 {
   int min_x = 0, max_x = 0, min_y = 0, max_y = 0;
 
-  city_map_iterate(city_x, city_y) {
+  /* use maximum possible squared city radius. */
+  city_map_iterate(CITY_MAP_MAX_RADIUS_SQ, city_index, city_x, city_y) {
     int canvas_x, canvas_y;
 
-    map_to_gui_vector(tileset, &canvas_x, &canvas_y,
-		      city_x - CITY_MAP_RADIUS, city_y - CITY_MAP_RADIUS);
+    map_to_gui_vector(tileset, &canvas_x, &canvas_y, CITY_ABS2REL(city_x),
+                      CITY_ABS2REL(city_y));
 
     min_x = MIN(canvas_x, min_x);
     max_x = MAX(canvas_x, max_x);
@@ -85,21 +84,21 @@ void generate_citydlg_dimensions(void)
   Converts a (cartesian) city position to citymap canvas coordinates.
   Returns TRUE if the city position is valid.
 **************************************************************************/
-bool city_to_canvas_pos(int *canvas_x, int *canvas_y, int city_x, int city_y)
+bool city_to_canvas_pos(int *canvas_x, int *canvas_y, int city_x,
+                        int city_y, int city_radius_sq)
 {
-  const int x0 = CITY_MAP_RADIUS, y0 = CITY_MAP_RADIUS;
   const int width = get_citydlg_canvas_width();
   const int height = get_citydlg_canvas_height();
 
   /* The citymap is centered over the center of the citydlg canvas. */
-  map_to_gui_vector(tileset, canvas_x, canvas_y, city_x - x0, city_y - y0);
+  map_to_gui_vector(tileset, canvas_x, canvas_y, CITY_ABS2REL(city_x),
+                    CITY_ABS2REL(city_y));
   *canvas_x += (width - tileset_tile_width(tileset)) / 2;
   *canvas_y += (height - tileset_tile_height(tileset)) / 2;
 
-  if (!is_valid_city_coords(city_x, city_y)) {
-    assert(FALSE);
-    return FALSE;
-  }
+  fc_assert_ret_val(is_valid_city_coords(city_radius_sq, city_x, city_y),
+                    FALSE);
+
   return TRUE;
 }
 
@@ -107,9 +106,12 @@ bool city_to_canvas_pos(int *canvas_x, int *canvas_y, int city_x, int city_y)
   Converts a citymap canvas position to a (cartesian) city coordinate
   position.  Returns TRUE iff the city position is valid.
 **************************************************************************/
-bool canvas_to_city_pos(int *city_x, int *city_y, int canvas_x, int canvas_y)
+bool canvas_to_city_pos(int *city_x, int *city_y, int city_radius_sq,
+                        int canvas_x, int canvas_y)
 {
+#ifdef DEBUG
   int orig_canvas_x = canvas_x, orig_canvas_y = canvas_y;
+#endif
   const int width = get_citydlg_canvas_width();
   const int height = get_citydlg_canvas_height();
 
@@ -135,13 +137,13 @@ bool canvas_to_city_pos(int *city_x, int *city_y, int canvas_x, int canvas_y)
 
   /* Add on the offset of the top-left corner to get the final
    * coordinates (like in canvas_to_map_pos). */
-  *city_x += CITY_MAP_RADIUS;
-  *city_y += CITY_MAP_RADIUS;
+  *city_x += CITY_MAP_MAX_RADIUS;
+  *city_y += CITY_MAP_MAX_RADIUS;
 
-  freelog(LOG_DEBUG, "canvas_to_city_pos(pos=(%d,%d))=(%d,%d)",
-	  orig_canvas_x, orig_canvas_y, *city_x, *city_y);
+  log_debug("canvas_to_city_pos(pos=(%d,%d))=(%d,%d)@radius=%d",
+            orig_canvas_x, orig_canvas_y, *city_x, *city_y, city_radius_sq);
 
-  return is_valid_city_coords(*city_x, *city_y);
+  return is_valid_city_coords(city_radius_sq, *city_x, *city_y);
 }
 
 /* Iterate over all known tiles in the city.  This iteration follows the
@@ -156,8 +158,8 @@ bool canvas_to_city_pos(int *city_x, int *city_y, int canvas_x, int canvas_y)
 		    (pcity)->tile->x, (pcity)->tile->y);		\
   _x##_0 -= (_x##_w - tileset_tile_width(tileset)) / 2;			\
   _y##_0 -= (_y##_h - tileset_tile_height(tileset)) / 2;		\
-  freelog(LOG_DEBUG, "citydlg: %d,%d + %dx%d",				\
-	  _x##_0, _y##_0, _x##_w, _y##_h);				\
+  log_debug("citydlg: %d,%d + %dx%d",					\
+	    _x##_0, _y##_0, _x##_w, _y##_h);				\
 									\
   gui_rect_iterate(_x##_0, _y##_0, _x##_w, _y##_h,			\
 		   ptile, pedge, pcorner, _x##_g, _y##_g) {		\
@@ -195,6 +197,40 @@ void city_dialog_redraw_map(struct city *pcity,
   } mapview_layer_iterate_end;
 }
 
+/**************************************************************************
+  Return a string describing the the cost for the production of the city
+  considerung several build slots for units.
+**************************************************************************/
+char *city_production_cost_str(const struct city *pcity)
+{
+  static char cost_str[50];
+  int cost = city_production_build_shield_cost(pcity);
+  int build_slots = city_build_slots(pcity);
+  int num_units;
+
+  if (build_slots > 1
+      && city_production_build_units(pcity, TRUE, &num_units)) {
+    /* the city could build more than one unit of the selected type */
+    if (num_units == 0) {
+      /* no unit will be finished this turn but one is build */
+      num_units++;
+    }
+
+    if (build_slots > num_units) {
+      /* some build slots for units will be unused */
+      fc_snprintf(cost_str, sizeof(cost_str), "{%d*%d}", num_units, cost);
+    } else {
+      /* maximal number of units will be build */
+      fc_snprintf(cost_str, sizeof(cost_str), "[%d*%d]", num_units, cost);
+    }
+  } else {
+    /* nothing special */
+    fc_snprintf(cost_str, sizeof(cost_str), "%3d", cost);
+  }
+
+  return cost_str;
+}
+
 /**************************************************************************
   Find the city dialog city production text for the given city, and
   place it into the buffer.  This will check the
@@ -204,11 +240,11 @@ void city_dialog_redraw_map(struct city *pcity,
 void get_city_dialog_production(struct city *pcity,
 				char *buffer, size_t buffer_len)
 {
-  char time[50];
-  int turns, cost, stock;
+  char time_str[50], *cost_str;
+  int turns, stock;
 
   if (pcity == NULL) {
-    /* 
+    /*
      * Some GUIs use this to build a "filler string" so that they can
      * properly size the widget to hold the string.  This has some
      * obvious problems; the big one is that we have two forms of time
@@ -217,36 +253,40 @@ void get_city_dialog_production(struct city *pcity,
      * translators can fudge it by changing this "filler" string. 
      */
     /* TRANS: Use longer of "XXX turns" and "never" */
-    mystrlcpy(buffer, Q_("?filler:XXX/XXX XXX turns"), buffer_len);
+    fc_strlcpy(buffer, Q_("?filler:XXX/XXX XXX turns"), buffer_len);
 
     return;
   }
 
   if (city_production_has_flag(pcity, IF_GOLD)) {
-    my_snprintf(buffer, buffer_len, _("%3d gold per turn"),
-                MAX(0, pcity->surplus[O_SHIELD]));
+    int gold = MAX(0, pcity->surplus[O_SHIELD]);
+    fc_snprintf(buffer, buffer_len, PL_("%3d gold per turn",
+                                        "%3d gold per turn", gold), gold);
     return;
   }
+
   turns = city_production_turns_to_build(pcity, TRUE);
   stock = pcity->shield_stock;
-  cost = city_production_build_shield_cost(pcity);
+  cost_str = city_production_cost_str(pcity);
 
   if (turns < FC_INFINITY) {
     if (concise_city_production) {
-      my_snprintf(time, sizeof(time), "%3d", turns);
+      fc_snprintf(time_str, sizeof(time_str), "%3d", turns);
     } else {
-      my_snprintf(time, sizeof(time),
+      fc_snprintf(time_str, sizeof(time_str),
                   PL_("%3d turn", "%3d turns", turns), turns);
     }
   } else {
-    my_snprintf(time, sizeof(time), "%s",
+    fc_snprintf(time_str, sizeof(time_str), "%s",
                 concise_city_production ? "-" : _("never"));
   }
 
   if (concise_city_production) {
-    my_snprintf(buffer, buffer_len, _("%3d/%3d:%s"), stock, cost, time);
+    fc_snprintf(buffer, buffer_len, _("%3d/%s:%s"), stock, cost_str,
+                time_str);
   } else {
-    my_snprintf(buffer, buffer_len, _("%3d/%3d %s"), stock, cost, time);
+    fc_snprintf(buffer, buffer_len, _("%3d/%s %s"), stock, cost_str,
+                time_str);
   }
 }
 
@@ -270,9 +310,8 @@ void get_city_dialog_production_full(char *buffer, size_t buffer_len,
 
   switch (target.kind) {
   case VUT_IMPROVEMENT:
-    mystrlcpy(buffer,
-	      city_improvement_name_translation(pcity, target.value.building),
-	      buffer_len);
+    fc_strlcpy(buffer, city_improvement_name_translation
+               (pcity, target.value.building), buffer_len);
 
     if (improvement_has_flag(target.value.building, IF_GOLD)) {
       cat_snprintf(buffer, buffer_len, " (--) ");
@@ -312,8 +351,8 @@ void get_city_dialog_production_row(char *buf[], size_t column_size,
   {
     struct unit_type *ptype = target.value.utype;
 
-    mystrlcpy(buf[1], utype_values_string(ptype), column_size);
-    my_snprintf(buf[2], column_size, "(%d)", utype_build_shield_cost(ptype));
+    fc_strlcpy(buf[1], utype_values_string(ptype), column_size);
+    fc_snprintf(buf[2], column_size, "(%d)", utype_build_shield_cost(ptype));
     break;
   }
   case VUT_IMPROVEMENT:
@@ -324,13 +363,13 @@ void get_city_dialog_production_row(char *buf[], size_t column_size,
     /* Total & turns left meaningless on capitalization */
     if (improvement_has_flag(pimprove, IF_GOLD)) {
       buf[1][0] = '\0';
-      my_snprintf(buf[2], column_size, "---");
+      fc_snprintf(buf[2], column_size, "---");
     } else {
       int upkeep = pcity ? city_improvement_upkeep(pcity, pimprove)
                          : pimprove->upkeep;
       /* from city.c city_improvement_name_translation() */
-      if (pcity && is_building_replaced(pcity, pimprove, RPT_CERTAIN)) {
-	my_snprintf(buf[1], column_size, "%d*", upkeep);
+      if (pcity && is_improvement_redundant(pcity, pimprove)) {
+        fc_snprintf(buf[1], column_size, "%d*", upkeep);
       } else {
 	const char *state = NULL;
 
@@ -347,21 +386,21 @@ void get_city_dialog_production_row(char *buf[], size_t column_size,
 	} else if (is_small_wonder(pimprove)) {
 	  if (improvement_obsolete(pplayer, pimprove)) {
 	    state = _("Obsolete");
-	  } else if (find_city_from_small_wonder(pplayer, target.value.building)) {
+          } else if (city_from_small_wonder(pplayer, target.value.building)) {
 	    state = _("Built");
           } else {
             state = _("Small Wonder");
           }
-	}
+        }
         if (state) {
-          mystrlcpy(buf[1], state, column_size);
+          fc_strlcpy(buf[1], state, column_size);
         } else {
-          my_snprintf(buf[1], column_size, "%d", upkeep);
+          fc_snprintf(buf[1], column_size, "%d", upkeep);
         }
       }
 
-      my_snprintf(buf[2], column_size, "%d",
-		  impr_build_shield_cost(pimprove));
+      fc_snprintf(buf[2], column_size, "%d",
+                  impr_build_shield_cost(pimprove));
     }
     break;
   }
@@ -375,19 +414,19 @@ void get_city_dialog_production_row(char *buf[], size_t column_size,
   if (pcity) {
     if (VUT_IMPROVEMENT == target.kind
      && improvement_has_flag(target.value.building, IF_GOLD)) {
-      my_snprintf(buf[3], column_size, _("%d/turn"),
-		  MAX(0, pcity->surplus[O_SHIELD]));
+      fc_snprintf(buf[3], column_size, _("%d/turn"),
+                  MAX(0, pcity->surplus[O_SHIELD]));
     } else {
       int turns = city_turns_to_build(pcity, target, FALSE);
 
       if (turns < FC_INFINITY) {
-	my_snprintf(buf[3], column_size, "%d", turns);
+        fc_snprintf(buf[3], column_size, "%d", turns);
       } else {
-	my_snprintf(buf[3], column_size, _("never"));
+        fc_snprintf(buf[3], column_size, _("never"));
       }
     }
   } else {
-    my_snprintf(buf[3], column_size, "---");
+    fc_snprintf(buf[3], column_size, "---");
   }
 }
 
@@ -431,7 +470,7 @@ void get_city_dialog_output_text(const struct city *pcity,
          *
          * NB: (pcity->trade_value[i] == 0) is valid case.  The trade route
          * is established but doesn't give trade surplus. */
-        struct city *trade_city = game_find_city_by_number(pcity->trade[i]);
+        struct city *trade_city = game_city_by_number(pcity->trade[i]);
         /* TRANS: "unknown" location */
         const char *name = trade_city ? city_name(trade_city) : _("(unknown)");
 
@@ -472,7 +511,7 @@ void get_city_dialog_output_text(const struct city *pcity,
 		     peffect->value);
 	total = new_total;
       } effect_list_iterate_end;
-      effect_list_free(plist);
+      effect_list_destroy(plist);
     }
   }
 
@@ -528,11 +567,11 @@ void get_city_dialog_illness_text(const struct city *pcity,
   illness = city_illness_calc(pcity, &ill_base, &ill_size, &ill_trade,
                               &ill_pollution);
 
-  cat_snprintf(buf, bufsz, _("%+2.1f : Risk from overcrowding\n"),
+  cat_snprintf(buf, bufsz, _("%+5.1f : Risk from overcrowding\n"),
                ((float)(ill_size) / 10.0));
-  cat_snprintf(buf, bufsz, _("%+2.1f : Risk from trade\n"),
+  cat_snprintf(buf, bufsz, _("%+5.1f : Risk from trade\n"),
                ((float)(ill_trade) / 10.0));
-  cat_snprintf(buf, bufsz, _("%+2.1f : Risk from pollution\n"),
+  cat_snprintf(buf, bufsz, _("%+5.1f : Risk from pollution\n"),
                ((float)(ill_pollution) / 10.0));
 
   plist = effect_list_new();
@@ -545,13 +584,13 @@ void get_city_dialog_illness_text(const struct city *pcity,
     get_effect_req_text(peffect, buf2, sizeof(buf2));
 
     cat_snprintf(buf, bufsz,
-                 _("%+2.1f : Bonus from %s\n"),
+                 _("%+5.1f : Bonus from %s\n"),
                  -(0.1 * ill_base * peffect->value / 100), buf2);
   } effect_list_iterate_end;
-  effect_list_free(plist);
+  effect_list_destroy(plist);
 
   cat_snprintf(buf, bufsz, _("==== : Adds up to\n"));
-  cat_snprintf(buf, bufsz, _("%2.1f : Total chance for a plague"),
+  cat_snprintf(buf, bufsz, _("%5.1f : Total chance for a plague"),
                ((float)(illness) / 10.0));
 }
 
@@ -589,7 +628,7 @@ int get_city_citizen_types(struct city *pcity, enum citizen_feeling index,
 			   enum citizen_category *citizens)
 {
   int i = 0, n;
-  assert(index >= 0 && index < FEELING_LAST);
+  fc_assert(index >= 0 && index < FEELING_LAST);
 
   for (n = 0; n < pcity->feel[CITIZEN_HAPPY][index]; n++, i++) {
     citizens[i] = CITIZEN_HAPPY;
@@ -611,11 +650,9 @@ int get_city_citizen_types(struct city *pcity, enum citizen_feeling index,
   } specialist_type_iterate_end;
 
   if (pcity->size != i) {
-    freelog(LOG_ERROR, "get_city_citizen_types()"
-            " %d citizens not equal %d city size in \"%s\".",
-            i,
-            pcity->size,
-            city_name(pcity));
+    log_error("get_city_citizen_types() %d citizens "
+              "not equal %d city size in \"%s\".",
+              i, pcity->size, city_name(pcity));
   }
   return i;
 }
@@ -638,7 +675,7 @@ void city_rotate_specialist(struct city *pcity, int citizen_index)
   /* Loop through all specialists in order until we find a usable one
    * (or run out of choices). */
   to = from;
-  assert(to >= 0 && to < specialist_count());
+  fc_assert(to >= 0 && to < specialist_count());
   do {
     to = (to + 1) % specialist_count();
   } while (to != from && !city_can_use_specialist(pcity, to));
@@ -957,9 +994,17 @@ int city_change_specialist(struct city *pcity, Specialist_type_id from,
 **************************************************************************/
 int city_toggle_worker(struct city *pcity, int city_x, int city_y)
 {
-  struct tile *ptile = city_map_to_tile(city_tile(pcity), city_x, city_y);
+  int city_radius_sq;
+  struct tile *ptile;
 
-  assert(is_valid_city_coords(city_x, city_y));
+  if (city_owner(pcity) != client_player()) {
+    return 0;
+  }
+
+  city_radius_sq = city_map_radius_sq_get(pcity);
+  fc_assert(is_valid_city_coords(city_radius_sq, city_x, city_y));
+  ptile = city_map_to_tile(city_tile(pcity), city_radius_sq,
+                           city_x, city_y);
   if (NULL == ptile) {
     return 0;
   }
