@@ -19,13 +19,14 @@
 #include <windows.h>	/* LoadLibrary() */
 #endif
 
-#include <assert.h>
 #include <math.h>
+#include <signal.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <time.h>
 
 /* utility */
+#include "bitvector.h"
 #include "capstr.h"
 #include "dataio.h"
 #include "fciconv.h"
@@ -39,11 +40,13 @@
 /* common */
 #include "ai.h"
 #include "diptreaty.h"
+#include "fc_interface.h"
 #include "game.h"
 #include "idex.h"
 #include "map.h"
 #include "netintf.h"
 #include "packets.h"
+#include "player.h"
 #include "version.h"
 
 /* include */
@@ -84,10 +87,13 @@
 #include "packhand.h"
 #include "tilespec.h"
 #include "themes_common.h"
+#include "update_queue.h"
 #include "voteinfo.h"
 
 #include "client_main.h"
 
+static void fc_interface_init_client(void);
+
 char *logfile = NULL;
 char *scriptfile = NULL;
 static char tileset_name[512] = "\0";
@@ -177,6 +183,7 @@ static void at_exit(void)
 {
   client_kill_server(TRUE);
   fc_shutdown_network();
+  update_queue_free();
 }
 
 /**************************************************************************
@@ -193,7 +200,8 @@ static void client_game_init(void)
   control_init();
   link_marks_init();
   voteinfo_queue_init();
-  settable_options_init();
+  server_options_init();
+  update_queue_init();
 }
 
 /**************************************************************************
@@ -204,7 +212,7 @@ static void client_game_free(void)
   editgui_popdown_all();
 
   packhand_free();
-  settable_options_free();
+  server_options_free();
   voteinfo_queue_free();
   link_marks_free();
   control_free();
@@ -212,6 +220,10 @@ static void client_game_free(void)
   attribute_free();
   agents_free();
   game_free();
+  /* update_queue_init() is correct at this point. The queue is reset to
+     a clean state which is also needed if the client is not connected to
+     the server! */
+  update_queue_init();
 
   client.conn.playing = NULL;
   client.conn.observer = FALSE;
@@ -244,11 +256,13 @@ static void client_game_reset(void)
 **************************************************************************/
 int client_main(int argc, char *argv[])
 {
-  int i, loglevel;
+  int i;
+  enum log_level loglevel = LOG_NORMAL;
   int ui_options = 0;
   bool ui_separator = FALSE;
   char *option=NULL;
   bool user_tileset = FALSE;
+  int fatal_assertions = -1;
 
   /* Load win32 post-crash debugger */
 #ifdef WIN32_NATIVE
@@ -263,6 +277,8 @@ int client_main(int argc, char *argv[])
 
   i_am_client(); /* Tell to libfreeciv that we are client */
 
+  fc_interface_init_client();
+
   /* Ensure that all AIs are initialized to unused state */
   ai_type_iterate(ai) {
     init_ai(ai);
@@ -272,9 +288,6 @@ int client_main(int argc, char *argv[])
   audio_init();
   init_character_encodings(gui_character_encoding, gui_use_transliteration);
 
-  /* default argument values are set in options.c */
-  loglevel=LOG_NORMAL;
-
   i = 1;
 
   announce = ANNOUNCE_DEFAULT;
@@ -289,11 +302,16 @@ int client_main(int argc, char *argv[])
       fc_fprintf(stderr, _("  -A, --Announce PROTO\tAnnounce game in LAN using protocol PROTO (IPv4/IPv6/none)\n"));
       fc_fprintf(stderr, _("  -a, --autoconnect\tSkip connect dialog\n"));
 #ifdef DEBUG
-      fc_fprintf(stderr, _("  -d, --debug NUM\tSet debug log level (0 to 4,"
-			   " or 4:file1,min,max:...)\n"));
+      fc_fprintf(stderr, _("  -d, --debug NUM\tSet debug log level (%d to "
+                           "%d, or %d:file1,min,max:...)\n"),
+                 LOG_FATAL, LOG_DEBUG, LOG_DEBUG);
 #else
-      fc_fprintf(stderr,
-		 _("  -d, --debug NUM\tSet debug log level (0 to 3)\n"));
+      fc_fprintf(stderr, _("  -d, --debug NUM\tSet debug log level (%d to "
+                           "%d)\n"), LOG_FATAL, LOG_VERBOSE);
+#endif
+#ifndef NDEBUG
+      fc_fprintf(stderr, _("  -F, --Fatal [SIGNAL]\t"
+                           "Raise a signal on failed assertion\n"));
 #endif
       fc_fprintf(stderr,
 		 _("  -h, --help\t\tPrint a summary of the options\n"));
@@ -328,6 +346,19 @@ int client_main(int argc, char *argv[])
       exit(EXIT_SUCCESS);
     } else if ((option = get_option_malloc("--log", argv, &i, argc))) {
       logfile = option; /* never free()d */
+#ifndef NDEBUG
+    } else if (is_option("--Fatal", argv[i])) {
+      if (i + 1 >= argc || '-' == argv[i + 1][0]) {
+        fatal_assertions = SIGABRT;
+      } else if (str_to_int(argv[i + 1], &fatal_assertions)) {
+        i++;
+      } else {
+        fc_fprintf(stderr, _("Invalid signal number \"%s\".\n"),
+                   argv[i + 1]);
+        fc_fprintf(stderr, _("Try using --help.\n"));
+        exit(EXIT_FAILURE);
+      }
+#endif
     } else  if ((option = get_option_malloc("--read", argv, &i, argc))) {
       scriptfile = option; /* never free()d */
     } else if ((option = get_option_malloc("--name", argv, &i, argc))) {
@@ -343,11 +374,11 @@ int client_main(int argc, char *argv[])
       sz_strlcpy(sound_plugin_name, option);
       free(option);
     } else if ((option = get_option_malloc("--port",argv,&i,argc))) {
-      if (sscanf(option, "%d", &server_port) != 1) {
-	fc_fprintf(stderr,
-		   _("Invalid port \"%s\" specified with --port option.\n"),
-		   option);
-	fc_fprintf(stderr, _("Try using --help.\n"));
+      if (!str_to_int(option, &server_port)) {
+        fc_fprintf(stderr,
+                   _("Invalid port \"%s\" specified with --port option.\n"),
+                   option);
+        fc_fprintf(stderr, _("Try using --help.\n"));
         exit(EXIT_FAILURE);
       }
       free(option);
@@ -357,12 +388,11 @@ int client_main(int argc, char *argv[])
     } else if (is_option("--autoconnect", argv[i])) {
       auto_connect = TRUE;
     } else if ((option = get_option_malloc("--debug", argv, &i, argc))) {
-      loglevel = log_parse_level_str(option);
-      if (loglevel == -1) {
-	fc_fprintf(stderr,
-		   _("Invalid debug level \"%s\" specified with --debug "
-		     "option.\n"), option);
-	fc_fprintf(stderr, _("Try using --help.\n"));
+      if (!log_parse_level_str(option, &loglevel)) {
+        fc_fprintf(stderr,
+                   _("Invalid debug level \"%s\" specified with --debug "
+                     "option.\n"), option);
+        fc_fprintf(stderr, _("Try using --help.\n"));
         exit(EXIT_FAILURE);
       }
       free(option);
@@ -400,7 +430,7 @@ int client_main(int argc, char *argv[])
   /* disallow running as root -- too dangerous */
   dont_run_as_root(argv[0], "freeciv_client");
 
-  log_init(logfile, loglevel, NULL);
+  log_init(logfile, loglevel, NULL, NULL, fatal_assertions);
 
   /* after log_init: */
 
@@ -408,12 +438,12 @@ int client_main(int argc, char *argv[])
   if (!is_valid_username(default_user_name)) {
     char buf[sizeof(default_user_name)];
 
-    my_snprintf(buf, sizeof(buf), "_%s", default_user_name);
+    fc_snprintf(buf, sizeof(buf), "_%s", default_user_name);
     if (is_valid_username(buf)) {
       sz_strlcpy(default_user_name, buf);
     } else {
-      my_snprintf(default_user_name, sizeof(default_user_name),
-		  "player%d", myrand(10000));
+      fc_snprintf(default_user_name, sizeof(default_user_name),
+                  "player%d", fc_rand(10000));
     }
   }
 
@@ -425,12 +455,12 @@ int client_main(int argc, char *argv[])
   ui_init();
   charsets_init();
   fc_init_network();
+  update_queue_init();
 
   /* register exit handler */ 
   atexit(at_exit);
 
   init_our_capability();
-  chatline_common_init();
   init_player_dlg_common();
   init_themes();
 
@@ -450,21 +480,26 @@ int client_main(int argc, char *argv[])
     sz_strlcpy(user_name, default_user_name); 
   if (metaserver[0] == '\0') {
     /* FIXME: Find a cleaner way to achieve this. */
+    /* www.cazfi.net/freeciv/metaserver/ was default metaserver
+     * over one release when meta.freeciv.org was unavailable. */
     const char *oldaddr = "http://www.cazfi.net/freeciv/metaserver/";
     if (0 == strcmp(default_metaserver, oldaddr)) {
-      freelog(LOG_NORMAL, _("Updating old metaserver address \"%s\"."),
-              oldaddr);
-      sz_strlcpy(default_metaserver, META_URL);
-      freelog(LOG_NORMAL, _("Default metaserver has been set to \"%s\"."),
-              META_URL);
+      log_normal(_("Updating old metaserver address \"%s\"."), oldaddr);
+      sz_strlcpy(default_metaserver, DEFAULT_METASERVER_OPTION);
+      log_normal(_("Default metaserver has been set to value \"%s\"."),
+                 DEFAULT_METASERVER_OPTION);
+    }
+    if (0 == strcmp(default_metaserver, DEFAULT_METASERVER_OPTION)) {
+      sz_strlcpy(metaserver, META_URL);
+    } else {
+      sz_strlcpy(metaserver, default_metaserver);
     }
-    sz_strlcpy(metaserver, default_metaserver);
   }
   if (server_port == -1) server_port = default_server_port;
 
   /* This seed is not saved anywhere; randoms in the client should
      have cosmetic effects only (eg city name suggestions).  --dwp */
-  mysrand(time(NULL));
+  fc_srand(time(NULL));
   helpdata_init();
   boot_help_texts(NULL);
 
@@ -504,16 +539,15 @@ void client_exit(void)
   tileset_free(tileset);
   
   ui_exit();
-  
-  chatline_common_done();
+
   options_free();
   if (client_state() >= C_S_PREPARING) {
     client_game_free();
   }
 
   helpdata_done(); /* client_exit() unlinks help text list */
-  conn_list_free(game.all_connections);
-  conn_list_free(game.est_connections);
+  conn_list_destroy(game.all_connections);
+  conn_list_destroy(game.est_connections);
 
   free_nls();
 
@@ -526,9 +560,19 @@ void client_exit(void)
 **************************************************************************/
 void client_packet_input(void *packet, int type)
 {
-  if (!client_handle_packet(type, packet)) {
-    freelog(LOG_ERROR, "Received unknown packet (type %d) from server!",
-	    type);
+  if (!client.conn.established
+      && PACKET_CONN_PING != type
+      && PACKET_PROCESSING_STARTED != type
+      && PACKET_PROCESSING_FINISHED != type
+      && PACKET_SERVER_JOIN_REPLY != type
+      && PACKET_AUTHENTICATION_REQ != type
+      && PACKET_CONNECT_MSG != type) {
+    log_error("Received packet %s (%d) before establishing connection!",
+              packet_name(type), type);
+    disconnect_from_server();
+  } else if (!client_handle_packet(type, packet)) {
+    log_error("Received unknown packet (type %d) from server!", type);
+    disconnect_from_server();
   }
 }
 
@@ -545,8 +589,8 @@ void user_ended_turn(void)
 **************************************************************************/
 void send_turn_done(void)
 {
-  freelog(LOG_DEBUG, "send_turn_done() turn_done_button_state=%d",
-	  get_turn_done_button_state());
+  log_debug("send_turn_done() turn_done_button_state=%d",
+            get_turn_done_button_state());
 
   if (!get_turn_done_button_state()) {
     /*
@@ -588,8 +632,7 @@ void set_client_state(enum client_states newstate)
 
   if (auto_connect && newstate == C_S_DISCONNECTED) {
     if (oldstate == C_S_DISCONNECTED) {
-      freelog(LOG_FATAL,
-              _("There was an error while auto connecting; aborting."));
+      log_fatal(_("There was an error while auto connecting; aborting."));
         exit(EXIT_FAILURE);
     } else {
       start_autoconnecting_to_server();
@@ -600,7 +643,7 @@ void set_client_state(enum client_states newstate)
   if (C_S_PREPARING == newstate
       && (client_has_player() || client_is_observer())) {
     /* Reset the delta-state. */
-    conn_clear_packet_cache(&client.conn);
+    conn_reset_delta_state(&client.conn);
   }
 
   if (oldstate == newstate) {
@@ -611,14 +654,14 @@ void set_client_state(enum client_states newstate)
 
   switch (newstate) {
   case C_S_INITIAL:
-    die("%d is not a valid client state to set", C_S_INITIAL);
+    log_error("%d is not a valid client state to set.", C_S_INITIAL);
     break;
 
   case C_S_DISCONNECTED:
     popdown_all_city_dialogs();
     close_all_diplomacy_dialogs();
     popdown_all_game_dialogs();
-    clear_notify_window();
+    meswin_clear();
 
     if (oldstate > C_S_DISCONNECTED) {
       set_unit_focus(NULL);
@@ -640,7 +683,7 @@ void set_client_state(enum client_states newstate)
     popdown_all_city_dialogs();
     close_all_diplomacy_dialogs();
     popdown_all_game_dialogs();
-    clear_notify_window();
+    meswin_clear();
 
     if (oldstate < C_S_PREPARING) {
       client_game_init();
@@ -655,7 +698,7 @@ void set_client_state(enum client_states newstate)
 
     if (get_client_page() != PAGE_SCENARIO
         && get_client_page() != PAGE_LOAD) {
-      set_client_page(PAGE_START);
+    set_client_page(PAGE_START);
     }
     break;
 
@@ -735,9 +778,9 @@ void set_client_state(enum client_states newstate)
     break;
   }
 
-  update_menus();
+  menus_init();
   update_turn_done_button_state();
-  update_conn_list_dialog();
+  conn_list_dialog_update();
   if (can_client_change_view()) {
     update_map_canvas_visible();
   }
@@ -760,11 +803,11 @@ enum client_states client_state(void)
 void client_remove_cli_conn(struct connection *pconn)
 {
   if (NULL != pconn->playing) {
-    conn_list_unlink(pconn->playing->connections, pconn);
+    conn_list_remove(pconn->playing->connections, pconn);
   }
-  conn_list_unlink(game.all_connections, pconn);
-  conn_list_unlink(game.est_connections, pconn);
-  RETURN_IF_FAIL(pconn != &client.conn);
+  conn_list_remove(game.all_connections, pconn);
+  conn_list_remove(game.est_connections, pconn);
+  fc_assert_ret(pconn != &client.conn);
   free(pconn);
 }
 
@@ -1011,3 +1054,28 @@ struct player *client_player(void)
 {
   return client.conn.playing;
 }
+
+/****************************************************************************
+  Return the vision of the player on a tile. Client version of
+  ./server/maphand/map_is_known_and_seen().
+****************************************************************************/
+static bool client_map_is_known_and_seen(const struct tile *ptile,
+                                         const struct player *pplayer,
+                                         enum vision_layer vlayer)
+{
+  return dbv_isset(&pplayer->client.tile_vision[vlayer], tile_index(ptile));
+}
+
+/***************************************************************
+  Initialize client specific functions.
+***************************************************************/
+static void fc_interface_init_client(void)
+{
+  struct functions *funcs = fc_interface_funcs();
+
+  funcs->player_tile_vision_get = client_map_is_known_and_seen;
+
+  /* Keep this function call at the end. It checks if all required functions
+     are defined. */
+  fc_interface_init();
+}
