@@ -15,7 +15,6 @@
 #include <config.h>
 #endif
 
-#include <assert.h>
 #include <stdarg.h>
 #include <stdio.h>
 #include <stdlib.h>
@@ -24,17 +23,20 @@
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 
-/* common & utility */
+/* utility */
+#include "bitvector.h"
 #include "fcintl.h"
+#include "log.h"
+#include "mem.h"
+#include "rand.h"
+#include "support.h"
+
+/* common */
 #include "game.h"
 #include "government.h"
 #include "map.h"
-#include "log.h"
-#include "mem.h"
 #include "packets.h"
 #include "player.h"
-#include "rand.h"
-#include "support.h"
 #include "unitlist.h"
 
 /* client */
@@ -65,7 +67,6 @@ static GtkWidget  *races_shell;
 struct player *races_player;
 static GtkWidget  *races_nation_list[MAX_NUM_NATION_GROUPS + 1];
 static GtkWidget  *races_leader;
-static GList      *races_leader_list;
 static GtkWidget  *races_sex[2];
 static GtkWidget  *races_city_style_list;
 static GtkTextBuffer *races_text;
@@ -320,7 +321,7 @@ static void pillage_callback(GtkWidget *w, gpointer data)
   struct unit *punit;
   int what = GPOINTER_TO_INT(data);
 
-  punit = game_find_unit_by_number(unit_to_use_to_pillage);
+  punit = game_unit_by_number(unit_to_use_to_pillage);
   if (punit) {
     Base_type_id pillage_base = -1;
 
@@ -368,7 +369,7 @@ void popup_pillage_dialog(struct unit *punit,
       BV_CLR_ALL(what_base);
 
       if (what > S_LAST) {
-        BV_SET(what_base, what - S_LAST - 1);
+        BV_SET(what_base, what % (S_LAST + 1));
       } else {
         BV_SET(what_bv, what);
       }
@@ -377,7 +378,7 @@ void popup_pillage_dialog(struct unit *punit,
                         G_CALLBACK(pillage_callback), GINT_TO_POINTER(what));
 
       if (what > S_LAST) {
-        BV_CLR(bases, what - S_LAST - 1);
+        BV_CLR(bases, what % (S_LAST + 1));
       } else {
         clear_special(&may_pillage, what);
       }
@@ -404,7 +405,7 @@ static void unit_select_row_activated(GtkTreeView *view, GtkTreePath *path)
   gtk_tree_model_get_iter(GTK_TREE_MODEL(unit_select_store), &it, path);
   gtk_tree_model_get(GTK_TREE_MODEL(unit_select_store), &it, 0, &id, -1);
  
-  if ((punit = player_find_unit_by_id(client.conn.playing, id))) {
+  if ((punit = player_unit_by_number(client_player(), id))) {
     set_unit_focus(punit);
   }
 
@@ -508,12 +509,12 @@ static void unit_select_cmd_callback(GtkWidget *w, gint rid, gpointer data)
           pmyunit = punit;
 
           /* Activate this unit. */
-	  punit->focus_status = FOCUS_AVAIL;
+	  punit->client.focus_status = FOCUS_AVAIL;
 	  if (unit_has_orders(punit)) {
 	    request_orders_cleared(punit);
 	  }
-	  if (punit->activity != ACTIVITY_IDLE || punit->ai.control) {
-	    punit->ai.control = FALSE;
+	  if (punit->activity != ACTIVITY_IDLE || punit->ai_controlled) {
+	    punit->ai_controlled = FALSE;
 	    request_new_unit_activity(punit, ACTIVITY_IDLE);
 	  }
         }
@@ -531,7 +532,7 @@ static void unit_select_cmd_callback(GtkWidget *w, gint rid, gpointer data)
       unit_list_iterate(ptile->units, punit) {
         if (unit_owner(punit) == client.conn.playing) {
           if ((punit->activity == ACTIVITY_IDLE) &&
-              !punit->ai.control &&
+              !punit->ai_controlled &&
               can_unit_do_activity(punit, ACTIVITY_SENTRY)) {
             request_new_unit_activity(punit, ACTIVITY_SENTRY);
           }
@@ -545,7 +546,7 @@ static void unit_select_cmd_callback(GtkWidget *w, gint rid, gpointer data)
       unit_list_iterate(ptile->units, punit) {
         if (unit_owner(punit) == client.conn.playing) {
           if (punit->activity == ACTIVITY_IDLE &&
-              !punit->ai.control) {
+              !punit->ai_controlled) {
             /* Give focus to it */
             add_unit_focus(punit);
           }
@@ -750,7 +751,7 @@ static GtkWidget* create_list_of_nations_in_group(struct nation_group* group,
       continue;
     }
 
-    if (group != NULL && !is_nation_in_group(pnation, group)) {
+    if (NULL != group && !nation_is_in_group(pnation, group)) {
       continue;
     }
 
@@ -805,12 +806,16 @@ static GtkWidget* create_nation_selection_list(void)
   gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);  
   gtk_box_pack_start(GTK_BOX(vbox), notebook, TRUE, TRUE, 0);
   
-  for (i = 0; i <= nation_group_count(); i++) {
-    struct nation_group* group = (i == 0 ? NULL: nation_group_by_number(i - 1));
+  for (i = 1; i <= nation_group_count(); i++) {
+    struct nation_group* group = (nation_group_by_number(i - 1));
     nation_list = create_list_of_nations_in_group(group, i);
-    group_name_label = gtk_label_new(group ? Q_(group->name) : _("All"));
+    group_name_label = gtk_label_new(nation_group_name_translation(group));
     gtk_notebook_append_page(GTK_NOTEBOOK(notebook), nation_list, group_name_label);
   }
+  
+  nation_list = create_list_of_nations_in_group(NULL, 0);
+  group_name_label = gtk_label_new(_("All"));
+  gtk_notebook_append_page(GTK_NOTEBOOK(notebook), nation_list, group_name_label);
 
   return vbox;
 }
@@ -895,13 +900,15 @@ static void create_races_dialog(struct player *pplayer)
   gtk_box_pack_start(GTK_BOX(vbox), table, TRUE, TRUE, 0);
 
   /* Leader. */ 
-  combo = gtk_combo_new();
+  {
+    GtkListStore *model = gtk_list_store_new(1, G_TYPE_STRING);
+    combo = gtk_combo_box_entry_new_with_model(GTK_TREE_MODEL(model), 0);
+    g_object_unref(G_OBJECT(model));
+  }
   races_leader = combo;
-  gtk_combo_disable_activate(GTK_COMBO(combo));
-  gtk_combo_set_value_in_list(GTK_COMBO(combo), FALSE, FALSE);
   label = g_object_new(GTK_TYPE_LABEL,
       "use-underline", TRUE,
-      "mnemonic-widget", GTK_COMBO(combo)->entry,
+      "mnemonic-widget", GTK_COMBO_BOX(combo),
       "label", _("_Leader:"),
       "xalign", 0.0,
       "yalign", 0.5,
@@ -1002,7 +1009,7 @@ static void create_races_dialog(struct player *pplayer)
   g_signal_connect(shell, "response",
       G_CALLBACK(races_response), NULL);
 
-  g_signal_connect(GTK_COMBO(races_leader)->list, "select_child",
+  g_signal_connect(GTK_COMBO_BOX(races_leader), "changed",
       G_CALLBACK(races_leader_callback), NULL);
 
   g_signal_connect(races_sex[0], "toggled",
@@ -1063,67 +1070,33 @@ void popdown_races_dialog(void)
  *****************************************************************/
 static void races_destroy_callback(GtkWidget *w, gpointer data)
 {
-  g_list_free(races_leader_list);
-  races_leader_list = NULL;
-
   races_shell = NULL;
 }
 
 /****************************************************************
-  ...
- *****************************************************************/
-static gint cmp_func(gconstpointer ap, gconstpointer bp)
+  Populates leader list.
+*****************************************************************/
+static void populate_leader_list(void)
 {
-  return strcmp((const char *)ap, (const char *)bp);
-}
+  int i;
+  int idx;
+  GtkListStore *model =
+      GTK_LIST_STORE(gtk_combo_box_get_model(GTK_COMBO_BOX(races_leader)));
 
-/****************************************************************
-  Selects a leader.
-  Updates the gui elements.
- *****************************************************************/
-static void select_random_leader(void)
-{
-  struct nation_leader *leaders;
-  int i, nleaders;
-  GList *items = NULL;
-  GtkWidget *text, *list;
-  gchar *name;
-  bool unique;
+  i = 0;
+  gtk_list_store_clear(model);
+  nation_leader_list_iterate(nation_leaders(nation_by_number
+                                            (selected_nation)), pleader) {
+    const char *leader_name = nation_leader_name(pleader);
+    GtkTreeIter iter; /* unused */
 
-  text = GTK_COMBO(races_leader)->entry;
-  list = GTK_COMBO(races_leader)->list;
-  name = gtk_editable_get_chars(GTK_EDITABLE(text), 0, -1);
+    gtk_list_store_insert_with_values(model, &iter, i, 0, leader_name, -1);
+    i++;
+  } nation_leader_list_iterate_end;
 
-  if (name[0] == '\0'
-      || g_list_find_custom(races_leader_list, name, cmp_func)) {
-    unique = FALSE;
-  } else {
-    unique = TRUE;
-  }
+  idx = fc_rand(i);
 
-  leaders
-    = get_nation_leaders(nation_by_number(selected_nation), &nleaders);
-  for (i = 0; i < nleaders; i++) {
-    items = g_list_prepend(items, leaders[i].name);
-  }
-
-  /* Populate combo box with minimum signal noise. */
-  g_signal_handlers_block_by_func(list, races_leader_callback, NULL);
-  gtk_combo_set_popdown_strings(GTK_COMBO(races_leader), items);
-  gtk_entry_set_text(GTK_ENTRY(text), "");
-  g_signal_handlers_unblock_by_func(list, races_leader_callback, NULL);
-
-  g_list_free(races_leader_list);
-  races_leader_list = items;
-
-  if (unique) {
-    gtk_entry_set_text(GTK_ENTRY(text), name);
-  } else {
-    i = myrand(nleaders);
-    gtk_entry_set_text(GTK_ENTRY(text), g_list_nth_data(items, i));
-  }
-
-  g_free(name);
+  gtk_combo_box_set_active(GTK_COMBO_BOX(races_leader), idx);
 }
 
 /**************************************************************************
@@ -1219,9 +1192,9 @@ static void races_nation_callback(GtkTreeSelection *select, gpointer data)
         }
       }
       selected_nation = selected_nation_copy;
-      
-      select_random_leader();
-      
+
+      populate_leader_list();
+
       /* Select city style for chosen nation. */
       cs = city_style_of_nation(nation_by_number(selected_nation));
       for (i = 0, j = 0; i < game.control.styles_count; i++) {
@@ -1259,15 +1232,18 @@ static void races_nation_callback(GtkTreeSelection *select, gpointer data)
 **************************************************************************/
 static void races_leader_callback(void)
 {
+  const struct nation_leader *pleader;
   const gchar *name;
 
-  name = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(races_leader)->entry));
+  name =
+    gtk_entry_get_text(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(races_leader))));
 
-  if (check_nation_leader_name(nation_by_number(selected_nation), name)) {
-    selected_sex = get_nation_leader_sex(nation_by_number(selected_nation),
-					 name);
+  if (selected_nation != -1
+      &&(pleader = nation_leader_by_name(nation_by_number(selected_nation),
+                                         name))) {
+    selected_sex = nation_leader_is_male(pleader);
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(races_sex[selected_sex]),
-				 TRUE);
+                                 TRUE);
   }
 }
 
@@ -1332,7 +1308,7 @@ static void races_response(GtkWidget *w, gint response, gpointer data)
       return;
     }
 
-    s = gtk_entry_get_text(GTK_ENTRY(GTK_COMBO(races_leader)->entry));
+    s = gtk_entry_get_text(GTK_ENTRY(gtk_bin_get_child(GTK_BIN(races_leader))));
 
     /* Perform a minimum of sanity test on the name. */
     /* This could call is_allowed_player_name if it were available. */
