@@ -29,6 +29,7 @@
 /* common */
 #include "diptreaty.h"
 #include "fcintl.h"
+#include "game.h"
 #include "government.h"
 #include "map.h"
 #include "packets.h"
@@ -178,7 +179,7 @@ static void popup_diplomacy_dialog(int other_player_id, int initiated_from)
     return;
   }
 
-  if (client.conn.playing->ai_data.control) {
+  if (client.conn.playing->ai_controlled) {
     return;			/* Don't show if we are AI controlled. */
   }
 
@@ -203,11 +204,11 @@ static void popup_diplomacy_dialog(int other_player_id, int initiated_from)
 ****************************************************************************/
 static gint sort_advance_names(gconstpointer a, gconstpointer b)
 {
-  struct advance *padvance1 = (struct advance *) a;
-  struct advance *padvance2 = (struct advance *) b;
+  const struct advance *padvance1 = (const struct advance *) a;
+  const struct advance *padvance2 = (const struct advance *) b;
 
-  return compare_strings(advance_name_translation(padvance1),
-                         advance_name_translation(padvance2));
+  return fc_strcoll(advance_name_translation(padvance1),
+                    advance_name_translation(padvance2));
 }
 
 /****************************************************************
@@ -249,9 +250,8 @@ static void popup_add_menu(GtkMenuShell *parent, gpointer data)
   gtk_menu_shell_append(GTK_MENU_SHELL(parent), item);
   gtk_widget_show_all(item);
 
-
-  /* Advances. */
-  {
+  /* Trading: advances */
+  if (game.info.trading_tech) {
     GtkWidget *advance_item;
     GList *sorting_list = NULL;
 
@@ -262,7 +262,7 @@ static void popup_add_menu(GtkMenuShell *parent, gpointer data)
       Tech_type_id i = advance_number(padvance);
 
       if (player_invention_state(pgiver, i) == TECH_KNOWN
-          && player_invention_reachable(pother, i)
+          && player_invention_reachable(pother, i, FALSE)
           && (player_invention_state(pother, i) == TECH_UNKNOWN
               || player_invention_state(pother, i) == TECH_PREREQS_KNOWN)) {
         sorting_list = g_list_prepend(sorting_list, padvance);
@@ -274,7 +274,7 @@ static void popup_add_menu(GtkMenuShell *parent, gpointer data)
       gtk_widget_set_sensitive(advance_item, FALSE);
     } else {
       GList *list_item;
-      struct advance *padvance;
+      const struct advance *padvance;
 
       sorting_list = g_list_sort(sorting_list, sort_advance_names);
       menu = gtk_menu_new();
@@ -294,7 +294,7 @@ static void popup_add_menu(GtkMenuShell *parent, gpointer data)
 
       for (list_item = sorting_list; NULL != list_item;
            list_item = g_list_next(list_item)) {
-        padvance = (struct advance *) list_item->data;
+        padvance = (const struct advance *) list_item->data;
         item =
             gtk_menu_item_new_with_label(advance_name_translation(padvance));
         gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
@@ -315,7 +315,7 @@ static void popup_add_menu(GtkMenuShell *parent, gpointer data)
   }
 
 
-  /* Cities. */
+  /* Trading: cities. */
 
   /****************************************************************
   Creates a sorted list of plr0's cities, excluding the capital and
@@ -324,7 +324,7 @@ static void popup_add_menu(GtkMenuShell *parent, gpointer data)
 
 			      - Kris Bubendorfer
   *****************************************************************/
-  {
+  if (game.info.trading_city) {
     int i = 0, j = 0, n = city_list_size(pgiver->cities);
     struct city **city_list_ptrs;
 
@@ -396,7 +396,7 @@ static void popup_add_menu(GtkMenuShell *parent, gpointer data)
   if (pgiver == pdialog->treaty.plr0) {
     enum diplstate_type ds;
 
-    ds = pplayer_get_diplstate(pgiver, pother)->type;
+    ds = player_diplstate_get(pgiver, pother)->type;
 
     menu = gtk_menu_new();
     item = gtk_menu_item_new_with_mnemonic(Q_("?diplomatic_state:Cease-fire"));
@@ -461,7 +461,7 @@ static void diplomacy_destroy(struct Diplomacy_dialog* pdialog)
      * by an other way. */
     gui_dialog_destroy(pdialog->dialog);
   }
-  dialog_list_unlink(dialog_list, pdialog);
+  dialog_list_remove(dialog_list, pdialog);
   free(pdialog);
 }
 
@@ -501,7 +501,7 @@ static struct Diplomacy_dialog *create_diplomacy_dialog(struct player *plr0,
   GtkCellRenderer *rend;
 
   struct Diplomacy_dialog *pdialog;
-  char buf[256];
+  char buf[256], plr_buf[4 * MAX_LEN_NAME];
 
   pdialog = fc_malloc(sizeof(*pdialog));
 
@@ -510,9 +510,8 @@ static struct Diplomacy_dialog *create_diplomacy_dialog(struct player *plr0,
 
   gui_dialog_new(&(pdialog->dialog), GTK_NOTEBOOK(top_notebook), pdialog);
   
-  my_snprintf(buf, sizeof(buf),
-              _("Diplomacy: %s"),
-	      nation_plural_for_player(plr1));
+  fc_snprintf(buf, sizeof(buf), _("Diplomacy: %s"),
+              nation_plural_for_player(plr1));
 
   gui_dialog_set_title(pdialog->dialog, buf);
   gui_dialog_response_set_callback(pdialog->dialog, diplomacy_response);
@@ -570,8 +569,7 @@ static struct Diplomacy_dialog *create_diplomacy_dialog(struct player *plr0,
 
   label = gtk_label_new(NULL);
   gtk_misc_set_alignment(GTK_MISC(label), 0.5, 0.5);
-  my_snprintf(buf, sizeof(buf),
-	      "<span size=\"large\"><u>%s</u></span>",
+  fc_snprintf(buf, sizeof(buf), "<span size=\"large\"><u>%s</u></span>",
               nation_plural_for_player(plr0));
   gtk_label_set_markup(GTK_LABEL(label), buf);
   gtk_box_pack_start(GTK_BOX(vbox), label, TRUE, TRUE, 0);
@@ -588,10 +586,9 @@ static struct Diplomacy_dialog *create_diplomacy_dialog(struct player *plr0,
 
   label = gtk_label_new(NULL);
   gtk_misc_set_alignment(GTK_MISC(label), 0.5, 0.5);
-  my_snprintf(buf, sizeof(buf),
-	      "<span size=\"large\" weight=\"bold\">%s %s</span>",
-	      ruler_title_translation(plr0),
-	      player_name(plr0));
+  fc_snprintf(buf, sizeof(buf),
+              "<span size=\"large\" weight=\"bold\">%s</span>",
+              ruler_title_for_player(plr0, plr_buf, sizeof(plr_buf)));
   gtk_label_set_markup(GTK_LABEL(label), buf);
   gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
 
@@ -603,21 +600,21 @@ static struct Diplomacy_dialog *create_diplomacy_dialog(struct player *plr0,
   gtk_table_set_row_spacings(GTK_TABLE(table), 6);
   gtk_box_pack_start(GTK_BOX(vbox), table, TRUE, TRUE, 0);
 
-  spin = gtk_spin_button_new_with_range(0.0, plr0->economic.gold + 0.1, 1.0);
-  gtk_spin_button_set_digits(GTK_SPIN_BUTTON(spin), 0);
-  gtk_table_attach_defaults(GTK_TABLE(table), spin, 1, 2, 0, 1);
-  g_object_set_data(G_OBJECT(spin), "plr", plr0);
-  g_signal_connect_after(spin, "value-changed",
-			 G_CALLBACK(diplo_dialog_returnkey), pdialog);
+  /* Trading gold: we */
+  if (game.info.trading_gold) {
+    spin = gtk_spin_button_new_with_range(0.0, plr0->economic.gold + 0.1,
+                                          1.0);
+    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(spin), 0);
+    gtk_table_attach_defaults(GTK_TABLE(table), spin, 1, 2, 0, 1);
+    g_object_set_data(G_OBJECT(spin), "plr", plr0);
+    g_signal_connect_after(spin, "value-changed",
+                           G_CALLBACK(diplo_dialog_returnkey), pdialog);
 
-  label = g_object_new(GTK_TYPE_LABEL,
-    "use-underline", TRUE,
-    "mnemonic-widget", spin,
-    "label", _("_Gold:"),
-    "xalign", 0.0,
-    "yalign", 0.5,
-    NULL);
-  gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 0, 1);
+    label = g_object_new(GTK_TYPE_LABEL, "use-underline", TRUE,
+                         "mnemonic-widget", spin, "label", _("_Gold:"),
+                         "xalign", 0.0, "yalign", 0.5, NULL);
+    gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 0, 1);
+  }
 
   menubar = gtk_menu_bar_new();
   gtk_table_attach_defaults(GTK_TABLE(table), menubar, 1, 2, 1, 2);
@@ -643,8 +640,7 @@ static struct Diplomacy_dialog *create_diplomacy_dialog(struct player *plr0,
 
   label = gtk_label_new(NULL);
   gtk_misc_set_alignment(GTK_MISC(label), 0.5, 0.5);
-  my_snprintf(buf, sizeof(buf),
-	      "<span size=\"large\"><u>%s</u></span>",
+  fc_snprintf(buf, sizeof(buf), "<span size=\"large\"><u>%s</u></span>",
               nation_plural_for_player(plr1));
   gtk_label_set_markup(GTK_LABEL(label), buf);
   gtk_box_pack_start(GTK_BOX(vbox), label, TRUE, TRUE, 0);
@@ -662,10 +658,9 @@ static struct Diplomacy_dialog *create_diplomacy_dialog(struct player *plr0,
 
   label = gtk_label_new(NULL);
   gtk_misc_set_alignment(GTK_MISC(label), 0.5, 0.5);
-  my_snprintf(buf, sizeof(buf),
-	      "<span size=\"large\" weight=\"bold\">%s %s</span>",
-	      ruler_title_translation(plr1),
-	      player_name(plr1));
+  fc_snprintf(buf, sizeof(buf),
+              "<span size=\"large\" weight=\"bold\">%s</span>",
+              ruler_title_for_player(plr1, plr_buf, sizeof(plr_buf)));
   gtk_label_set_markup(GTK_LABEL(label), buf);
   gtk_box_pack_start(GTK_BOX(hbox), label, TRUE, TRUE, 0);
 
@@ -677,21 +672,20 @@ static struct Diplomacy_dialog *create_diplomacy_dialog(struct player *plr0,
   gtk_table_set_row_spacings(GTK_TABLE(table), 6);
   gtk_box_pack_start(GTK_BOX(vbox), table, TRUE, TRUE, 0);
 
-  spin = gtk_spin_button_new_with_range(0.0, plr1->economic.gold + 0.1, 1.0);
-  gtk_spin_button_set_digits(GTK_SPIN_BUTTON(spin), 0);
-  gtk_table_attach_defaults(GTK_TABLE(table), spin, 1, 2, 0, 1);
-  g_object_set_data(G_OBJECT(spin), "plr", plr1);
-  g_signal_connect_after(spin, "value-changed",
-			 G_CALLBACK(diplo_dialog_returnkey), pdialog);
+  /* Trading gold: they */
+  if (game.info.trading_gold) {
+    spin = gtk_spin_button_new_with_range(0.0, plr1->economic.gold + 0.1, 1.0);
+    gtk_spin_button_set_digits(GTK_SPIN_BUTTON(spin), 0);
+    gtk_table_attach_defaults(GTK_TABLE(table), spin, 1, 2, 0, 1);
+    g_object_set_data(G_OBJECT(spin), "plr", plr1);
+    g_signal_connect_after(spin, "value-changed",
+                           G_CALLBACK(diplo_dialog_returnkey), pdialog);
 
-  label = g_object_new(GTK_TYPE_LABEL,
-    "use-underline", TRUE,
-    "mnemonic-widget", spin,
-    "label", _("_Gold:"),
-    "xalign", 0.0,
-    "yalign", 0.5,
-    NULL);
-  gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 0, 1);
+    label = g_object_new(GTK_TYPE_LABEL, "use-underline", TRUE,
+                         "mnemonic-widget", spin, "label", _("_Gold:"),
+                         "xalign", 0.0, "yalign", 0.5, NULL);
+    gtk_table_attach_defaults(GTK_TABLE(table), label, 0, 1, 0, 1);
+  }
 
   menubar = gtk_menu_bar_new();
   gtk_table_attach_defaults(GTK_TABLE(table), menubar, 1, 2, 1, 2);
@@ -775,14 +769,14 @@ static void diplomacy_dialog_tech_callback(GtkWidget *w, gpointer data)
     struct player *pgiver = player_by_number(giver);
     struct player *pdest = player_by_number(dest);
 
-    RETURN_IF_FAIL(NULL != pgiver);
-    RETURN_IF_FAIL(NULL != pdest);
+    fc_assert_ret(NULL != pgiver);
+    fc_assert_ret(NULL != pdest);
 
     advance_iterate(A_FIRST, padvance) {
       Tech_type_id i = advance_number(padvance);
 
       if (player_invention_state(pgiver, i) == TECH_KNOWN
-          && player_invention_reachable(pdest, i)
+          && player_invention_reachable(pdest, i, FALSE)
           && (player_invention_state(pdest, i) == TECH_UNKNOWN
               || player_invention_state(pdest, i) == TECH_PREREQS_KNOWN)) {
         dsend_packet_diplomacy_create_clause_req(&client.conn, other, giver,
@@ -937,7 +931,7 @@ void diplomacy_dialog_init()
 *****************************************************************/
 void diplomacy_dialog_done()
 {
-  dialog_list_free(dialog_list);
+  dialog_list_destroy(dialog_list);
 }
 
 /*****************************************************************
