@@ -15,7 +15,6 @@
 #include <config.h>
 #endif
 
-#include <assert.h>
 #include <stdio.h>
 
 #ifdef HAVE_UNISTD_H
@@ -275,12 +274,15 @@ void set_indicator_icons(struct sprite *bulb, struct sprite *sol,
 }
 
 /****************************************************************************
-  Return the dimensions of the area (container widget; maximum size) for
-  the overview.
+  Return the maximum dimensions of the area (container widget) for the
+  overview. Due to the fact that the scaling factor is at least 1, the real
+  size could be larger. The calculation in calculate_overview_dimensions()
+  limit it to the smallest possible size.
 ****************************************************************************/
 void get_overview_area_dimensions(int *width, int *height)
 {
-  gdk_drawable_get_size(overview_canvas->window, width, height);
+  *width = GUI_GTK_OVERVIEW_MIN_XSIZE;
+  *height = GUI_GTK_OVERVIEW_MIN_YSIZE;
 }
 
 /**************************************************************************
@@ -340,7 +342,7 @@ void mapview_thaw(void)
   if (1 < mapview_frozen_level) {
     mapview_frozen_level--;
   } else {
-    assert(0 < mapview_frozen_level);
+    fc_assert(0 < mapview_frozen_level);
     mapview_frozen_level = 0;
     dirty_all();
   }
@@ -610,8 +612,8 @@ static void pixmap_put_sprite(GdkDrawable *pixmap,
 #ifdef DEBUG
   sprites++;
   if (sprites % 1000 == 0) {
-    freelog(LOG_DEBUG, "%5d / %5d pixbufs = %d%%",
-	    pixbufs, sprites, 100 * pixbufs / sprites);
+    log_debug("%5d / %5d pixbufs = %d%%",
+              pixbufs, sprites, 100 * pixbufs / sprites);
   }
 #endif
 }
@@ -675,9 +677,8 @@ void pixmap_put_overlay_tile_draw(GdkDrawable *pixmap,
 	  || (!ssprite->pixmap && !ssprite->pixbuf_fogged))) {
     fog_sprite(ssprite);
     if ((ssprite->pixmap && !ssprite->pixmap_fogged)
-	|| (!ssprite->pixmap && !ssprite->pixbuf_fogged)) {
-      freelog(LOG_NORMAL,
-	      _("Better fog will only work in truecolor.  Disabling it"));
+        || (!ssprite->pixmap && !ssprite->pixbuf_fogged)) {
+      log_normal(_("Better fog will only work in truecolor. Disabling it"));
       gui_gtk2_better_fog = FALSE;
     }
   }
@@ -737,6 +738,28 @@ void put_cross_overlay_tile(struct tile *ptile)
   }
 }
 
+/*****************************************************************************
+ Sets the position of the overview scroll window based on mapview position.
+*****************************************************************************/
+void update_overview_scroll_window_pos(int x, int y)
+{
+  gdouble ov_scroll_x, ov_scroll_y;
+  GtkAdjustment *ov_hadj, *ov_vadj;
+
+  ov_hadj = gtk_scrolled_window_get_hadjustment(
+    GTK_SCROLLED_WINDOW(overview_scrolled_window));
+  ov_vadj = gtk_scrolled_window_get_vadjustment(
+    GTK_SCROLLED_WINDOW(overview_scrolled_window));
+
+  ov_scroll_x = MIN(x - (overview_canvas_store_width / 2),
+                    ov_hadj->upper -ov_hadj->page_size);
+  ov_scroll_y = MIN(y - (overview_canvas_store_height / 2),
+                    ov_vadj->upper -ov_vadj->page_size);
+
+  gtk_adjustment_set_value(GTK_ADJUSTMENT(ov_hadj), ov_scroll_x);
+  gtk_adjustment_set_value(GTK_ADJUSTMENT(ov_vadj), ov_scroll_y);
+}
+
 /**************************************************************************
 ...
 **************************************************************************/
@@ -846,7 +869,7 @@ void draw_selection_rectangle(int canvas_x, int canvas_y, int w, int h)
 **************************************************************************/
 void tileset_changed(void)
 {
-  science_dialog_redraw();
+  science_report_dialog_redraw();
   reset_city_dialogs();
   reset_unit_table();
   blank_max_unit_size();
