@@ -15,7 +15,6 @@
 #include <config.h>
 #endif
 
-#include <assert.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
@@ -23,19 +22,20 @@
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 
+/* common & utility */
 #include "diptreaty.h"
 #include "fcintl.h"
-#include "game.h"
 #include "packets.h"
 #include "nation.h"
 #include "player.h"
 #include "support.h"
 
+/* client */
 #include "chatline.h"
-#include "civclient.h"
+#include "client_main.h"
 #include "climisc.h"
-#include "clinet.h"
 #include "connectdlg_common.h"
+#include "dialogs.h"
 #include "gui_main.h"
 #include "gui_stuff.h"
 #include "inteldlg.h"
@@ -56,10 +56,11 @@ static GtkWidget *players_meet_command;
 static GtkWidget *players_war_command;
 static GtkWidget *players_vision_command;
 static GtkWidget *players_sship_command;
-static GtkListStore *store;
-static GtkTreeModel *model;
 
-static gint ncolumns;
+static GtkListStore *players_dialog_store;
+#define PLR_DLG_COL_STYLE       (0 + num_player_dlg_columns)
+#define PLR_DLG_COL_WEIGHT      (1 + num_player_dlg_columns)
+#define PLR_DLG_COL_ID          (2 + num_player_dlg_columns)
 
 static void create_players_dialog(void);
 static void players_meet_callback(GtkMenuItem *item, gpointer data);
@@ -100,7 +101,7 @@ void popdown_players_dialog(void)
 /**************************************************************************
 ...
 **************************************************************************/
-static GdkPixbuf *create_player_icon(struct player *plr)
+static GdkPixbuf *create_player_icon(const struct player *plr)
 {
   int width, height;
   GdkPixbuf *tmp;
@@ -137,18 +138,18 @@ static void update_players_menu(void)
     struct player *plr;
     gint plrno;
 
-    gtk_tree_model_get(model, &it, ncolumns - 1, &plrno, -1);
+    gtk_tree_model_get(model, &it, PLR_DLG_COL_ID, &plrno, -1);
     plr = player_by_number(plrno);
-  
+
     if (plr->spaceship.state != SSHIP_NONE) {
       gtk_widget_set_sensitive(players_sship_command, TRUE);
     } else {
       gtk_widget_set_sensitive(players_sship_command, FALSE);
     }
 
-    if (game.player_ptr) {
-      switch (pplayer_get_diplstate(game.player_ptr,
-				    player_by_number(plrno))->type) {
+    if (NULL != client.conn.playing) {
+      switch (player_diplstate_get(client.conn.playing,
+                                   player_by_number(plrno))->type) {
       case DS_WAR:
       case DS_NO_CONTACT:
 	gtk_widget_set_sensitive(players_war_command, FALSE);
@@ -156,7 +157,7 @@ static void update_players_menu(void)
       default:
 	gtk_widget_set_sensitive(players_war_command,
 				 can_client_issue_orders()
-				 && game.info.player_idx != plrno);
+				 && player_by_number(plrno) != client.conn.playing);
       }
     } else {
       gtk_widget_set_sensitive(players_war_command, FALSE);
@@ -164,7 +165,7 @@ static void update_players_menu(void)
 
     gtk_widget_set_sensitive(players_vision_command,
 			     can_client_issue_orders()
-			     && gives_shared_vision(game.player_ptr, plr));
+			     && gives_shared_vision(client.conn.playing, plr));
 
     gtk_widget_set_sensitive(players_meet_command, can_meet_with_player(plr));
     gtk_widget_set_sensitive(players_int_command, can_intel_with_player(plr));
@@ -193,23 +194,23 @@ static gboolean button_press_callback(GtkTreeView *view, GdkEventButton *ev)
 
     gtk_tree_view_get_cursor(view, &path, NULL);
     if (path) {
+      GtkTreeModel *model = gtk_tree_view_get_model(view);
       GtkTreeIter it;
       gint id;
       struct player *plr;
 
-      gtk_tree_model_get_iter(GTK_TREE_MODEL(store), &it, path);
+      gtk_tree_model_get_iter(model, &it, path);
       gtk_tree_path_free(path);
 
-      gtk_tree_model_get(GTK_TREE_MODEL(store), &it,
-	  ncolumns - 1, &id, -1);
+      gtk_tree_model_get(model, &it, PLR_DLG_COL_ID, &id, -1);
       plr = player_by_number(id);
 
       if (ev->button == 1) {
-	if (can_intel_with_player(plr)) {
-	  popup_intel_dialog(plr);
-	}
-      } else {
-	dsend_packet_diplomacy_init_meeting_req(&aconnection, id);
+        if (can_intel_with_player(plr)) {
+          popup_intel_dialog(plr);
+        }
+      } else if (can_meet_with_player(plr)) {
+        dsend_packet_diplomacy_init_meeting_req(&client.conn, id);
       }
     }
   }
@@ -230,21 +231,22 @@ static gint plrdlg_sort_func(GtkTreeModel *model,
   n = GPOINTER_TO_INT(data);
 
   gtk_tree_model_get_value(model, a, num_player_dlg_columns + 2, &value);
-  player1 = &game.players[g_value_get_int(&value)];
+  player1 = player_by_number(g_value_get_int(&value));
   g_value_unset(&value);
   
   gtk_tree_model_get_value(model, b, num_player_dlg_columns + 2, &value);
-  player2 = &game.players[g_value_get_int(&value)];
+  player2 = player_by_number(g_value_get_int(&value));
   g_value_unset(&value);
   
   return player_dlg_columns[n].sort_func(player1, player2);
 }
 
-/**************************************************************************
-...
-**************************************************************************/
-static void create_store(void)
+/****************************************************************************
+  Create a player dialog store.
+****************************************************************************/
+static GtkListStore *players_dialog_store_new(void)
 {
+  GtkListStore *store;
   GType model_types[num_player_dlg_columns + 3];
   int i;
 
@@ -266,21 +268,22 @@ static void create_store(void)
     }
   }
   /* special (invisible rows) - Text style, weight and player id */
-  model_types[i++] = G_TYPE_INT;
-  model_types[i++] = G_TYPE_INT;
-  model_types[i++] = G_TYPE_INT;
-  
-  ncolumns = i;
-  store = gtk_list_store_newv(ncolumns, model_types);  
-  model = GTK_TREE_MODEL(store);
-  
+  model_types[i++] = G_TYPE_INT;        /* PLR_DLG_COL_STYLE. */
+  model_types[i++] = G_TYPE_INT;        /* PLR_DLG_COL_WEIGHT. */
+  model_types[i++] = G_TYPE_INT;        /* PLR_DLG_COL_ID. */
+
+  store = gtk_list_store_newv(i, model_types);
+
   /* Set sort order */
   for (i = 0; i < num_player_dlg_columns; i++) {
     if (player_dlg_columns[i].sort_func != NULL) {
-        gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(model), i,
-	plrdlg_sort_func, GINT_TO_POINTER(i), NULL);
+        gtk_tree_sortable_set_sort_func(GTK_TREE_SORTABLE(store), i,
+                                        plrdlg_sort_func, GINT_TO_POINTER(i),
+                                        NULL);
     }
   }
+
+  return store;
 }
 
 /**************************************************************************
@@ -301,7 +304,63 @@ static void toggle_dead_players(GtkCheckMenuItem* item, gpointer data)
 {
   player_dlg_show_dead_players = 
     gtk_check_menu_item_get_active(GTK_CHECK_MENU_ITEM(item));
-  update_players_dialog();
+  real_players_dialog_update();
+}
+
+/**************************************************************************
+  Create and return the "diplomacy" menu for the player report. This menu
+  contains diplomacy actions the current player can use on other nations.
+**************************************************************************/
+static GtkWidget *create_diplomacy_menu(void)
+{
+  GtkWidget *menu, *item;
+
+  menu = gtk_menu_new();
+
+  item = gtk_menu_item_new_with_mnemonic(_("_Meet"));
+  g_signal_connect(item, "activate",
+                   G_CALLBACK(players_meet_callback), NULL);
+  gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+  players_meet_command = item;
+
+  item = gtk_menu_item_new_with_mnemonic(_("Cancel _Treaty"));
+  g_signal_connect(item, "activate",
+                   G_CALLBACK(players_war_callback), NULL);
+  gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+  players_war_command = item;
+
+  item = gtk_menu_item_new_with_mnemonic(_("_Withdraw Vision"));
+  g_signal_connect(item, "activate",
+                   G_CALLBACK(players_vision_callback), NULL);
+  gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+  players_vision_command = item;
+
+  return menu;
+}
+
+/**************************************************************************
+  Create and return the "intelligence" menu. The items in this menu are
+  used by the player to see more detailed information about other nations.
+**************************************************************************/
+static GtkWidget *create_intelligence_menu(void)
+{
+  GtkWidget *menu, *item;
+
+  menu = gtk_menu_new();
+
+  item = gtk_menu_item_new_with_mnemonic(_("_Report"));
+  g_signal_connect(item, "activate",
+                   G_CALLBACK(players_intel_callback), NULL);
+  gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+  players_int_command = item;
+
+  item = gtk_menu_item_new_with_mnemonic(_("_Spaceship"));
+  g_signal_connect(item, "activate",
+                   G_CALLBACK(players_sship_callback), NULL);
+  gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+  players_sship_command = item;
+
+  return menu;
 }
 
 /**************************************************************************
@@ -344,6 +403,7 @@ void create_players_dialog(void)
   int i;
   GtkWidget *sep, *sw;
   GtkWidget *menubar, *menu, *item, *vbox;
+  enum ai_level level;
 
   gui_dialog_new(&players_dialog_shell, GTK_NOTEBOOK(top_notebook), NULL);
   /* TRANS: Nations report title */
@@ -354,11 +414,12 @@ void create_players_dialog(void)
 
   gui_dialog_set_default_size(players_dialog_shell, -1, 270);
 
-  create_store();
+  players_dialog_store = players_dialog_store_new();
 
-  players_list = gtk_tree_view_new_with_model(GTK_TREE_MODEL(store));
-  g_object_unref(store);
-  gtk_widget_set_name(players_list, "small font");
+  players_list = gtk_tree_view_new_with_model(GTK_TREE_MODEL
+                                              (players_dialog_store));
+  g_object_unref(players_dialog_store);
+  gtk_widget_set_name(players_list, "small_font");
 
   players_selection = gtk_tree_view_get_selection(GTK_TREE_VIEW(players_list));
   g_signal_connect(players_selection, "changed",
@@ -440,41 +501,6 @@ void create_players_dialog(void)
   gtk_box_pack_start(GTK_BOX(players_dialog_shell->vbox), sw,
 		     TRUE, TRUE, 0);
 
-  players_int_command = gtk_button_new_with_mnemonic(_("_Intelligence"));
-  g_signal_connect(players_int_command, "clicked",
-      G_CALLBACK(players_intel_callback), NULL);
-  gui_dialog_add_widget(players_dialog_shell, players_int_command);  
-  gtk_size_group_add_widget(players_dialog_shell->gui_button, 
-      players_int_command);
-
-  players_meet_command = gtk_button_new_with_mnemonic(_("_Meet"));
-  g_signal_connect(players_meet_command, "clicked",
-      G_CALLBACK(players_meet_callback), NULL);
-  gui_dialog_add_widget(players_dialog_shell, players_meet_command);
-  gtk_size_group_add_widget(players_dialog_shell->gui_button, 
-      players_meet_command);
-
-  players_war_command = gtk_button_new_with_mnemonic(_("Cancel _Treaty"));
-  g_signal_connect(players_war_command, "clicked",
-      G_CALLBACK(players_war_callback), NULL);
-  gui_dialog_add_widget(players_dialog_shell, players_war_command);  
-  gtk_size_group_add_widget(players_dialog_shell->gui_button,
-      players_war_command);
-
-  players_vision_command  = gtk_button_new_with_mnemonic(_("_Withdraw vision"));
-  g_signal_connect(players_vision_command, "clicked",
-      G_CALLBACK(players_vision_callback), NULL);
-  gui_dialog_add_widget(players_dialog_shell, players_vision_command);  
-  gtk_size_group_add_widget(players_dialog_shell->gui_button,
-     players_vision_command);
-
-  players_sship_command = gtk_button_new_with_mnemonic(_("_Spaceship"));
-  g_signal_connect(players_sship_command, "clicked",
-      G_CALLBACK(players_sship_callback), NULL);
-  gui_dialog_add_widget(players_dialog_shell, players_sship_command);
-  gtk_size_group_add_widget(players_dialog_shell->gui_button,
-      players_sship_command);
-
   vbox = gtk_vbox_new(FALSE, 0);
   
   sep = gtk_hseparator_new();
@@ -485,7 +511,17 @@ void create_players_dialog(void)
 
   gui_dialog_add_widget(players_dialog_shell, vbox);
   gtk_box_set_child_packing(GTK_BOX(players_dialog_shell->action_area), 
-      vbox, FALSE, FALSE, 0, GTK_PACK_END);
+                            vbox, FALSE, FALSE, 0, GTK_PACK_START);
+
+  item = gtk_menu_item_new_with_mnemonic(_("Di_plomacy"));
+  menu = create_diplomacy_menu();
+  gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), menu);
+  gtk_menu_shell_append(GTK_MENU_SHELL(menubar), item);
+
+  item = gtk_menu_item_new_with_mnemonic(_("_Intelligence"));
+  menu = create_intelligence_menu();
+  gtk_menu_item_set_submenu(GTK_MENU_ITEM(item), menu);
+  gtk_menu_shell_append(GTK_MENU_SHELL(menubar), item);
 
   item = gtk_menu_item_new_with_mnemonic(_("_Display"));
   menu = create_show_menu();
@@ -506,18 +542,22 @@ void create_players_dialog(void)
   sep = gtk_separator_menu_item_new();
   gtk_menu_shell_append(GTK_MENU_SHELL(menu), sep);
 
-  for (i = 0; i < NUM_SKILL_LEVELS; i++) {
-    item = gtk_menu_item_new_with_label(_(skill_level_names[i]));
-    g_signal_connect(item, "activate",
-	G_CALLBACK(players_ai_skill_callback), GUINT_TO_POINTER(i));
-    gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+  for (level = 0; level < AI_LEVEL_LAST; level++) {
+    if (is_settable_ai_level(level)) {
+      const char *level_name = ai_level_name(level);
+
+      item = gtk_menu_item_new_with_label(level_name);
+      g_signal_connect(item, "activate",
+                       G_CALLBACK(players_ai_skill_callback),
+                       GUINT_TO_POINTER(level));
+      gtk_menu_shell_append(GTK_MENU_SHELL(menu), item);
+    }
   }
   gtk_widget_show_all(menu);
 
   gui_dialog_show_all(players_dialog_shell);
 
-  gtk_list_store_clear(store);
-  update_players_dialog();
+  real_players_dialog_update();
 
   gui_dialog_set_default_response(players_dialog_shell,
     GTK_RESPONSE_CLOSE);
@@ -532,12 +572,14 @@ void create_players_dialog(void)
 #define MIN_DIMENSION 5
 
 /**************************************************************************
- Builds the flag pixmap.
+ Builds the flag pixmap. May return NULL if there is not enough memory.
+ You must call g_object_unref on the returned pixbuf when it is no
+ longer needed.
 **************************************************************************/
 GdkPixbuf *get_flag(const struct nation_type *nation)
 {
   int x0, y0, x1, y1, w, h;
-  GdkPixbuf *im, *im2;
+  GdkPixbuf *im;
   struct sprite *flag;
 
   flag = get_nation_flag_sprite(tileset, nation);
@@ -545,71 +587,71 @@ GdkPixbuf *get_flag(const struct nation_type *nation)
   /* calculate the bounding box ... */
   sprite_get_bounding_box(flag, &x0, &y0, &x1, &y1);
 
-  assert(x0 != -1);
-  assert(y0 != -1);
-  assert(x1 != -1);
-  assert(y1 != -1);
+  fc_assert_ret_val(x0 != -1, NULL);
+  fc_assert_ret_val(y0 != -1, NULL);
+  fc_assert_ret_val(x1 != -1, NULL);
+  fc_assert_ret_val(y1 != -1, NULL);
 
   w = (x1 - x0) + 1;
   h = (y1 - y0) + 1;
 
   /* if the flag is smaller then 5 x 5, something is wrong */
-  assert(w >= MIN_DIMENSION && h >= MIN_DIMENSION);
+  fc_assert_ret_val(w >= MIN_DIMENSION && h >= MIN_DIMENSION, NULL);
 
   /* croping */
-  im = gdk_pixbuf_new_subpixbuf(sprite_get_pixbuf(flag), x0, y0, w, h);
-  im2 = gdk_pixbuf_copy(im);
-  g_object_unref(im);
+  im = gdk_pixbuf_new(GDK_COLORSPACE_RGB, TRUE, 8, w, h);
+  if (im != NULL) {
+    gdk_pixbuf_copy_area(sprite_get_pixbuf(flag), x0, y0, w, h,
+                         im, 0, 0);
+  }
 
   /* and finaly store the scaled flag pixbuf in the static flags array */
-  return im2;
+  return im;
 }
 
 
 /**************************************************************************
- Builds the text for the cells of a row in the player report. If
- update is TRUE, only the changable entries are build.
+  Fills the player list with the information for 'pplayer' at the row
+  given by 'it'.
 **************************************************************************/
-static void build_row(GtkTreeIter *it, int i)
+static void fill_row(GtkListStore *store, GtkTreeIter *it,
+                     const struct player *pplayer)
 {
-  struct player *plr = player_by_number(i);
+  struct player_dlg_column* pcol;
   GdkPixbuf *pixbuf;
-  gint style = PANGO_STYLE_NORMAL, weight = PANGO_WEIGHT_NORMAL;
+  int style = PANGO_STYLE_NORMAL, weight = PANGO_WEIGHT_NORMAL;
   int k;
-  gchar *p;
 
   for (k = 0; k < num_player_dlg_columns; k++) {
-    struct player_dlg_column* pcol = &player_dlg_columns[k];
-    switch(pcol->type) {
-      case COL_TEXT:
-      case COL_RIGHT_TEXT:
-        p = (gchar*)(pcol->func(plr));
-	gtk_list_store_set(store, it, k, p, -1);
-	break;
-      case COL_FLAG:
-        pixbuf = get_flag(nation_of_player(plr));
+    pcol = &player_dlg_columns[k];
+    switch (pcol->type) {
+    case COL_TEXT:
+    case COL_RIGHT_TEXT:
+      gtk_list_store_set(store, it, k, pcol->func(pplayer), -1);
+      break;
+    case COL_FLAG:
+      pixbuf = get_flag(nation_of_player(pplayer));
+      if (pixbuf != NULL) {
         gtk_list_store_set(store, it, k, pixbuf, -1);
         g_object_unref(pixbuf);
-	break;
-      case COL_COLOR:
-	pixbuf = create_player_icon(plr);
+      }
+      break;
+    case COL_COLOR:
+      pixbuf = create_player_icon(pplayer);
+      if (pixbuf != NULL) {
         gtk_list_store_set(store, it, k, pixbuf, -1);
-	g_object_unref(pixbuf);
-	break;
-      case COL_BOOLEAN:
-        gtk_list_store_set(store, it, k, (gboolean)pcol->bool_func(plr), -1);
-	break;
+        g_object_unref(pixbuf);
+      }
+      break;
+    case COL_BOOLEAN:
+      gtk_list_store_set(store, it, k, pcol->bool_func(pplayer), -1);
+      break;
     }
   }
 
-  /* The playerid */
-  gtk_list_store_set(store, it,
-    ncolumns - 1, (gint)i,
-    -1);
-
    /* now add some eye candy ... */
-  if (game.player_ptr) {
-    switch (pplayer_get_diplstate(game.player_ptr, plr)->type) {
+  if (client_has_player()) {
+    switch (player_diplstate_get(client_player(), pplayer)->type) {
     case DS_WAR:
       weight = PANGO_WEIGHT_NORMAL;
       style = PANGO_STYLE_ITALIC;
@@ -627,83 +669,61 @@ static void build_row(GtkTreeIter *it, int i)
       style = PANGO_STYLE_NORMAL;
       break;
     case DS_LAST:
-      assert(0);
       break;
     }
   }
-   gtk_list_store_set(store, it,
-       num_player_dlg_columns, style,
-       num_player_dlg_columns + 1, weight,
-       -1);
+
+  gtk_list_store_set(store, it,
+                     PLR_DLG_COL_STYLE, style,
+                     PLR_DLG_COL_WEIGHT, weight,
+                     PLR_DLG_COL_ID, player_number(pplayer),
+                     -1);
 }
 
 /**************************************************************************
-...
+  Return TRUE if the player should be shown in the player list.
 **************************************************************************/
-static bool player_should_be_shown(int plrno) {
-  return is_valid_player_id(plrno)
-	 && (player_dlg_show_dead_players
-	     || game.players[plrno].is_alive)
-	 && (!is_barbarian(&game.players[plrno]));
-}
-
-/**************************************************************************
-...
-**************************************************************************/
-void update_players_dialog(void)
+static bool player_should_be_shown(const struct player *pplayer)
 {
-  if (players_dialog_shell && !is_plrdlg_frozen()) {
-    gboolean exists[game.info.nplayers];
-    gint i;
-    GtkTreeIter it, it_next;
+  return NULL != pplayer && (player_dlg_show_dead_players
+                             || pplayer->is_alive)
+         && (!is_barbarian(pplayer));
+}
 
-    for (i = 0; i < game.info.nplayers; i++) {
-      exists[i] = FALSE;
-    }
+/**************************************************************************
+  Clear and refill the entire player list.
+**************************************************************************/
+void real_players_dialog_update(void)
+{
+  GtkTreeModel *model;
+  GtkTreeIter iter;
+  int selected;
 
-    if (gtk_tree_model_get_iter_first(model, &it)) {
-      gint plrno;
-      bool more;
-
-      do {
-	it_next = it;
-	more = gtk_tree_model_iter_next(model, &it_next);
-
-	gtk_tree_model_get(model, &it, ncolumns - 1, &plrno, -1);
-
-	/*
-	 * The nation already had a row in the player report. In that
-	 * case we just update the row. If player is dead we remove him
-	 * if necessary.
-	 */
-	if (player_should_be_shown(plrno)) {
-	  exists[plrno] = TRUE;
-	  build_row(&it, plrno);
-	} else {
-	  gtk_list_store_remove(store, &it);
-	}
-	it = it_next;
-      } while (more);
-    }
-
-    players_iterate(pplayer) {
-      /* skip barbarians */
-      if (player_should_be_shown(pplayer->player_no)) {
-	if (!exists[pplayer->player_no]) {
-	  /* 
-	   * A nation is not in the player report yet. This happens when
-	   * the report is just opened and after a split.
-	   */
-	  gtk_list_store_append(store, &it);
-
-	  build_row(&it, pplayer->player_no);
-	}
-      }
-    } players_iterate_end;
-
-    update_players_menu();
-    update_views();
+  if (NULL == players_dialog_shell) {
+    return;
   }
+
+  /* Save the selection. */
+  if (gtk_tree_selection_get_selected(players_selection, &model, &iter)) {
+    gtk_tree_model_get(model, &iter, PLR_DLG_COL_ID, &selected, -1);
+  } else {
+    selected = -1;
+  }
+
+  gtk_list_store_clear(players_dialog_store);
+  players_iterate(pplayer) {
+    if (!player_should_be_shown(pplayer)) {
+      continue;
+    }
+    gtk_list_store_append(players_dialog_store, &iter);
+    fill_row(players_dialog_store, &iter, pplayer);
+    if (player_number(pplayer) == selected) {
+      /* Restore the selection. */
+      gtk_tree_selection_select_iter(players_selection, &iter);
+    }
+  } players_iterate_end;
+
+  update_views();
 }
 
 /**************************************************************************
@@ -718,9 +738,9 @@ void players_meet_callback(GtkMenuItem *item, gpointer data)
   if (gtk_tree_selection_get_selected(players_selection, &model, &it)) {
     gint plrno;
 
-    gtk_tree_model_get(model, &it, ncolumns - 1, &plrno, -1);
+    gtk_tree_model_get(model, &it, PLR_DLG_COL_ID, &plrno, -1);
 
-    dsend_packet_diplomacy_init_meeting_req(&aconnection, plrno);
+    dsend_packet_diplomacy_init_meeting_req(&client.conn, plrno);
   }
 }
 
@@ -735,10 +755,10 @@ void players_war_callback(GtkMenuItem *item, gpointer data)
   if (gtk_tree_selection_get_selected(players_selection, &model, &it)) {
     gint plrno;
 
-    gtk_tree_model_get(model, &it, ncolumns - 1, &plrno, -1);
+    gtk_tree_model_get(model, &it, PLR_DLG_COL_ID, &plrno, -1);
 
     /* can be any pact clause */
-    dsend_packet_diplomacy_cancel_pact(&aconnection, plrno,
+    dsend_packet_diplomacy_cancel_pact(&client.conn, plrno,
 				       CLAUSE_CEASEFIRE);
   }
 }
@@ -754,8 +774,8 @@ void players_vision_callback(GtkMenuItem *item, gpointer data)
   if (gtk_tree_selection_get_selected(players_selection, &model, &it)) {
     gint plrno;
 
-    gtk_tree_model_get(model, &it, ncolumns - 1, &plrno, -1);
-    dsend_packet_diplomacy_cancel_pact(&aconnection, plrno, CLAUSE_VISION);
+    gtk_tree_model_get(model, &it, PLR_DLG_COL_ID, &plrno, -1);
+    dsend_packet_diplomacy_cancel_pact(&client.conn, plrno, CLAUSE_VISION);
   }
 }
 
@@ -770,10 +790,10 @@ void players_intel_callback(GtkMenuItem *item, gpointer data)
   if (gtk_tree_selection_get_selected(players_selection, &model, &it)) {
     gint plrno;
 
-    gtk_tree_model_get(model, &it, ncolumns - 1, &plrno, -1);
+    gtk_tree_model_get(model, &it, PLR_DLG_COL_ID, &plrno, -1);
 
-    if (can_intel_with_player(&game.players[plrno])) {
-      popup_intel_dialog(&game.players[plrno]);
+    if (can_intel_with_player(player_by_number(plrno))) {
+      popup_intel_dialog(player_by_number(plrno));
     }
   }
 }
@@ -789,8 +809,8 @@ void players_sship_callback(GtkMenuItem *item, gpointer data)
   if (gtk_tree_selection_get_selected(players_selection, &model, &it)) {
     gint plrno;
 
-    gtk_tree_model_get(model, &it, ncolumns - 1, &plrno, -1);
-    popup_spaceship_dialog(&game.players[plrno]);
+    gtk_tree_model_get(model, &it, PLR_DLG_COL_ID, &plrno, -1);
+    popup_spaceship_dialog(player_by_number(plrno));
   }
 }
 
@@ -804,13 +824,10 @@ static void players_ai_toggle_callback(GtkMenuItem *item, gpointer data)
 
   if (gtk_tree_selection_get_selected(players_selection, &model, &it)) {
     gint plrno;
-    char buf[512];
 
-    gtk_tree_model_get(model, &it, ncolumns - 1, &plrno, -1);
+    gtk_tree_model_get(model, &it, PLR_DLG_COL_ID, &plrno, -1);
 
-    my_snprintf(buf, sizeof(buf), "/aitoggle \"%s\"",
-                player_name(player_by_number(plrno)));
-    send_chat(buf);
+    send_chat_printf("/aitoggle \"%s\"", player_name(player_by_number(plrno)));
   }
 }
 
@@ -824,14 +841,12 @@ static void players_ai_skill_callback(GtkMenuItem *item, gpointer data)
 
   if (gtk_tree_selection_get_selected(players_selection, &model, &it)) {
     gint plrno;
-    char buf[512];
 
-    gtk_tree_model_get(model, &it, ncolumns - 1, &plrno, -1);
+    gtk_tree_model_get(model, &it, PLR_DLG_COL_ID, &plrno, -1);
 
-    my_snprintf(buf, sizeof(buf), "/%s %s",
-                skill_level_names[GPOINTER_TO_UINT(data)],
-                player_name(player_by_number(plrno)));
-    send_chat(buf);
+    send_chat_printf("/%s %s",
+                     ai_level_cmd(GPOINTER_TO_UINT(data)),
+                     player_name(player_by_number(plrno)));
   }
 }
 
