@@ -54,7 +54,7 @@ static SDL_Surface *pFuture_Tech_Icon;
 #define load_GUI_surface(pSpr, pStruct, pSurf, tag)		  \
 do {								  \
   pSpr = theme_lookup_sprite_tag_alt(theme, LOG_FATAL, tag, "", "", ""); \
-  assert(pSpr != NULL);				                  \
+  fc_assert_action(pSpr != NULL, break);                          \
   pStruct->pSurf = adj_surf(GET_SURF_REAL(pSpr));                 \
   FREESURFACE(GET_SURF_REAL(pSpr));                               \
 } while(0)
@@ -126,7 +126,7 @@ void tilespec_setup_city_gfx(void) {
 
   pCity_Surf = (pSpr ? adj_surf(GET_SURF_REAL(pSpr)) : NULL);
   
-  assert(pCity_Surf != NULL);
+  fc_assert(pCity_Surf != NULL);
 }
 
 /**********************************************************************
@@ -558,7 +558,7 @@ SDL_Surface * get_tech_icon(Tech_type_id tech)
 **************************************************************************/
 SDL_Color * get_tech_color(Tech_type_id tech_id)
 {
-  if (player_invention_reachable(client.conn.playing, tech_id))
+  if (player_invention_reachable(client.conn.playing, tech_id, FALSE))
   {
     switch (player_invention_state(client.conn.playing, tech_id))
     {
