@@ -29,15 +29,16 @@
 #include <X11/Xaw/List.h>
 #include <X11/Xaw/Viewport.h>
 
-
+/* common & utility */
 #include "fcintl.h"
 #include "log.h"
-#include "mem.h"     /* mystrdup() */
+#include "mem.h"     /* fc_strdup() */
 #include "support.h"
 #include "version.h"
 
-#include "civclient.h"
-#include "clinet.h"
+/* client */
+#include "client_main.h"
+#include "clinet.h"		/* connect_to_server() */
 #include "packhand.h"
 #include "servers.h"
 
@@ -139,7 +140,8 @@ static void password_callback(Widget w, XtPointer client_data,
  configure the dialog depending on what type of authentication request the
  server is making.
 **************************************************************************/
-void handle_authentication_req(enum authentication_type type, char *message)
+void handle_authentication_req(enum authentication_type type,
+                               const char *message)
 {
   XtVaSetValues(iinput, XtNstring, "", NULL);
   XtVaSetValues(connw, XtNlabel, _("Next"), NULL);
@@ -160,7 +162,7 @@ void handle_authentication_req(enum authentication_type type, char *message)
       struct packet_authentication_reply reply;
 
       sz_strlcpy(reply.password, password);
-      send_packet_authentication_reply(&aconnection, &reply);
+      send_packet_authentication_reply(&client.conn, &reply);
       return;
     } else {
       dialog_config = ENTER_PASSWORD_TYPE;
@@ -170,7 +172,8 @@ void handle_authentication_req(enum authentication_type type, char *message)
     dialog_config = ENTER_PASSWORD_TYPE;
     break;
   default:
-    assert(0);
+    log_error("Unsupported authentication type %d: %s.", type, message);
+    break;
   }
 
   XtPopup(shell, XtGrabNone);
@@ -214,7 +217,7 @@ void gui_server_connect(void)
   ihost=XtVaCreateManagedWidget("chosti", asciiTextWidgetClass, form, 
 			  XtNstring, server_host, NULL);
 
-  my_snprintf(buf, sizeof(buf), "%d", server_port);
+  fc_snprintf(buf, sizeof(buf), "%d", server_port);
   
   I_L(XtVaCreateManagedWidget("cportl", labelWidgetClass, form, NULL));
   iport=XtVaCreateManagedWidget("cporti", asciiTextWidgetClass, form, 
@@ -252,7 +255,7 @@ void gui_server_connect(void)
   this regenerates the player information from a loaded game on the server.
   currently a stub. TODO
 **************************************************************************/
-void handle_game_load(struct packet_game_load *packet)
+void handle_game_load(bool load_successful, const char *filename)
 { 
   /* PORTME */
 }
@@ -303,7 +306,7 @@ void connect_callback(Widget w, XtPointer client_data,
       popup_start_page();
       return;
     } else {
-      append_output_window(errbuf);
+      output_window_append(ftc_client, errbuf);
     }
   case NEW_PASSWORD_TYPE:
     XtVaGetValues(iinput, XtNstring, &dp, NULL);
@@ -319,7 +322,7 @@ void connect_callback(Widget w, XtPointer client_data,
       XtSetSensitive(connw, FALSE);
       memset(password, 0, MAX_LEN_NAME);
       password[0] = '\0';
-      send_packet_authentication_reply(&aconnection, &reply);
+      send_packet_authentication_reply(&client.conn, &reply);
     } else {
       XtVaSetValues(iinput, XtNstring, "", NULL);
       XtVaSetValues(imsg, XtNlabel, 
@@ -332,10 +335,11 @@ void connect_callback(Widget w, XtPointer client_data,
     XtSetSensitive(connw, FALSE);
     XtVaGetValues(iinput, XtNstring, &dp, NULL);
     sz_strlcpy(reply.password, (char*)dp);
-    send_packet_authentication_reply(&aconnection, &reply);
+    send_packet_authentication_reply(&client.conn, &reply);
     break;
   default:
-    assert(0);
+    log_error("Unsupported dialog configuration: %d", dialog_config)
+    break;
   }
 }
 
@@ -345,8 +349,8 @@ void connect_callback(Widget w, XtPointer client_data,
 static void server_scan_error(struct server_scan *scan,
 			      const char *message)
 {
-  append_output_window(message);
-  freelog(LOG_NORMAL, "%s", message);
+  output_window_append(ftc_client, message);
+  log_normal("%s", message);
   switch (server_scan_get_type(scan)) {
   case SERVER_SCAN_LOCAL:
     server_scan_finish(lan);
@@ -411,7 +415,7 @@ static void server_list_timer(XtPointer meta_list, XtIntervalId * id)
   if (get_server_list(server_list, errbuf, sizeof(errbuf))!=-1)  {
     XawListChange(meta_list, server_list, 0, 0, True);
   } else if (!lan_mode) {
-    append_output_window(errbuf);
+    output_window_append(ftc_client, errbuf);
   }
   num_lanservers_timer++;
 
@@ -541,7 +545,7 @@ static int get_server_list(char **list, char *errbuf, int n_errbuf)
       server_list = server_scan_get_list(lan);
       if (server_list == NULL) {
 	if (num_lanservers_timer == 0) {
-	  *list = mystrdup(" ");;
+	  *list = fc_strdup(" ");;
 	  return 0;
 	} else {
 	  return -1;
@@ -560,11 +564,11 @@ static int get_server_list(char **list, char *errbuf, int n_errbuf)
 
   server_list_iterate(server_list,pserver) {
     if (pserver == NULL) continue;
-    my_snprintf(line, sizeof(line), "%-35s %-5d %-11s %-11s %2d   %s",
+    fc_snprintf(line, sizeof(line), "%-35s %-5d %-11s %-11s %2d   %s",
 		pserver->host, pserver->port, pserver->version,
 		_(pserver->state), pserver->nplayers, pserver->message);
     if (*list) free(*list);
-    *list=mystrdup(line);
+    *list=fc_strdup(line);
     list++;
   } server_list_iterate_end;
 
