@@ -40,6 +40,8 @@
 #include "client_main.h"
 #include "control.h"
 
+/* client/gui-xaw */
+#include "citydlg.h"
 #include "dialogs.h"
 #include "gui_main.h"
 #include "gui_stuff.h"
@@ -95,10 +97,10 @@ static void diplomat_bribe_callback(Widget w, XtPointer client_data,
 {
   destroy_message_dialog(w);
 
-  if (game_find_unit_by_number(diplomat_id)
-   && game_find_unit_by_number(diplomat_target_id)) {
+  if (NULL != game_unit_by_number(diplomat_id)
+      && NULL != game_unit_by_number(diplomat_target_id)) {
     request_diplomat_answer(DIPLOMAT_BRIBE, diplomat_id,
-			    diplomat_target_id, 0);
+                            diplomat_target_id, 0);
   }
 }
 
@@ -107,26 +109,31 @@ static void diplomat_bribe_callback(Widget w, XtPointer client_data,
 **************************************************************************/
 void popup_bribe_dialog(struct unit *punit, int cost)
 {
-  char buf[128];
+  char tbuf[128], buf[128];
+
+  fc_snprintf(tbuf, ARRAY_SIZE(tbuf), PL_("Treasury contains %d gold.",
+                                          "Treasury contains %d gold.",
+                                          client_player()->economic.gold),
+              client_player()->economic.gold);
 
   if (unit_has_type_flag(punit, F_UNBRIBABLE)) {
     popup_message_dialog(toplevel, "diplomatbribedialog",
                          _("This unit cannot be bribed!"),
                          diplomat_bribe_no_callback, 0, 0, NULL);
-  } else if (cost <= client.conn.playing->economic.gold) {
-    my_snprintf(buf, sizeof(buf),
-		_("Bribe unit for %d gold?\n"
-		  "Treasury contains %d gold."), 
-		cost, client.conn.playing->economic.gold);
+  } else if (cost <= client_player()->economic.gold) {
+    fc_snprintf(buf, sizeof(buf),
+                /* TRANS: %s is pre-pluralised "Treasury contains %d gold." */
+                PL_("Bribe unit for %d gold?\n%s",
+                    "Bribe unit for %d gold?\n%s", cost), cost, tbuf);
     popup_message_dialog(toplevel, "diplomatbribedialog", buf,
 			 diplomat_bribe_yes_callback, 0, 0,
 			 diplomat_bribe_no_callback, 0, 0,
 			 NULL);
   } else {
-    my_snprintf(buf, sizeof(buf),
-		_("Bribing the unit costs %d gold.\n"
-		  "Treasury contains %d gold."), 
-		cost, client.conn.playing->economic.gold);
+    fc_snprintf(buf, sizeof(buf),
+                /* TRANS: %s is pre-pluralised "Treasury contains %d gold." */
+                PL_("Bribing the unit costs %d gold.\n%s",
+                    "Bribing the unit costs %d gold.\n%s", cost), cost, tbuf);
     popup_message_dialog(toplevel, "diplomatnogolddialog", buf,
 			 diplomat_bribe_no_callback, 0, 0,
 			 NULL);
@@ -142,10 +149,10 @@ static void diplomat_sabotage_callback(Widget w, XtPointer client_data,
   destroy_message_dialog(w);
   diplomat_dialog = NULL;
 
-  if(game_find_unit_by_number(diplomat_id) && 
-     game_find_city_by_number(diplomat_target_id)) { 
+  if (NULL != game_unit_by_number(diplomat_id)
+      && NULL != game_city_by_number(diplomat_target_id)) {
     request_diplomat_action(DIPLOMAT_SABOTAGE, diplomat_id,
-			    diplomat_target_id, -1);
+                            diplomat_target_id, B_LAST + 1);
   }
 
   process_diplomat_arrival(NULL, 0);
@@ -160,10 +167,10 @@ static void diplomat_embassy_callback(Widget w, XtPointer client_data,
   destroy_message_dialog(w);
   diplomat_dialog = NULL;
 
-  if(game_find_unit_by_number(diplomat_id) && 
-     (game_find_city_by_number(diplomat_target_id))) { 
+  if (NULL != game_unit_by_number(diplomat_id)
+      && NULL != game_city_by_number(diplomat_target_id)) {
     request_diplomat_action(DIPLOMAT_EMBASSY, diplomat_id,
-			    diplomat_target_id, 0);
+                            diplomat_target_id, 0);
   }
 
   process_diplomat_arrival(NULL, 0);
@@ -172,16 +179,16 @@ static void diplomat_embassy_callback(Widget w, XtPointer client_data,
 /****************************************************************
 ...
 *****************************************************************/
-static void diplomat_investigate_callback(Widget w, XtPointer client_data, 
-					  XtPointer call_data)
+static void diplomat_investigate_callback(Widget w, XtPointer client_data,
+                                          XtPointer call_data)
 {
   destroy_message_dialog(w);
   diplomat_dialog = NULL;
 
-  if(game_find_unit_by_number(diplomat_id) && 
-     (game_find_city_by_number(diplomat_target_id))) { 
+  if (NULL != game_city_by_number(diplomat_target_id)
+      && NULL != game_unit_by_number(diplomat_id)) {
     request_diplomat_action(DIPLOMAT_INVESTIGATE, diplomat_id,
-			    diplomat_target_id, 0);
+                            diplomat_target_id, 0);
   }
 
   process_diplomat_arrival(NULL, 0);
@@ -208,8 +215,8 @@ static void spy_poison_callback(Widget w, XtPointer client_data,
   destroy_message_dialog(w);
   diplomat_dialog = NULL;
 
-  if(game_find_unit_by_number(diplomat_id) && 
-     (game_find_city_by_number(diplomat_target_id))) { 
+  if (NULL != game_unit_by_number(diplomat_id)
+      && NULL != game_city_by_number(diplomat_target_id)) {
     request_diplomat_action(SPY_POISON, diplomat_id, diplomat_target_id, 0);
   }
 
@@ -225,8 +232,8 @@ static void diplomat_steal_callback(Widget w, XtPointer client_data,
   destroy_message_dialog(w);
   diplomat_dialog = NULL;
 
-  if(game_find_unit_by_number(diplomat_id) && 
-     game_find_city_by_number(diplomat_target_id)) { 
+  if (NULL != game_unit_by_number(diplomat_id)
+      && NULL != game_city_by_number(diplomat_target_id)) {
     request_diplomat_action(DIPLOMAT_STEAL, diplomat_id,
 			    diplomat_target_id, A_UNSET);
   }
@@ -306,15 +313,15 @@ static void spy_steal_callback(Widget w, XtPointer client_data,
   spy_tech_shell = 0l;
   
   if(!steal_advance){
-    freelog(LOG_ERROR, "Bug in spy steal tech code");
+    log_error("Bug in spy steal tech code");
     process_diplomat_arrival(NULL, 0);
     return;
   }
-  
-  if(game_find_unit_by_number(diplomat_id) && 
-     game_find_city_by_number(diplomat_target_id)) { 
+
+  if (NULL != game_unit_by_number(diplomat_id)
+      && NULL != game_city_by_number(diplomat_target_id)) {
     request_diplomat_action(DIPLOMAT_STEAL, diplomat_id,
-			    diplomat_target_id, steal_advance);
+                            diplomat_target_id, steal_advance);
   }
 
   process_diplomat_arrival(NULL, 0);
@@ -330,15 +337,15 @@ static void spy_sabotage_callback(Widget w, XtPointer client_data,
   spy_sabotage_shell = 0l;
   
   if(!sabotage_improvement){
-    freelog(LOG_ERROR, "Bug in spy sabotage code");
+    log_error("Bug in spy sabotage code");
     process_diplomat_arrival(NULL, 0);
     return;
   }
-  
-  if(game_find_unit_by_number(diplomat_id) && 
-     game_find_city_by_number(diplomat_target_id)) { 
+
+  if (NULL != game_unit_by_number(diplomat_id)
+      && NULL != game_city_by_number(diplomat_target_id)) {
     request_diplomat_action(DIPLOMAT_SABOTAGE, diplomat_id,
-			    diplomat_target_id, sabotage_improvement + 1);
+                            diplomat_target_id, sabotage_improvement + 1);
   }
 
   process_diplomat_arrival(NULL, 0);
@@ -518,10 +525,10 @@ static int create_improvements_list(struct player *pplayer,
 /****************************************************************
 ...
 *****************************************************************/
-static void spy_steal_popup(Widget w, XtPointer client_data, 
-			    XtPointer call_data)
+static void spy_steal_popup(Widget w, XtPointer client_data,
+                            XtPointer call_data)
 {
-  struct city *pvcity = game_find_city_by_number(diplomat_target_id);
+  struct city *pvcity = game_city_by_number(diplomat_target_id);
   struct player *pvictim = NULL;
 
   if(pvcity)
@@ -561,8 +568,8 @@ static void spy_request_sabotage_list(Widget w, XtPointer client_data,
   destroy_message_dialog(w);
   diplomat_dialog = NULL;
 
-  if (game_find_unit_by_number(diplomat_id)
-   && game_find_city_by_number(diplomat_target_id)) {
+  if (NULL != game_unit_by_number(diplomat_id)
+      && NULL != game_city_by_number(diplomat_target_id)) {
     request_diplomat_answer(DIPLOMAT_SABOTAGE, diplomat_id,
 			    diplomat_target_id, 0);
   }
@@ -626,8 +633,8 @@ static void diplomat_incite_callback(Widget w, XtPointer client_data,
   destroy_message_dialog(w);
   diplomat_dialog = NULL;
 
-  if (game_find_unit_by_number(diplomat_id)
-   && game_find_city_by_number(diplomat_target_id)) {
+  if (NULL != game_unit_by_number(diplomat_id)
+      && NULL != game_city_by_number(diplomat_target_id)) {
     request_diplomat_answer(DIPLOMAT_INCITE, diplomat_id,
 			    diplomat_target_id, 0);
   }
@@ -638,31 +645,36 @@ static void diplomat_incite_callback(Widget w, XtPointer client_data,
 **************************************************************************/
 void popup_incite_dialog(struct city *pcity, int cost)
 {
-  char buf[128];
+  char tbuf[128], buf[128];
+
+  fc_snprintf(tbuf, ARRAY_SIZE(tbuf), PL_("Treasury contains %d gold.",
+                                          "Treasury contains %d gold.",
+                                          client_player()->economic.gold),
+              client_player()->economic.gold);
 
   if (INCITE_IMPOSSIBLE_COST == cost) {
-    my_snprintf(buf, sizeof(buf), _("You can't incite a revolt in %s."),
+    fc_snprintf(buf, sizeof(buf), _("You can't incite a revolt in %s."),
 		city_name(pcity));
     popup_message_dialog(toplevel, "diplomatnogolddialog", buf,
 			 diplomat_incite_no_callback, 0, 0, NULL);
-  } else if (cost <= client.conn.playing->economic.gold) {
-    my_snprintf(buf, sizeof(buf),
-		_("Incite a revolt for %d gold?\n"
-		  "Treasury contains %d gold."), 
-		cost, client.conn.playing->economic.gold);
-   diplomat_target_id = pcity->id;
-   popup_message_dialog(toplevel, "diplomatrevoltdialog", buf,
-			diplomat_incite_yes_callback, 0, 0,
-			diplomat_incite_no_callback, 0, 0,
-			NULL);
+  } else if (cost <= client_player()->economic.gold) {
+    fc_snprintf(buf, sizeof(buf),
+                /* TRANS: %s is pre-pluralised "Treasury contains %d gold." */
+                PL_("Incite a revolt for %d gold?\n%s",
+                    "Incite a revolt for %d gold?\n%s", cost), cost, tbuf);
+    diplomat_target_id = pcity->id;
+    popup_message_dialog(toplevel, "diplomatrevoltdialog", buf,
+                         diplomat_incite_yes_callback, 0, 0,
+                         diplomat_incite_no_callback, 0, 0,
+                         NULL);
   } else {
-   my_snprintf(buf, sizeof(buf),
-	       _("Inciting a revolt costs %d gold.\n"
-		 "Treasury contains %d gold."), 
-	       cost, client.conn.playing->economic.gold);
-   popup_message_dialog(toplevel, "diplomatnogolddialog", buf,
-			diplomat_incite_no_callback, 0, 0,
-			NULL);
+    fc_snprintf(buf, sizeof(buf),
+                /* TRANS: %s is pre-pluralised "Treasury contains %d gold." */
+                PL_("Inciting a revolt costs %d gold.\n%s",
+                    "Inciting a revolt costs %d gold.\n%s", cost), cost, tbuf);
+    popup_message_dialog(toplevel, "diplomatnogolddialog", buf,
+                         diplomat_incite_no_callback, 0, 0,
+                         NULL);
   }
 }
 
@@ -680,11 +692,11 @@ static void diplomat_keep_moving_callback(Widget w, XtPointer client_data,
   destroy_message_dialog(w);
   diplomat_dialog = NULL;
 
-  if( (punit=game_find_unit_by_number(diplomat_id))
-      && (pcity=game_find_city_by_number(diplomat_target_id))
-      && !same_pos(punit->tile, pcity->tile)) {
+  if ((punit = game_unit_by_number(diplomat_id))
+      && (pcity = game_city_by_number(diplomat_target_id))
+      && !same_pos(unit_tile(punit), city_tile(pcity))) {
     request_diplomat_action(DIPLOMAT_MOVE, diplomat_id,
-			    diplomat_target_id, 0);
+                            diplomat_target_id, 0);
   }
   process_diplomat_arrival(NULL, 0);
 }
@@ -716,7 +728,7 @@ void popup_diplomat_dialog(struct unit *punit, struct tile *dest_tile)
     /* Spy/Diplomat acting against a city */
 
     diplomat_target_id=pcity->id;
-    my_snprintf(buf, sizeof(buf),
+    fc_snprintf(buf, sizeof(buf),
 		_("Your %s has arrived at %s.\nWhat is your command?"),
 		unit_name_translation(punit),
 		city_name(pcity));
