@@ -16,7 +16,10 @@
 #include <config.h>
 #endif
 
-#include <assert.h>
+#ifdef AUDIO_SDL
+#include "SDL.h"
+#endif
+
 #include <stdio.h>
 #include <stdlib.h>
 
@@ -37,31 +40,36 @@
 #include "canvas.h"
 #include "pixcomm.h"
 
+/* utility */
 #include "fciconv.h"
 #include "fcintl.h"
 #include "log.h"
 #include "support.h"
 
+/* common */
 #include "game.h"
 #include "government.h"
 #include "map.h"
 #include "unitlist.h"
 #include "version.h"
 
-#include "civclient.h"
+/* client */
+#include "client_main.h"
 #include "climisc.h"
 #include "clinet.h"
 #include "control.h"
+#include "editgui_g.h"
+#include "ggz_g.h"
 #include "options.h"
 #include "text.h"
 #include "tilespec.h"
 
+/* gui-xaw */
 #include "actions.h"
 #include "colors.h"
 #include "dialogs.h"
 #include "graphics.h"
 #include "gui_stuff.h"		/* I_SW() */
-#include "helpdata.h"		/* boot_help_texts() */
 #include "mapview.h"
 #include "menu.h"
 #include "optiondlg.h"
@@ -75,11 +83,6 @@ const char *client_string = "gui-xaw";
 const char * const gui_character_encoding = NULL;
 const bool gui_use_transliteration = TRUE;
 
-client_option gui_options[] = {
-  /* None. */
-};
-const int num_gui_options = ARRAY_SIZE(gui_options);
-
 static AppResources appResources;
 
 /* ids of the units icons in information display: (or 0) */
@@ -219,7 +222,7 @@ Atom wm_delete_window;
 void set_city_names_font_sizes(int my_city_names_font_size,
 			       int my_city_productions_font_size)
 {
-  freelog(LOG_ERROR, "Unimplemented set_city_names_font_sizes.");
+  log_error("Unimplemented set_city_names_font_sizes.");
   /* PORTME */
 }
 
@@ -244,7 +247,8 @@ static void print_usage(const char *argv0)
   /* add client-specific usage information here */
   fc_fprintf(stderr, _("This client has no special command line options\n\n"));
 
-  fc_fprintf(stderr, _("Report bugs at %s.\n"), BUG_URL);
+  /* TRANS: No full stop after the URL, could cause confusion. */
+  fc_fprintf(stderr, _("Report bugs at %s\n"), BUG_URL);
 }
 
 /**************************************************************************
@@ -274,7 +278,7 @@ static Boolean toplevel_work_proc(XtPointer client_data)
   /* This will cause the connect dialog to pop-up.
      We do it here so that the main window exists when that happens,
      so that the connect dialog can position itself relative to it. */
-  set_client_state(C_S_PREPARING);
+  set_client_state(C_S_DISCONNECTED);
   return (True);
 }
 
@@ -286,8 +290,24 @@ void ui_init(void)
 
 }
 
+/****************************************************************************
+  Extra initializers for client options.
+****************************************************************************/
+void gui_options_extra_init(void)
+{
+  /* Nothing to do. */
+}
+
 /**************************************************************************
-...
+  Entry point for whole freeciv client program.
+**************************************************************************/
+int main(int argc, char **argv)
+{
+  return client_main(argc, argv);
+}
+
+/**************************************************************************
+  Entry point for GUI specific portion. Called from client_main()
 **************************************************************************/
 void ui_main(int argc, char *argv[])
 {
@@ -320,25 +340,25 @@ void ui_main(int argc, char *argv[])
   XSetErrorHandler(myerr);*/
 
   if(appResources.version==NULL)  {
-    freelog(LOG_FATAL, _("No version number in resources."));
-    freelog(LOG_FATAL, _("You probably have an old (circa V1.0)"
-			 " Freeciv resource file somewhere."));
+    log_fatal(_("No version number in resources."));
+    log_fatal(_("You probably have an old (circa V1.0)"
+                " Freeciv resource file somewhere."));
     exit(EXIT_FAILURE);
   }
 
   /* TODO: Use capabilities here instead of version numbers */
   if (0 != strncmp(appResources.version, VERSION_STRING,
 		   strlen(appResources.version))) {
-    freelog(LOG_FATAL, _("Game version does not match Resource version."));
-    freelog(LOG_FATAL, _("Game version: %s - Resource version: %s"), 
-	    VERSION_STRING, appResources.version);
-    freelog(LOG_FATAL, _("You might have an old Freeciv resourcefile"
-			 " in /usr/lib/X11/app-defaults"));
+    log_fatal(_("Game version does not match Resource version."));
+    log_fatal(_("Game version: %s - Resource version: %s"),
+              VERSION_STRING, appResources.version);
+    log_fatal(_("You might have an old Freeciv resourcefile"
+                " in /usr/lib/X11/app-defaults"));
     exit(EXIT_FAILURE);
   }
   
   if(!appResources.gotAppDefFile) {
-    freelog(LOG_NORMAL, _("Using fallback resources - which is OK"));
+    log_normal(_("Using fallback resources - which is OK"));
   }
 
   display = XtDisplay(toplevel);
@@ -349,7 +369,7 @@ void ui_main(int argc, char *argv[])
   display_color_type=get_visual(); 
   
   if(display_color_type!=COLOR_DISPLAY) {
-    freelog(LOG_FATAL, _("Only color displays are supported for now..."));
+    log_fatal(_("Only color displays are supported for now..."));
     /*    exit(EXIT_FAILURE); */
   }
 
@@ -363,24 +383,22 @@ void ui_main(int argc, char *argv[])
     values.graphics_exposures = False;
     civ_gc = XCreateGC(display, root_window, GCGraphicsExposures, &values);
 
-    city_names_font = mystrdup("-*-*-*-*-*-*-14-*");
+    city_names_font = fc_strdup("-*-*-*-*-*-*-14-*");
 
-    city_productions_font_name = mystrdup("-*-*-*-*-*-*-14-*");
+    city_productions_font_name = fc_strdup("-*-*-*-*-*-*-14-*");
 
     main_font_set = XCreateFontSet(display, city_names_font,
 	&missing_charset_list_return,
 	&missing_charset_count_return,
 	&def_string_return);
     if (!main_font_set) {
-      freelog(LOG_FATAL, _("Unable to open fontset: %s"),
-	      city_names_font);
-      freelog(LOG_FATAL,
-	      _("Doing 'xset fp rehash' may temporarily solve a problem."));
+      log_fatal(_("Unable to open fontset: %s"), city_names_font);
+      log_fatal(_("Doing 'xset fp rehash' may temporarily solve a problem."));
       exit(EXIT_FAILURE);
     }
     for (i = 0; i < missing_charset_count_return; i++) {
-      freelog(LOG_ERROR, _("Font for charset %s is lacking"),
-	      missing_charset_list_return[i]);
+      log_error(_("Font for charset %s is lacking"),
+                missing_charset_list_return[i]);
     }
     values.foreground = get_color(tileset, COLOR_MAPVIEW_CITYTEXT)->color.pixel;
     values.background = get_color(tileset, COLOR_MAPVIEW_UNKNOWN)->color.pixel;
@@ -393,15 +411,13 @@ void ui_main(int argc, char *argv[])
 	&missing_charset_count_return,
 	&def_string_return);
     if (!prod_font_set) {
-      freelog(LOG_FATAL, _("Unable to open fontset: %s"),
-	      city_productions_font_name);
-      freelog(LOG_FATAL,
-	      _("Doing 'xset fp rehash' may temporarily solve a problem."));
+      log_fatal(_("Unable to open fontset: %s"), city_productions_font_name);
+      log_fatal(_("Doing 'xset fp rehash' may temporarily solve a problem."));
       exit(EXIT_FAILURE);
     }
     for (i = 0; i < missing_charset_count_return; i++) {
-      freelog(LOG_ERROR, _("Font for charset %s is lacking"),
-	      missing_charset_list_return[i]);
+      log_error(_("Font for charset %s is lacking"),
+                missing_charset_list_return[i]);
     }
     values.foreground = get_color(tileset, COLOR_MAPVIEW_CITYTEXT)->color.pixel;
     values.background = get_color(tileset, COLOR_MAPVIEW_UNKNOWN)->color.pixel;
@@ -507,6 +523,14 @@ void ui_exit()
 
 }
 
+/**************************************************************************
+  Return our GUI type
+**************************************************************************/
+enum gui_type get_gui_type(void)
+{
+  return GUI_XAW;
+}
+
 /**************************************************************************
   Callack for when user clicks one of the unit icons on left hand side
   (units on same square as current unit).  Use unit_ids[] data and change
@@ -518,12 +542,13 @@ static void unit_icon_callback(Widget w, XtPointer client_data,
   struct unit *punit;
   int i = (size_t)client_data;
 
-  assert(i>=0 && i<num_units_below);
-  if (unit_ids[i] == 0) /* no unit displayed at this place */
+  fc_assert_ret(i >= 0 && i < num_units_below);
+  if (unit_ids[i] == 0) { /* no unit displayed at this place */
     return;
-  punit=game_find_unit_by_number(unit_ids[i]);
-  if(punit) { /* should always be true at this point */
-    if (unit_owner(punit) == game.player_ptr) {
+  }
+  punit = game_unit_by_number(unit_ids[i]);
+  if (punit) { /* should always be true at this point */
+    if (unit_owner(punit) == client.conn.playing) {
       /* may be non-true if alliance */
       set_unit_focus(punit);
     }
@@ -657,7 +682,7 @@ void setup_widgets(void)
 
   for(i=0; i<num_units_below; i++) {
     char unit_below_name[32];
-    my_snprintf(unit_below_name, sizeof(unit_below_name),
+    fc_snprintf(unit_below_name, sizeof(unit_below_name),
 		"unitbelowcanvas%ld", i);
     unit_below_canvas[i] = XtVaCreateManagedWidget(unit_below_name,
 						   pixcommWidgetClass,
@@ -718,6 +743,7 @@ void setup_widgets(void)
 **************************************************************************/
 void xaw_ui_exit(void)
 {
+  free_mapcanvas_and_overview();
   tileset_free_tiles(tileset);
   client_exit();
 }
@@ -758,7 +784,7 @@ void main_show_info_popup(XEvent *event)
 /**************************************************************************
  Update the connected users list at pregame state.
 **************************************************************************/
-void update_conn_list_dialog(void)
+void real_conn_list_dialog_update(void)
 {
   /* PORTME */
   update_start_page();
@@ -790,9 +816,9 @@ static void set_wait_for_writable_socket(struct connection *pc,
 
   if (previous_state == socket_writable)
     return;
-  freelog(LOG_DEBUG, "set_wait_for_writable_socket(%d)", socket_writable);
+  log_debug("set_wait_for_writable_socket(%d)", socket_writable);
   XtRemoveInput(x_input_id);
-  x_input_id = XtAppAddInput(app_context, aconnection.sock,
+  x_input_id = XtAppAddInput(app_context, client.conn.sock,
 			     (XtPointer) (XtInputReadMask |
 					  (socket_writable ?
 					   XtInputWriteMask : 0) |
@@ -811,7 +837,7 @@ void add_net_input(int sock)
 			     (XtPointer) (XtInputReadMask |
 					  XtInputExceptMask),
 			     (XtInputCallbackProc) get_net_input, NULL);
-  aconnection.notify_of_writable_data = set_wait_for_writable_socket;
+  client.conn.notify_of_writable_data = set_wait_for_writable_socket;
 }
 
 /**************************************************************************
@@ -871,8 +897,8 @@ void timer_callback(XtPointer client_data, XtIntervalId * id)
 void set_unit_icon(int idx, struct unit *punit)
 {
   Widget w;
-  
-  assert(idx>=-1 && idx<num_units_below);
+
+  fc_assert_ret(idx >= -1 && idx < num_units_below);
   if (idx == -1) {
     w = unit_pix_canvas;
   } else {
@@ -900,7 +926,8 @@ void set_unit_icons_more_arrow(bool onoff)
 
   if (onoff && !showing) {
     /* FIXME: what about the mask? */
-    xaw_set_bitmap(more_arrow_label, get_arrow_sprite(tileset)->pixmap);
+    xaw_set_bitmap(more_arrow_label,
+		   get_arrow_sprite(tileset, ARROW_RIGHT)->pixmap);
     showing = TRUE;
   }
   else if(!onoff && showing) {
@@ -909,6 +936,16 @@ void set_unit_icons_more_arrow(bool onoff)
   }
 }
 
+/****************************************************************************
+  Called when the set of units in focus (get_units_in_focus()) changes.
+  Standard updates like update_unit_info_label() are handled in the platform-
+  independent code, so some clients will not need to do anything here.
+****************************************************************************/
+void real_focus_units_changed(void)
+{
+  /* Nothing to do */
+}
+
 struct callback {
   void (*callback)(void *data);
   void *data;
@@ -1036,7 +1073,7 @@ void select_battlegroup(int battlegroup)
 ****************************************************************************/
 void add_unit_to_battlegroup(int battlegroup)
 {
-  if (game.player_ptr && can_client_issue_orders()) {
+  if (NULL != client.conn.playing && can_client_issue_orders()) {
     struct unit *punit;
 
     punit = head_of_units_in_focus();
@@ -1049,3 +1086,60 @@ void add_unit_to_battlegroup(int battlegroup)
     }
   }
 }
+
+/****************************************************************************
+  Stub for editor function
+****************************************************************************/
+void editgui_tileset_changed(void)
+{}
+
+/****************************************************************************
+  Stub for editor function
+****************************************************************************/
+void editgui_refresh(void)
+{}
+
+/****************************************************************************
+  Stub for editor function
+****************************************************************************/
+void editgui_popup_properties(const struct tile_list *tiles, int objtype)
+{}
+
+/****************************************************************************
+  Stub for editor function
+****************************************************************************/
+void editgui_popdown_all(void)
+{}
+
+/****************************************************************************
+  Stub for editor function
+****************************************************************************/
+void editgui_notify_object_changed(int objtype, int object_id, bool remove)
+{}
+
+/****************************************************************************
+  Stub for editor function
+****************************************************************************/
+void editgui_notify_object_created(int tag, int id)
+{}
+
+/****************************************************************************
+  Stub for ggz function
+****************************************************************************/
+void gui_ggz_embed_leave_table(void)
+{}
+
+/****************************************************************************
+  Stub for ggz function
+****************************************************************************/
+void gui_ggz_embed_ensure_server(void)
+{}
+
+
+/**************************************************************************
+  Updates a gui font style.
+**************************************************************************/
+void gui_update_font(const char *font_name, const char *font_value)
+{
+  /* PORTME */
+}
