@@ -57,11 +57,10 @@
 
 /* utility */
 #include "fcintl.h"
-#include "hash.h"
 #include "log.h"
 #include "mem.h"
 #include "netintf.h"
-#include "rand.h" /* myrand() */
+#include "rand.h" /* fc_rand() */
 #include "registry.h"
 #include "support.h"
 
@@ -108,11 +107,11 @@ extern enum announce_type announce;
 static struct server_list *parse_metaserver_data(fz_FILE *f)
 {
   struct server_list *server_list;
-  struct section_file the_file, *file = &the_file;
+  struct section_file *file;
   int nservers, i, j;
 
   /* This call closes f. */
-  if (!section_file_load_from_stream(file, f)) {
+  if (!(file = secfile_from_stream(f, TRUE))) {
     return NULL;
   }
 
@@ -120,59 +119,63 @@ static struct server_list *parse_metaserver_data(fz_FILE *f)
   nservers = secfile_lookup_int_default(file, 0, "main.nservers");
 
   for (i = 0; i < nservers; i++) {
-    char *host, *port, *version, *state, *message, *nplayers;
+    const char *host, *port, *version, *state, *message, *nplayers, *nhumans;
     int n;
     struct server *pserver = (struct server*)fc_malloc(sizeof(struct server));
 
     host = secfile_lookup_str_default(file, "", "server%d.host", i);
-    pserver->host = mystrdup(host);
+    pserver->host = fc_strdup(host);
 
     port = secfile_lookup_str_default(file, "", "server%d.port", i);
     pserver->port = atoi(port);
 
     version = secfile_lookup_str_default(file, "", "server%d.version", i);
-    pserver->version = mystrdup(version);
+    pserver->version = fc_strdup(version);
 
     state = secfile_lookup_str_default(file, "", "server%d.state", i);
-    pserver->state = mystrdup(state);
+    pserver->state = fc_strdup(state);
 
     message = secfile_lookup_str_default(file, "", "server%d.message", i);
-    pserver->message = mystrdup(message);
+    pserver->message = fc_strdup(message);
 
     nplayers = secfile_lookup_str_default(file, "0", "server%d.nplayers", i);
     n = atoi(nplayers);
     pserver->nplayers = n;
 
-    if (n > 0) {
-      pserver->players = fc_malloc(n * sizeof(*pserver->players));
+    nhumans = secfile_lookup_str_default(file, "-1", "server%d.humans", i);
+    n = atoi(nhumans);
+    pserver->humans = n;
+
+    if (pserver->nplayers > 0) {
+      pserver->players = fc_malloc(pserver->nplayers * sizeof(*pserver->players));
     } else {
       pserver->players = NULL;
     }
-      
-    for (j = 0; j < n; j++) {
-      char *name, *nation, *type, *host;
+
+    for (j = 0; j < pserver->nplayers ; j++) {
+      const char *name, *nation, *type, *host;
 
       name = secfile_lookup_str_default(file, "", 
                                         "server%d.player%d.name", i, j);
-      pserver->players[j].name = mystrdup(name);
+      pserver->players[j].name = fc_strdup(name);
 
       type = secfile_lookup_str_default(file, "",
                                         "server%d.player%d.type", i, j);
-      pserver->players[j].type = mystrdup(type);
+      pserver->players[j].type = fc_strdup(type);
 
       host = secfile_lookup_str_default(file, "", 
                                         "server%d.player%d.host", i, j);
-      pserver->players[j].host = mystrdup(host);
+      pserver->players[j].host = fc_strdup(host);
 
       nation = secfile_lookup_str_default(file, "",
                                           "server%d.player%d.nation", i, j);
-      pserver->players[j].nation = mystrdup(nation);
+      pserver->players[j].nation = fc_strdup(nation);
     }
 
     server_list_append(server_list, pserver);
   }
 
-  section_file_free(file);
+  secfile_destroy(file);
   return server_list;
 }
 
@@ -186,11 +189,7 @@ static void my_uname(char *buf, size_t len)
     struct utsname un;
 
     uname(&un);
-    my_snprintf(buf, len,
-		"%s %s [%s]",
-		un.sysname,
-		un.release,
-		un.machine);
+    fc_snprintf(buf, len, "%s %s [%s]", un.sysname, un.release, un.machine);
   }
 #else /* ! HAVE_UNAME */
   /* Fill in here if you are making a binary without sys/utsname.h and know
@@ -251,8 +250,8 @@ static void my_uname(char *buf, size_t len)
 	    ptype = 6;
 	  else
 	    ptype = sysinfo.wProcessorLevel;
-	  
-	  my_snprintf(cpuname, sizeof(cpuname), "i%d86", ptype);
+
+          fc_snprintf(cpuname, sizeof(cpuname), "i%d86", ptype);
 	}
 	break;
 
@@ -276,14 +275,11 @@ static void my_uname(char *buf, size_t len)
 	sz_strlcpy(cpuname, "unknown");
 	break;
     }
-    my_snprintf(buf, len,
-		"%s %ld.%ld [%s]",
-		osname, osvi.dwMajorVersion, osvi.dwMinorVersion,
-		cpuname);
+    fc_snprintf(buf, len, "%s %ld.%ld [%s]",
+                osname, osvi.dwMajorVersion, osvi.dwMinorVersion, cpuname);
   }
 #else
-  my_snprintf(buf, len,
-              "unknown unknown [unknown]");
+  fc_snprintf(buf, len, "unknown unknown [unknown]");
 #endif
 #endif /* HAVE_UNAME */
 }
@@ -301,7 +297,7 @@ static void meta_send_request(struct server_scan *scan)
 
   capstr = fc_url_encode(our_capability);
 
-  my_snprintf(str, sizeof(str),
+  fc_snprintf(str, sizeof(str),
     "POST %s HTTP/1.1\r\n"
     "Host: %s:%d\r\n"
     "User-Agent: Freeciv/%s %s %s\r\n"
@@ -338,7 +334,7 @@ static void meta_read_response(struct server_scan *scan)
     char filename[MAX_PATH];
 
     GetTempPath(sizeof(filename), filename);
-    cat_snprintf(filename, sizeof(filename), "fctmp%d", myrand(1000));
+    cat_snprintf(filename, sizeof(filename), "fctmp%d", fc_rand(1000));
 
     scan->meta.fp = fc_fopen(filename, "w+b");
 #else
@@ -368,7 +364,13 @@ static void meta_read_response(struct server_scan *scan)
       rewind(scan->meta.fp);
 
       f = fz_from_stream(scan->meta.fp);
-      assert(f != NULL);
+      if (NULL == f) {
+        fc_snprintf(str, sizeof(str),
+                    _("Failed to read the metaserver data from http://%s."),
+                    scan->meta.name);
+        scan->error_func(scan, str);
+        return;
+      }
 
       /* skip HTTP headers */
       /* XXX: TODO check for magic Content-Type: text/x-ini -vasc */
@@ -386,9 +388,10 @@ static void meta_read_response(struct server_scan *scan)
       scan->meta.fp = NULL;
 
       if (NULL == scan->servers) {
-        my_snprintf(str, sizeof(str),
-                    _("Failed to parse the metaserver data from http://%s."),
-                    scan->meta.name);
+        fc_snprintf(str, sizeof(str),
+                    _("Failed to parse the metaserver data from http://%s:\n"
+                      "%s."),
+                    scan->meta.name, secfile_error());
         scan->error_func(scan, str);
       }
 
@@ -520,11 +523,9 @@ get_metaserver_list(struct server_scan *scan)
   case META_DONE:
     return SCAN_STATUS_DONE;
     break;
-  default:
-    break;
   }
 
-  assert(0);
+  log_error("Unsupported metaserver state: %d.", scan->meta.state);
   return SCAN_STATUS_ERROR;
 }
 
@@ -561,7 +562,7 @@ static void delete_server_list(struct server_list *server_list)
     free(ptmp);
   } server_list_iterate_end;
 
-  server_list_free(server_list);
+  server_list_destroy(server_list);
 }
 
 /**************************************************************************
@@ -608,13 +609,13 @@ static bool begin_lanserver_scan(struct server_scan *scan)
 
   /* Create a socket for broadcasting to servers. */
   if ((sock = socket(family, SOCK_DGRAM, 0)) < 0) {
-    freelog(LOG_ERROR, "socket failed: %s", fc_strerror(fc_get_errno()));
+    log_error("socket failed: %s", fc_strerror(fc_get_errno()));
     return FALSE;
   }
 
   if (setsockopt(sock, SOL_SOCKET, SO_REUSEADDR,
                  (char *)&opt, sizeof(opt)) == -1) {
-    freelog(LOG_ERROR, "SO_REUSEADDR failed: %s", fc_strerror(fc_get_errno()));
+    log_error("SO_REUSEADDR failed: %s", fc_strerror(fc_get_errno()));
   }
 
   /* Set the UDP Multicast group IP address. */
@@ -647,14 +648,14 @@ static bool begin_lanserver_scan(struct server_scan *scan)
   ttl = SERVER_LAN_TTL;
   if (setsockopt(sock, IPPROTO_IP, IP_MULTICAST_TTL, (const char*)&ttl, 
                  sizeof(ttl))) {
-    freelog(LOG_ERROR, "setsockopt failed: %s", fc_strerror(fc_get_errno()));
+    log_error("setsockopt failed: %s", fc_strerror(fc_get_errno()));
     return FALSE;
   }
 #endif /* HAVE_WINSOCK */
 
   if (setsockopt(sock, SOL_SOCKET, SO_BROADCAST, (const char*)&opt, 
                  sizeof(opt))) {
-    freelog(LOG_ERROR, "setsockopt failed: %s", fc_strerror(fc_get_errno()));
+    log_error("setsockopt failed: %s", fc_strerror(fc_get_errno()));
     return FALSE;
   }
 
@@ -667,11 +668,11 @@ static bool begin_lanserver_scan(struct server_scan *scan)
              sockaddr_size(&addr)) < 0) {
     /* This can happen when there's no network connection - it should
      * give an in-game message. */
-    freelog(LOG_ERROR, "lanserver scan sendto failed: %s",
-	    fc_strerror(fc_get_errno()));
+    log_error("lanserver scan sendto failed: %s",
+              fc_strerror(fc_get_errno()));
     return FALSE;
   } else {
-    freelog(LOG_DEBUG, ("Sending request for server announcement on LAN."));
+    log_debug("Sending request for server announcement on LAN.");
   }
 
   fc_closesocket(sock);
@@ -686,9 +687,9 @@ static bool begin_lanserver_scan(struct server_scan *scan)
 
   if (setsockopt(scan->sock, SOL_SOCKET, SO_REUSEADDR,
                  (char *)&opt, sizeof(opt)) == -1) {
-    freelog(LOG_ERROR, "SO_REUSEADDR failed: %s", fc_strerror(fc_get_errno()));
+    log_error("SO_REUSEADDR failed: %s", fc_strerror(fc_get_errno()));
   }
-                                                                               
+
   memset(&addr, 0, sizeof(addr));
 
 #ifdef IPV6_SUPPORT
@@ -789,7 +790,7 @@ get_lan_server_list(struct server_scan *scan)
     dio_get_string(&din, players, sizeof(players));
     dio_get_string(&din, message, sizeof(message));
 
-    if (!mystrcasecmp("none", servername)) {
+    if (!fc_strcasecmp("none", servername)) {
       bool nameinfo = FALSE;
 #ifdef IPV6_SUPPORT
       char dst[INET6_ADDRSTRLEN];
@@ -828,26 +829,26 @@ get_lan_server_list(struct server_scan *scan)
 
     /* UDP can send duplicate or delayed packets. */
     server_list_iterate(scan->servers, aserver) {
-      if (0 == mystrcasecmp(aserver->host, servername)
+      if (0 == fc_strcasecmp(aserver->host, servername)
           && aserver->port == port) {
 	duplicate = TRUE;
 	break;
-      } 
+      }
     } server_list_iterate_end;
     if (duplicate) {
       continue;
     }
 
-    freelog(LOG_DEBUG,
-            ("Received a valid announcement from a server on the LAN."));
-    
+    log_debug("Received a valid announcement from a server on the LAN.");
+
     pserver = fc_malloc(sizeof(*pserver));
-    pserver->host = mystrdup(servername);
+    pserver->host = fc_strdup(servername);
     pserver->port = port;
-    pserver->version = mystrdup(version);
-    pserver->state = mystrdup(status);
+    pserver->version = fc_strdup(version);
+    pserver->state = fc_strdup(status);
     pserver->nplayers = atoi(players);
-    pserver->message = mystrdup(message);
+    pserver->humans = -1;
+    pserver->message = fc_strdup(message);
     pserver->players = NULL;
     found_new = TRUE;
 
