@@ -23,7 +23,6 @@
 #include <config.h>
 #endif
 
-#include <assert.h>
 #include <limits.h>
 #include <stdint.h>
 #include <stdio.h>
@@ -43,13 +42,17 @@
 #include <winsock.h>
 #endif
 
+/* utility */
+#include "bitvector.h"
 #include "capability.h"
-#include "events.h"
 #include "log.h"
 #include "mem.h"
+#include "support.h"
+
+/* common */
+#include "events.h"
 #include "player.h"
 #include "requirements.h"
-#include "support.h"
 #include "tech.h"
 #include "worklist.h"
 
@@ -200,7 +203,7 @@ void dio_put_uint8(struct data_out *dout, int value)
   if (enough_space(dout, 1)) {
     uint8_t x = value;
 
-    assert(sizeof(x) == 1);
+    fc_assert(sizeof(x) == 1);
     memcpy(ADD_TO_POINTER(dout->dest, dout->current), &x, 1);
     dout->current++;
   }
@@ -214,7 +217,7 @@ void dio_put_uint16(struct data_out *dout, int value)
   if (enough_space(dout, 2)) {
     uint16_t x = htons(value);
 
-    assert(sizeof(x) == 2);
+    fc_assert(sizeof(x) == 2);
     memcpy(ADD_TO_POINTER(dout->dest, dout->current), &x, 2);
     dout->current += 2;
   }
@@ -228,7 +231,7 @@ void dio_put_uint32(struct data_out *dout, int value)
   if (enough_space(dout, 4)) {
     uint32_t x = htonl(value);
 
-    assert(sizeof(x) == 4);
+    fc_assert(sizeof(x) == 4);
     memcpy(ADD_TO_POINTER(dout->dest, dout->current), &x, 4);
     dout->current += 4;
   }
@@ -240,7 +243,7 @@ void dio_put_uint32(struct data_out *dout, int value)
 void dio_put_bool8(struct data_out *dout, bool value)
 {
   if (value != TRUE && value != FALSE) {
-    freelog(LOG_ERROR, "Trying to put a non-boolean: %d", (int) value);
+    log_error("Trying to put a non-boolean: %d", (int) value);
     value = FALSE;
   }
 
@@ -253,7 +256,7 @@ void dio_put_bool8(struct data_out *dout, bool value)
 void dio_put_bool32(struct data_out *dout, bool value)
 {
   if (value != TRUE && value != FALSE) {
-    freelog(LOG_ERROR, "Trying to put a non-boolean: %d", (int) value);
+    log_error("Trying to put a non-boolean: %d", (int) value);
     value = FALSE;
   }
 
@@ -344,8 +347,8 @@ void dio_put_bit_string(struct data_out *dout, const char *value)
   size_t max = (unsigned short)(-1);
 
   if (bits > max) {
-    freelog(LOG_ERROR, "Bit string too long: %lu bits.", (unsigned long)bits);
-    assert(FALSE);
+    fc_assert_msg(FALSE, "Bit string too long: %lu bits.",
+                  (unsigned long) bits);
     bits = max;
   }
   bytes = (bits + 7) / 8;
@@ -388,16 +391,14 @@ void dio_put_tech_list(struct data_out *dout, const int *value)
 **************************************************************************/
 void dio_put_worklist(struct data_out *dout, const struct worklist *pwl)
 {
-  dio_put_bool8(dout, pwl->is_valid);
+  int i, length = worklist_length(pwl);
 
-  if (pwl->is_valid) {
-    int i, length = worklist_length(pwl);
+  dio_put_uint8(dout, length);
+  for (i = 0; i < length; i++) {
+    const struct universal *pcp = &(pwl->entries[i]);
 
-    dio_put_uint8(dout, length);
-    for (i = 0; i < length; i++) {
-      dio_put_bool8(dout, pwl->entries[i].is_unit);
-      dio_put_uint8(dout, pwl->entries[i].value);
-    }
+    dio_put_uint8(dout, pcp->kind);
+    dio_put_uint8(dout, universal_number(pcp));
   }
 }
 
@@ -411,7 +412,7 @@ void dio_get_uint8(struct data_in *din, int *dest)
     if (dest) {
       uint8_t x;
 
-      assert(sizeof(x) == 1);
+      fc_assert(sizeof(x) == 1);
       memcpy(&x, ADD_TO_POINTER(din->src, din->current), 1);
       *dest = x;
     }
@@ -431,7 +432,7 @@ void dio_get_uint16(struct data_in *din, int *dest)
     if (dest) {
       uint16_t x;
 
-      assert(sizeof(x) == 2);
+      fc_assert(sizeof(x) == 2);
       memcpy(&x, ADD_TO_POINTER(din->src, din->current), 2);
       *dest = ntohs(x);
     }
@@ -451,7 +452,7 @@ void dio_get_uint32(struct data_in *din, int *dest)
     if (dest) {
       uint32_t x;
 
-      assert(sizeof(x) == 4);
+      fc_assert(sizeof(x) == 4);
       memcpy(&x, ADD_TO_POINTER(din->src, din->current), 4);
       *dest = ntohl(x);
     }
@@ -471,7 +472,7 @@ void dio_get_bool8(struct data_in *din, bool * dest)
   dio_get_uint8(din, &ival);
 
   if (ival != 0 && ival != 1) {
-    freelog(LOG_ERROR, "Received value isn't boolean: %d", ival);
+    log_error("Received value isn't boolean: %d", ival);
     ival = 1;
   }
 
@@ -488,7 +489,7 @@ void dio_get_bool32(struct data_in *din, bool * dest)
   dio_get_uint32(din, &ival);
 
   if (ival != 0 && ival != 1) {
-    freelog(LOG_ERROR, "Received value isn't boolean: %d", ival);
+    log_error("Received value isn't boolean: %d", ival);
     ival = 1;
   }
 
@@ -549,7 +550,7 @@ void dio_get_string(struct data_in *din, char *dest, size_t max_dest_size)
   size_t ps_len;		/* length in packet, not including null */
   size_t offset, remaining;
 
-  assert(max_dest_size > 0 || dest == NULL);
+  fc_assert(max_dest_size > 0 || dest == NULL);
 
   if (!enough_data(din, 1)) {
     dest[0] = '\0';
@@ -590,7 +591,7 @@ void dio_get_bit_string(struct data_in *din, char *dest,
   int npack = 0;		/* number claimed in packet */
   int i;			/* iterate the bytes */
 
-  assert(dest != NULL && max_dest_size > 0);
+  fc_assert(dest != NULL && max_dest_size > 0);
 
   if (!enough_data(din, 1)) {
     dest[0] = '\0';
@@ -599,8 +600,8 @@ void dio_get_bit_string(struct data_in *din, char *dest,
 
   dio_get_uint16(din, &npack);
   if (npack >= max_dest_size) {
-      freelog(LOG_NORMAL, "Have size for %lu, got %d",
-              (unsigned long)max_dest_size, npack);
+      log_error("Have size for %lu, got %d",
+                (unsigned long) max_dest_size, npack);
     din->bad_bit_string = TRUE;
     dest[0] = '\0';
     return;
@@ -650,21 +651,19 @@ void dio_get_tech_list(struct data_in *din, int *dest)
 **************************************************************************/
 void dio_get_worklist(struct data_in *din, struct worklist *pwl)
 {
-  dio_get_bool8(din, &pwl->is_valid);
+  int i, length;
 
-  if (pwl->is_valid) {
-    int i, length;
+  worklist_init(pwl);
 
-    worklist_init(pwl);
+  dio_get_uint8(din, &length);
+  for (i = 0; i < length; i++) {
+    int identifier;
+    int kind;
 
-    dio_get_uint8(din, &length);
-    for (i = 0; i < length; i++) {
-      struct city_production prod;
+    dio_get_uint8(din, &kind);
+    dio_get_uint8(din, &identifier);
 
-      dio_get_bool8(din, &prod.is_unit);
-      dio_get_uint8(din, &prod.value);
-      worklist_append(pwl, prod);
-    }
+    worklist_append(pwl, universal_by_number(kind, identifier));
   }
 }
 
@@ -706,40 +705,6 @@ void dio_get_uint16_vec8(struct data_in *din, int **values, int stop_value)
   }
 }
 
-/**************************************************************************
-  De-serialize a player diplomatic state.
-**************************************************************************/
-void dio_get_diplstate(struct data_in *din, struct player_diplstate *pds)
-{
-  int value = 0;
-
-  /* backward compatible order defined for this transaction */
-  dio_get_uint8(din, &value);
-  pds->type = value;
-  dio_get_uint16(din, &pds->turns_left);
-  dio_get_uint16(din, &pds->contact_turns_left);
-  dio_get_uint8(din, &pds->has_reason_to_cancel);
-  dio_get_uint16(din, &pds->first_contact_turn);
-  value = 0;
-  dio_get_uint8(din, &value);
-  pds->max_state = value;
-}
-
-/**************************************************************************
-  Serialize a player diplomatic state.
-**************************************************************************/
-void dio_put_diplstate(struct data_out *dout,
-		       const struct player_diplstate *pds)
-{
-  /* backward compatible order defined for this transaction */
-  dio_put_uint8(dout, pds->type);
-  dio_put_uint16(dout, pds->turns_left);
-  dio_put_uint16(dout, pds->contact_turns_left);
-  dio_put_uint8(dout, pds->has_reason_to_cancel);
-  dio_put_uint16(dout, pds->first_contact_turn);
-  dio_put_uint8(dout, pds->max_state);
-}
-
 /**************************************************************************
   De-serialize a requirement.
 **************************************************************************/
