@@ -20,10 +20,10 @@
 # have to call delta_stats_report to get these.
 generate_stats=0
 
-# generate_freelogs will generate freelog calls to debug the delta code.
-generate_freelogs=0
-freelog_log_level="LOG_PACKET"
-generate_variant_freelogs=0
+# generate_logs will generate log calls to debug the delta code.
+generate_logs=1
+use_log_macro="log_packet"
+generate_variant_logs=1
 
 ### The following parameters CHANGE the protocol. You have been warned.
 fold_bool_into_header=1
@@ -217,15 +217,13 @@ class Field:
         result.update(vars)
         return result
 
-    def get_handle_type(self,const=0):
+    def get_handle_type(self):
         if self.dataio_type=="string":
-            if const: return "const char *"
-            else:     return "char *"
-            
+            return "const char *"
         if self.dataio_type=="worklist":
             return "const %s *"%self.struct_type
         if self.is_array:
-            return "%s *"%self.struct_type
+            return "const %s *"%self.struct_type
         return self.struct_type+" "
 
     # Returns code which is used in the declaration of the field in
@@ -305,13 +303,23 @@ class Field:
         if fold_bool_into_header and self.struct_type=="bool" and \
            not self.is_array:
             b="packet->%(name)s"%self.get_dict(vars())
-        else:
-            b="differ"
-        return '''%s
-  if(differ) {different++;}
-  if(%s) {BV_SET(fields, %d);}
+            return '''%s
+  if(differ) {
+    different++;
+  }
+  if (%s) {
+    BV_SET(fields, %d);
+  }
 
 '''%(cmp,b,i)
+        else:
+            return '''%s
+  if (differ) {
+    different++;
+    BV_SET(fields, %d);
+  }
+
+'''%(cmp,i)
 
     # Returns a code fragement which will put this field if the
     # content has changed. Does nothing for bools-in-header.    
@@ -321,9 +329,9 @@ class Field:
             return "  /* field %(i)d is folded into the header */\n"%vars()
         put=self.get_put()
         packet_name=packet.name
-        log_level=packet.log_level
-        if packet.gen_freelog:
-            f='    freelog(%(log_level)s, "  field \'%(name)s\' has changed");\n'%self.get_dict(vars())
+        log_macro=packet.log_macro
+        if packet.gen_log:
+            f='    %(log_macro)s("  field \'%(name)s\' has changed");\n'%self.get_dict(vars())
         else:
             f=""
         if packet.gen_stats:
@@ -354,18 +362,39 @@ class Field:
            (self.dataio_type not in arr_types and self.is_array==0):
             return "  dio_put_%(dataio_type)s(&dout, real_packet->%(name)s);"%self.__dict__
         if self.is_struct:
-            c="dio_put_%(dataio_type)s(&dout, &real_packet->%(name)s[i]);"%self.__dict__
+            if self.is_array==2:
+                c="dio_put_%(dataio_type)s(&dout, &real_packet->%(name)s[i][j]);"%self.__dict__
+            else:
+                c="dio_put_%(dataio_type)s(&dout, &real_packet->%(name)s[i]);"%self.__dict__
         elif self.dataio_type=="string":
             c="dio_put_%(dataio_type)s(&dout, real_packet->%(name)s[i]);"%self.__dict__
             array_size_u=self.array_size1_u
 
         elif self.struct_type=="float":
-            c="  dio_put_uint32(&dout, (int)(real_packet->%(name)s[i] * %(float_factor)d));"%self.__dict__
+            if self.is_array==2:
+                c="  dio_put_uint32(&dout, (int)(real_packet->%(name)s[i][j] * %(float_factor)d));"%self.__dict__
+            else:
+                c="  dio_put_uint32(&dout, (int)(real_packet->%(name)s[i] * %(float_factor)d));"%self.__dict__
         else:
-            c="dio_put_%(dataio_type)s(&dout, real_packet->%(name)s[i]);"%self.__dict__
+            if self.is_array==2:
+                c="dio_put_%(dataio_type)s(&dout, real_packet->%(name)s[i][j]);"%self.__dict__
+            else:
+                c="dio_put_%(dataio_type)s(&dout, real_packet->%(name)s[i]);"%self.__dict__
 
         if not self.diff:
-            return '''
+            if self.is_array==2 and self.dataio_type!="string":
+                return '''
+    {
+      int i, j;
+
+      for (i = 0; i < %(array_size1_u)s; i++) {
+        for (j = 0; j < %(array_size2_u)s; j++) {
+          %(c)s
+        }
+      }
+    } '''%self.get_dict(vars())
+            else:
+                return '''
     {
       int i;
 
@@ -378,7 +407,7 @@ class Field:
     {
       int i;
 
-      assert(%(array_size_u)s < 255);
+      fc_assert(%(array_size_u)s < 255);
 
       for (i = 0; i < %(array_size_u)s; i++) {
         if(old->%(name)s[i] != real_packet->%(name)s[i]) {
@@ -397,9 +426,9 @@ class Field:
            not self.is_array:
             return  "  real_packet->%(name)s = BV_ISSET(fields, %(i)d);\n"%self.get_dict(vars())
         get=prefix("    ",get)
-        log_level=packet.log_level
-        if packet.gen_freelog:
-            f="    freelog(%(log_level)s, \"  got field '%(name)s'\");\n"%self.get_dict(vars())
+        log_macro=packet.log_macro
+        if packet.gen_log:
+            f="    %(log_macro)s(\"  got field '%(name)s'\");\n"%self.get_dict(vars())
         else:
             f=""
         return '''  if (BV_ISSET(fields, %(i)d)) {
@@ -437,16 +466,35 @@ class Field:
 }'''%self.__dict__
 
         if self.is_struct:
-            c="dio_get_%(dataio_type)s(&din, &real_packet->%(name)s[i]);"%self.__dict__
+            if self.is_array==2:
+                c="dio_get_%(dataio_type)s(&din, &real_packet->%(name)s[i][j]);"%self.__dict__
+            else:
+                c="dio_get_%(dataio_type)s(&din, &real_packet->%(name)s[i]);"%self.__dict__
         elif self.dataio_type=="string":
             c="dio_get_%(dataio_type)s(&din, real_packet->%(name)s[i], sizeof(real_packet->%(name)s[i]));"%self.__dict__
         elif self.struct_type=="float":
-            c='''int tmp;
+            if self.is_array==2:
+                c='''int tmp;
+
+    dio_get_uint32(&din, &tmp);
+    real_packet->%(name)s[i][j] = (float)(tmp) / %(float_factor)d.0;'''%self.__dict__
+            else:
+                c='''int tmp;
 
     dio_get_uint32(&din, &tmp);
     real_packet->%(name)s[i] = (float)(tmp) / %(float_factor)d.0;'''%self.__dict__
         elif self.struct_type=="bool":
-            c="dio_get_%(dataio_type)s(&din, &real_packet->%(name)s[i]);"%self.__dict__
+            if self.is_array==2:
+                c="dio_get_%(dataio_type)s(&din, &real_packet->%(name)s[i][j]);"%self.__dict__
+            else:
+                c="dio_get_%(dataio_type)s(&din, &real_packet->%(name)s[i]);"%self.__dict__
+        elif self.is_array==2:
+            c='''{
+  int readin;
+
+  dio_get_%(dataio_type)s(&din, &readin);
+  real_packet->%(name)s[i][j] = readin;
+}'''%self.__dict__
         else:
             c='''{
   int readin;
@@ -465,7 +513,7 @@ class Field:
             if array_size_u != array_size_d:
                 extra='''
   if(%(array_size_u)s > %(array_size_d)s) {
-    freelog(LOG_ERROR, "packets_gen.c: WARNING: truncation array");
+    log_error("packets_gen.c: WARNING: truncation array");
     %(array_size_u)s = %(array_size_d)s;
   }'''%self.get_dict(vars())
             else:
@@ -473,7 +521,19 @@ class Field:
             if self.dataio_type=="memory":
                 return '''%(extra)s
   dio_get_%(dataio_type)s(&din, real_packet->%(name)s, %(array_size_u)s);'''%self.get_dict(vars())
-            return '''
+            elif self.is_array==2 and self.dataio_type!="string":
+                return '''
+{
+  int i, j;
+%(extra)s
+  for (i = 0; i < %(array_size1_u)s; i++) {
+    for (j = 0; j < %(array_size2_u)s; j++) {
+      %(c)s
+    }
+  }
+}'''%self.get_dict(vars())
+            else:
+                return '''
 {
   int i;
 %(extra)s
@@ -491,7 +551,7 @@ for (;;) {
     break;
   }
   if(i > %(array_size_u)s) {
-    freelog(LOG_ERROR, "packets_gen.c: WARNING: ignoring intra array diff");
+    log_error("packets_gen.c: WARNING: ignoring intra array diff");
   } else {
     %(c)s
   }
@@ -502,9 +562,9 @@ for (;;) {
 # Class which represents a capability variant.
 class Variant:
     def __init__(self,poscaps,negcaps,name,fields,packet,no):
-        self.log_level=freelog_log_level
+        self.log_macro=use_log_macro
         self.gen_stats=generate_stats
-        self.gen_freelog=generate_freelogs
+        self.gen_log=generate_logs
         self.name=name
         self.packet_name=packet.name
         self.fields=fields
@@ -516,8 +576,10 @@ class Variant:
         self.want_post_send=packet.want_post_send
         self.type=packet.type
         self.delta=packet.delta
-        self.is_action=packet.is_action
-        
+        self.is_info=packet.is_info
+        self.cancel=packet.cancel
+        self.want_force=packet.want_force
+
         self.poscaps=poscaps
         self.negcaps=negcaps
         if self.poscaps or self.negcaps:
@@ -544,11 +606,10 @@ class Variant:
         if len(self.fields)>5 or string.split(self.name,"_")[1]=="ruleset":
             self.handle_via_packet=1
 
-        self.extra_force_arg=""
         self.extra_send_args=""
         self.extra_send_args2=""
         self.extra_send_args3=string.join(
-            map(lambda x:"%s%s"%(x.get_handle_type(1), x.name),
+            map(lambda x:"%s%s"%(x.get_handle_type(), x.name),
                 self.fields),", ")
         if self.extra_send_args3:
             self.extra_send_args3=", "+self.extra_send_args3
@@ -556,10 +617,14 @@ class Variant:
         if not self.no_packet:
             self.extra_send_args=', const struct %(packet_name)s *packet'%self.__dict__+self.extra_send_args
             self.extra_send_args2=', packet'+self.extra_send_args2
-        if not self.is_action:
-            self.extra_force_arg=', bool force_send'
+
+        if self.want_force:
+            self.extra_send_args=self.extra_send_args+', bool force_to_send'
+            self.extra_send_args2=self.extra_send_args2+', force_to_send'
+            self.extra_send_args3=self.extra_send_args3+', bool force_to_send'
+
         self.receive_prototype='static struct %(packet_name)s *receive_%(name)s(struct connection *pc, enum packet_type type)'%self.__dict__
-        self.send_prototype='static int send_%(name)s(struct connection *pc%(extra_force_arg)s%(extra_send_args)s)'%self.__dict__
+        self.send_prototype='static int send_%(name)s(struct connection *pc%(extra_send_args)s)'%self.__dict__
 
     # See Field.get_dict
     def get_dict(self,vars):
@@ -592,11 +657,11 @@ static char *stats_%(name)s_names[] = {%(names)s};
         return '''
   if (stats_%(name)s_sent > 0 &&
       stats_%(name)s_discarded != stats_%(name)s_sent) {
-    freelog(LOG_TEST, \"%(name)s %%d out of %%d got discarded\",
+    log_test(\"%(name)s %%d out of %%d got discarded\",
       stats_%(name)s_discarded, stats_%(name)s_sent);
     for (i = 0; i < %(bits)d; i++) {
       if(stats_%(name)s_counters[i] > 0) {
-        freelog(LOG_TEST, \"  %%4d / %%4d: %%2d = %%s\",
+        log_test(\"  %%4d / %%4d: %%2d = %%s\",
           stats_%(name)s_counters[i],
           (stats_%(name)s_sent - stats_%(name)s_discarded),
           i, stats_%(name)s_names[i]);
@@ -621,7 +686,7 @@ static char *stats_%(name)s_names[] = {%(names)s};
         if len(self.key_fields)==0:
             return "#define hash_%(name)s hash_const\n\n"%self.__dict__
         else:
-            intro='''static unsigned int hash_%(name)s(const void *vkey, unsigned int num_buckets)
+            intro='''static genhash_val_t hash_%(name)s(const void *vkey, size_t num_buckets)
 {
 '''%self.__dict__
 
@@ -647,23 +712,18 @@ static char *stats_%(name)s_names[] = {%(names)s};
         if len(self.key_fields)==0:
             return "#define cmp_%(name)s cmp_const\n\n"%self.__dict__
         else:
-            intro='''static int cmp_%(name)s(const void *vkey1, const void *vkey2)
+            intro='''static bool cmp_%(name)s(const void *vkey1, const void *vkey2)
 {
 '''%self.__dict__
             body=""
             body=body+'''  const struct %(packet_name)s *key1 = (const struct %(packet_name)s *) vkey1;
   const struct %(packet_name)s *key2 = (const struct %(packet_name)s *) vkey2;
-  int diff;
 
 '''%self.__dict__
             for field in self.key_fields:
-                body=body+'''  diff = key1->%s - key2->%s;
-  if (diff != 0) {
-    return diff;
-  }
-
+                body=body+'''  return key1->%s == key2->%s;
 '''%(field.name,field.name)
-            extro="  return 0;\n}\n\n"
+            extro="}\n\n"
             return intro+body+extro
 
     # Returns a code fragement which is the implementation of the send
@@ -673,7 +733,7 @@ static char *stats_%(name)s_names[] = {%(names)s};
         temp='''%(send_prototype)s
 {
 <real_packet1><delta_header>  SEND_PACKET_START(%(type)s);
-<freelog><report><pre1><body><pre2>  <post>SEND_PACKET_END;
+<log><report><pre1><body><pre2>  <post>SEND_PACKET_END;
 }
 
 '''
@@ -684,10 +744,10 @@ static char *stats_%(name)s_names[] = {%(names)s};
 '''
         else:
             report=""
-        if self.gen_freelog:
-            freelog='\n  freelog(%(log_level)s, "%(name)s: sending info about (%(keys_format)s)"%(keys_arg)s);\n'
+        if self.gen_log:
+            log='\n  %(log_macro)s("%(name)s: sending info about (%(keys_format)s)"%(keys_arg)s);\n'
         else:
-            freelog=""
+            log=""
         if self.want_pre_send:
             pre1='''
   {
@@ -714,16 +774,16 @@ static char *stats_%(name)s_names[] = {%(names)s};
 
         if not self.no_packet:
             if self.delta:
-                body=self.get_delta_send_body()
-                if self.is_action:
-                    force_send_var="\n  bool force_send = TRUE;"
+                if self.want_force:
+                    diff='force_to_send'
                 else:
-                    force_send_var=""
+                    diff='0'
+                body=self.get_delta_send_body()
                 delta_header='''  %(name)s_fields fields;
-  struct %(packet_name)s *old, *clone;
-  bool differ, old_from_hash;%(force_send_var)s
-  struct hash_table **hash = &pc->phs.sent[%(type)s];
-  int different = 0;
+  struct %(packet_name)s *old;
+  bool differ;
+  struct genhash **hash = pc->phs.sent + %(type)s;
+  int different = %(diff)s;
 '''
             else:
                 body="\n"
@@ -754,17 +814,18 @@ static char *stats_%(name)s_names[] = {%(names)s};
     # Helper for get_send()
     def get_delta_send_body(self):
         intro='''
-  if (!*hash) {
-    *hash = hash_new(hash_%(name)s, cmp_%(name)s);
+  if (NULL == *hash) {
+    *hash = genhash_new_full(hash_%(name)s, cmp_%(name)s,
+                             NULL, NULL, NULL, free);
   }
   BV_CLR_ALL(fields);
 
-  old = hash_lookup_data(*hash, real_packet);
-  old_from_hash = (old != NULL);
-  if (!old) {
+  if (!genhash_lookup(*hash, real_packet, (void **) &old)) {
     old = fc_malloc(sizeof(*old));
+    *old = *real_packet;
+    genhash_insert(*hash, old, old);
     memset(old, 0, sizeof(*old));
-    force_send = TRUE;
+    different = 1;      /* Force to send. */
   }
 
 '''
@@ -772,21 +833,26 @@ static char *stats_%(name)s_names[] = {%(names)s};
         for i in range(len(self.other_fields)):
             field=self.other_fields[i]
             body=body+field.get_cmp_wrapper(i)
-        if self.gen_freelog:
-            fl='    freelog(%(log_level)s, "  no change -> discard");\n'
+        if self.gen_log:
+            fl='    %(log_macro)s("  no change -> discard");\n'
         else:
             fl=""
         if self.gen_stats:
             s='    stats_%(name)s_discarded++;\n'
         else:
             s=""
-        body=body+'''  if (different == 0 && !force_send) {
+
+        if self.is_info != "no":
+            body=body+'''
+  if (different == 0) {
 %(fl)s%(s)s<pre2>    return 0;
   }
-
-  DIO_BV_PUT(&dout, fields);
 '''%self.get_dict(vars())
 
+        body=body+'''
+  DIO_BV_PUT(&dout, fields);
+'''
+
         for field in self.key_fields:
             body=body+field.get_put()+"\n"
         body=body+"\n"
@@ -795,16 +861,18 @@ static char *stats_%(name)s_names[] = {%(names)s};
             field=self.other_fields[i]
             body=body+field.get_put_wrapper(self,i)
         body=body+'''
-
-  if (old_from_hash) {
-    hash_delete_entry(*hash, old);
-  }
-
-  clone = old;
-
-  *clone = *real_packet;
-  hash_insert(*hash, clone, clone);
+  *old = *real_packet;
 '''
+
+        # Cancel some is-info packets.
+        for i in self.cancel:
+            body=body+'''
+  hash = pc->phs.sent + %s;
+  if (NULL != *hash) {
+    genhash_remove(*hash, real_packet);
+  }
+'''%i
+
         return intro+body
 
     # Returns a code fragement which is the implementation of the receive
@@ -814,15 +882,14 @@ static char *stats_%(name)s_names[] = {%(names)s};
         temp='''%(receive_prototype)s
 {
 <delta_header>  RECEIVE_PACKET_START(%(packet_name)s, real_packet);
-<delta_body1><body1><freelog><body2><post>  RECEIVE_PACKET_END(real_packet);
+<delta_body1><body1><log><body2><post>  RECEIVE_PACKET_END(real_packet);
 }
 
 '''
         if self.delta:
             delta_header='''  %(name)s_fields fields;
   struct %(packet_name)s *old;
-  struct hash_table **hash = &pc->phs.received[type];
-  struct %(packet_name)s *clone;
+  struct genhash **hash = pc->phs.received + type;
 '''
             delta_body1="\n  DIO_BV_GET(&din, fields);\n"
             body1=""
@@ -839,10 +906,10 @@ static char *stats_%(name)s_names[] = {%(names)s};
             body1=body1+"\n"
             body2=""
 
-        if self.gen_freelog:
-            freelog='  freelog(%(log_level)s, "%(name)s: got info about (%(keys_format)s)"%(keys_arg)s);\n'
+        if self.gen_log:
+            log='  %(log_macro)s("%(name)s: got info about (%(keys_format)s)"%(keys_arg)s);\n'
         else:
-            freelog=""
+            log=""
         
         if self.want_post_recv:
             post="  post_receive_%(packet_name)s(pc, real_packet);\n"
@@ -863,17 +930,17 @@ static char *stats_%(name)s_names[] = {%(names)s};
         key2=string.join(key2,"\n")
         if key1: key1=key1+"\n\n"
         if key2: key2="\n\n"+key2
-        if self.gen_freelog:
-            fl='    freelog(%(log_level)s, "  no old info");\n'
+        if self.gen_log:
+            fl='    %(log_macro)s("  no old info");\n'
         else:
             fl=""
         body='''
-  if (!*hash) {
-    *hash = hash_new(hash_%(name)s, cmp_%(name)s);
+  if (NULL == *hash) {
+    *hash = genhash_new_full(hash_%(name)s, cmp_%(name)s,
+                             NULL, NULL, NULL, free);
   }
-  old = hash_delete_entry(*hash, real_packet);
 
-  if (old) {
+  if (genhash_lookup(*hash, real_packet, (void **) &old)) {
     *real_packet = *old;
   } else {
 %(key1)s%(fl)s    memset(real_packet, 0, sizeof(*real_packet));%(key2)s
@@ -885,24 +952,34 @@ static char *stats_%(name)s_names[] = {%(names)s};
             body=body+field.get_get_wrapper(self,i)
 
         extro='''
-  clone = fc_malloc(sizeof(*clone));
-  *clone = *real_packet;
-  if (old) {
-    free(old);
+  if (NULL == old) {
+    old = fc_malloc(sizeof(*old));
+    *old = *real_packet;
+    genhash_insert(*hash, old, old);
+  } else {
+    *old = *real_packet;
   }
-  hash_insert(*hash, clone, clone);
-
 '''%self.get_dict(vars())
+
+        # Cancel some is-info packets.
+        for i in self.cancel:
+            extro=extro+'''
+  hash = pc->phs.received + %s;
+  if (NULL != *hash) {
+    genhash_remove(*hash, real_packet);
+  }
+'''%i
+
         return body+extro
-    
+
 
 # Class which represents a packet. A packet contains a list of fields.
 class Packet:
     def __init__(self,str, types):
         self.types=types
-        self.log_level=freelog_log_level
+        self.log_macro=use_log_macro
         self.gen_stats=generate_stats
-        self.gen_freelog=generate_freelogs
+        self.gen_log=generate_logs
         str=string.strip(str)
         lines=string.split(str,"\n")
         
@@ -929,9 +1006,17 @@ class Packet:
             arr.remove("cs")
         assert len(self.dirs)>0,repr(self.name)+repr(self.dirs)
 
-        self.is_action="is-info" not in arr
-        if not self.is_action: arr.remove("is-info")
-        
+        # "no" means normal packet
+        # "yes" means is-info packet
+        # "game" means is-game-info packet
+        self.is_info="no"
+        if "is-info" in arr:
+            self.is_info="yes"
+            arr.remove("is-info")
+        if "is-game-info" in arr:
+            self.is_info="game"
+            arr.remove("is-game-info")
+
         self.want_pre_send="pre-send" in arr
         if self.want_pre_send: arr.remove("pre-send")
         
@@ -962,8 +1047,22 @@ class Packet:
         self.want_lsend="lsend" in arr
         if self.want_lsend: arr.remove("lsend")
 
+        self.want_force="force" in arr
+        if self.want_force: arr.remove("force")
+
+        self.cancel=[]
+        removes=[]
+        remaining=[]
+        for i in arr:
+            mo=re.search("^cancel\((.*)\)$",i)
+            if mo:
+                self.cancel.append(mo.group(1))
+                continue
+            remaining.append(i)
+        arr=remaining
+
         assert len(arr)==0,repr(arr)
-        
+
         if disable_delta:
             self.delta=0
 
@@ -993,7 +1092,7 @@ class Packet:
         self.extra_send_args=""
         self.extra_send_args2=""
         self.extra_send_args3=string.join(
-            map(lambda x:"%s%s"%(x.get_handle_type(1), x.name),
+            map(lambda x:"%s%s"%(x.get_handle_type(), x.name),
                 self.fields),", ")
         if self.extra_send_args3:
             self.extra_send_args3=", "+self.extra_send_args3
@@ -1002,17 +1101,17 @@ class Packet:
             self.extra_send_args=', const struct %(name)s *packet'%self.__dict__+self.extra_send_args
             self.extra_send_args2=', packet'+self.extra_send_args2
 
+        if self.want_force:
+            self.extra_send_args=self.extra_send_args+', bool force_to_send'
+            self.extra_send_args2=self.extra_send_args2+', force_to_send'
+            self.extra_send_args3=self.extra_send_args3+', bool force_to_send'
+
         self.receive_prototype='struct %(name)s *receive_%(name)s(struct connection *pc, enum packet_type type)'%self.__dict__
-        self.force_arg=""
-        self.force_value=""
-        if not self.is_action:
-            self.force_arg=", bool force_send"
-            self.force_value=", force_send"
-        self.send_prototype='int send_%(name)s(struct connection *pc%(force_arg)s%(extra_send_args)s)'%self.__dict__
+        self.send_prototype='int send_%(name)s(struct connection *pc%(extra_send_args)s)'%self.__dict__
         if self.want_lsend:
-            self.lsend_prototype='void lsend_%(name)s(struct conn_list *dest%(force_arg)s%(extra_send_args)s)'%self.__dict__
+            self.lsend_prototype='void lsend_%(name)s(struct conn_list *dest%(extra_send_args)s)'%self.__dict__
         if self.want_dsend:
-            self.dsend_prototype='int dsend_%(name)s(struct connection *pc%(force_arg)s%(extra_send_args3)s)'%self.__dict__
+            self.dsend_prototype='int dsend_%(name)s(struct connection *pc%(extra_send_args3)s)'%self.__dict__
             if self.want_lsend:
                 self.dlsend_prototype='void dlsend_%(name)s(struct conn_list *dest%(extra_send_args3)s)'%self.__dict__
 
@@ -1091,12 +1190,13 @@ class Packet:
             name2=v.name
             no=v.no
             result=result+'  } else if(%(cond)s) {\n    variant = %(no)s;\n'%self.get_dict(vars())
-        if generate_variant_freelogs and len(self.variants)>1:
-            log='  freelog(LOG_TEST, "%(name)s: using variant=%%d cap=%%s", variant, pc->capability);\n'%self.get_dict(vars())
+        if generate_variant_logs and len(self.variants)>1:
+            log='  log_test("%(name)s: using variant=%%d cap=%%s", variant, pc->capability);\n'%self.get_dict(vars())
         else:
             log=""
         result=result+'''  } else {
-    die("unknown variant");
+    log_error("Unknown %(type)s variant for connection %%s", conn_description(pc));
+    variant = -2;       /* Keep something invalid. */
   }
 %(log)s  pc->phs.variant[%(type)s] = variant;
 }
@@ -1112,12 +1212,12 @@ class Packet:
         only_server=len(self.dirs)==1 and self.dirs[0]=="cs"
         if only_client:
             restrict='''  if (is_server()) {
-    freelog(LOG_ERROR, "Receiving %(name)s at the server.");
+    log_error("Receiving %(name)s at the server.");
   }
 '''%self.get_dict(vars())
         elif only_server:
             restrict='''  if (!is_server()) {
-    freelog(LOG_ERROR, "Receiving %(name)s at the client.");
+    log_error("Receiving %(name)s at the client.");
   }
 '''%self.get_dict(vars())
         else:
@@ -1126,21 +1226,27 @@ class Packet:
         result='''%(receive_prototype)s
 {
   if(!pc->used) {
-    freelog(LOG_ERROR,
-	    "WARNING: trying to read data from the closed connection %%s",
-	    conn_description(pc));
+    log_error("WARNING: trying to read data from the closed connection %%s",
+              conn_description(pc));
     return NULL;
   }
-  assert(pc->phs.variant != NULL);
+  fc_assert_ret_val(NULL != pc->phs.variant, NULL);
 %(restrict)s  ensure_valid_variant_%(name)s(pc);
 
-  switch(pc->phs.variant[%(type)s]) {
-'''%self.get_dict(vars())
+  switch(pc->phs.variant[%(type)s]) {'''%self.get_dict(vars())
         for v in self.variants:
             name2=v.name
             no=v.no
-            result=result+'    case %(no)s: return receive_%(name2)s(pc, type);\n'%self.get_dict(vars())
-        result=result+'    default: die("unknown variant"); return NULL;\n  }\n}\n\n'
+            result=result+'''
+  case %(no)s:
+    return receive_%(name2)s(pc, type);'''%self.get_dict(vars())
+        result=result+'''
+  default:
+    log_debug("Unknown %(type)s variant for connection %%s", conn_description(pc));
+    return NULL;
+  }
+}
+'''%self.get_dict(vars())
         return result
 
     def get_send(self):
@@ -1148,12 +1254,12 @@ class Packet:
         only_server=len(self.dirs)==1 and self.dirs[0]=="sc"
         if only_client:
             restrict='''  if (is_server()) {
-    freelog(LOG_ERROR, "Sending %(name)s from the server.");
+    log_error("Sending %(name)s from the server.");
   }
 '''%self.get_dict(vars())
         elif only_server:
             restrict='''  if (!is_server()) {
-    freelog(LOG_ERROR, "Sending %(name)s from the client.");
+    log_error("Sending %(name)s from the client.");
   }
 '''%self.get_dict(vars())
         else:
@@ -1162,26 +1268,36 @@ class Packet:
         result='''%(send_prototype)s
 {
   if(!pc->used) {
-    freelog(LOG_ERROR,
-	    "WARNING: trying to send data to the closed connection %%s",
-	    conn_description(pc));
+    log_error("WARNING: trying to send data to the closed connection %%s",
+              conn_description(pc));
     return -1;
   }
-  assert(pc->phs.variant != NULL);
+  fc_assert_ret_val(NULL != pc->phs.variant, -1);
 %(restrict)s  ensure_valid_variant_%(name)s(pc);
 
   switch(pc->phs.variant[%(type)s]) {
 '''%self.get_dict(vars())
         args="pc"
-        if not self.is_action:
-            args=args+', force_send'
         if not self.no_packet:
-            args=args+', packet'
+            args=args+", packet"
+        if self.want_force:
+            args=args+", force_to_send"
         for v in self.variants:
             name2=v.name
             no=v.no
-            result=result+'    case %(no)s: return send_%(name2)s(%(args)s);\n'%self.get_dict(vars())
-        result=result+'    default: die("unknown variant"); return -1;\n  }\n}\n\n'
+
+
+
+            result=result+'''
+  case %(no)s:
+    return send_%(name2)s(%(args)s);'''%self.get_dict(vars())
+        result=result+'''
+  default:
+    log_debug("Unknown %(type)s variant for connection %%s", conn_description(pc));
+    return -1;
+  }
+}
+'''%self.get_dict(vars())
         return result
 
     def get_variants(self):
@@ -1203,7 +1319,7 @@ class Packet:
         return '''%(lsend_prototype)s
 {
   conn_list_iterate(dest, pconn) {
-    send_%(name)s(pconn%(force_value)s%(extra_send_args2)s);
+    send_%(name)s(pconn%(extra_send_args2)s);
   } conn_list_iterate_end;
 }
 
@@ -1220,7 +1336,7 @@ class Packet:
 
 %(fill)s
   
-  return send_%(name)s(pc, real_packet%(force_value)s);
+  return send_%(name)s(pc, real_packet);
 }
 
 '''%self.get_dict(vars())
@@ -1279,15 +1395,15 @@ void delta_stats_reset(void) {
 def get_get_packet_helper(packets):
     intro='''void *get_packet_from_connection_helper(struct connection *pc,\n    enum packet_type type)
 {
-  switch(type) {
+  switch (type) {
 
 '''
     body=""
     for p in packets:
         body=body+"  case %(type)s:\n    return receive_%(name)s(pc, type);\n\n"%p.__dict__
     extro='''  default:
-    freelog(LOG_ERROR, "unknown packet type %d received from %s",
-	    type, conn_description(pc));
+    log_error("unknown packet type %d received from %s",
+              type, conn_description(pc));
     remove_packet_from_buffer(pc->buffer);
     return NULL;
   };
@@ -1297,11 +1413,11 @@ def get_get_packet_helper(packets):
     return intro+body+extro
 
 # Returns a code fragement which is the implementation of the
-# get_packet_name() function.
-def get_get_packet_name(packets):
-    intro='''const char *get_packet_name(enum packet_type type)
+# packet_name() function.
+def get_packet_name(packets):
+    intro='''const char *packet_name(enum packet_type type)
 {
-  switch(type) {
+  switch (type) {
 
 '''
     body=""
@@ -1312,6 +1428,29 @@ def get_get_packet_name(packets):
   }
 }
 
+'''
+    return intro+body+extro
+
+# Returns a code fragement which is the implementation of the
+# packet_has_game_info_flag() function.
+def get_packet_has_game_info_flag(packets):
+    intro='''bool packet_has_game_info_flag(enum packet_type type)
+{
+  switch (type) {
+
+'''
+    body=""
+    for p in packets:
+        body=body+'  case %(type)s:\n'%p.__dict__
+        if p.is_info != "game":
+            body=body+'    return FALSE;\n\n'
+        else:
+            body=body+'    return TRUE;\n\n'
+    extro='''  default:
+    return FALSE;
+  }
+}
+
 '''
     return intro+body+extro
 
@@ -1435,12 +1574,12 @@ void *get_packet_from_connection_helper(struct connection *pc, enum packet_type
 #include <config.h>
 #endif
 
-#include <assert.h>
 #include <string.h>
 
 /* utility */
+#include "bitvector.h"
 #include "capability.h"
-#include "hash.h"
+#include "genhash.h"
 #include "log.h"
 #include "mem.h"
 #include "support.h"
@@ -1453,14 +1592,14 @@ void *get_packet_from_connection_helper(struct connection *pc, enum packet_type
 
 #include "packets.h"
 
-static unsigned int hash_const(const void *vkey, unsigned int num_buckets)
+static genhash_val_t hash_const(const void *vkey, size_t num_buckets)
 {
   return 0;
 }
 
-static int cmp_const(const void *vkey1, const void *vkey2)
+static bool cmp_const(const void *vkey1, const void *vkey2)
 {
-  return 0;
+  return TRUE;
 }
 
 ''')
@@ -1480,7 +1619,8 @@ static int stats_total_sent;
     output_c.write(get_reset(packets))
 
     output_c.write(get_get_packet_helper(packets))
-    output_c.write(get_get_packet_name(packets))
+    output_c.write(get_packet_name(packets))
+    output_c.write(get_packet_has_game_info_flag(packets))
 
     # write hash, cmp, send, receive
     for p in packets:
@@ -1509,15 +1649,17 @@ static int stats_total_sent;
 #ifndef FC__HAND_GEN_H
 #define FC__HAND_GEN_H
 
+/* utility */
 #include "shared.h"
 
+/* common */
 #include "fc_types.h"
 #include "packets.h"
 
 struct connection;
 
-bool server_handle_packet(enum packet_type type, void *packet,
-			  struct player *pplayer, struct connection *pconn);
+bool server_handle_packet(enum packet_type type, const void *packet,
+                          struct player *pplayer, struct connection *pconn);
 
 ''')
     
@@ -1533,9 +1675,9 @@ bool server_handle_packet(enum packet_type type, void *packet,
             if p.handle_via_packet:
                 f.write('struct %s;\n'%p.name)
                 if p.handle_per_conn:
-                    f.write('void handle_%s(struct connection *pc, struct %s *packet);\n'%(a,p.name))
+                    f.write('void handle_%s(struct connection *pc, const struct %s *packet);\n'%(a,p.name))
                 else:
-                    f.write('void handle_%s(struct player *pplayer, struct %s *packet);\n'%(a,p.name))
+                    f.write('void handle_%s(struct player *pplayer, const struct %s *packet);\n'%(a,p.name))
             else:
                 if p.handle_per_conn:
                     f.write('void handle_%s(struct connection *pc%s);\n'%(a,b))
@@ -1551,10 +1693,13 @@ bool server_handle_packet(enum packet_type type, void *packet,
 #ifndef FC__PACKHAND_GEN_H
 #define FC__PACKHAND_GEN_H
 
-#include "packets.h"
+/* utility */
 #include "shared.h"
 
-bool client_handle_packet(enum packet_type type, void *packet);
+/* common */
+#include "packets.h"
+
+bool client_handle_packet(enum packet_type type, const void *packet);
 
 ''')
     for p in packets:
@@ -1569,7 +1714,7 @@ bool client_handle_packet(enum packet_type type, void *packet);
             b="void"
         if p.handle_via_packet:
             f.write('struct %s;\n'%p.name)
-            f.write('void handle_%s(struct %s *packet);\n'%(a,p.name))
+            f.write('void handle_%s(const struct %s *packet);\n'%(a,p.name))
         else:
             f.write('void handle_%s(%s);\n'%(a,b))
     f.write('''
@@ -1584,12 +1729,13 @@ bool client_handle_packet(enum packet_type type, void *packet);
 #include <config.h>
 #endif
 
+/* common */
 #include "packets.h"
 
 #include "hand_gen.h"
     
-bool server_handle_packet(enum packet_type type, void *packet,
-			  struct player *pplayer, struct connection *pconn)
+bool server_handle_packet(enum packet_type type, const void *packet,
+                          struct player *pplayer, struct connection *pconn)
 {
   switch(type) {
 ''')
@@ -1597,7 +1743,7 @@ bool server_handle_packet(enum packet_type type, void *packet,
         if "cs" not in p.dirs: continue
         if p.no_handle: continue
         a=p.name[len("packet_"):]
-        c='((struct %s *)packet)->'%p.name
+        c='((const struct %s *)packet)->'%p.name
         b=[]
         for x in p.fields:
             y="%s%s"%(c,x.name)
@@ -1639,11 +1785,12 @@ bool server_handle_packet(enum packet_type type, void *packet,
 #include <config.h>
 #endif
 
+/* common */
 #include "packets.h"
 
 #include "packhand_gen.h"
     
-bool client_handle_packet(enum packet_type type, void *packet)
+bool client_handle_packet(enum packet_type type, const void *packet)
 {
   switch(type) {
 ''')
@@ -1651,7 +1798,7 @@ bool client_handle_packet(enum packet_type type, void *packet)
         if "sc" not in p.dirs: continue
         if p.no_handle: continue
         a=p.name[len("packet_"):]
-        c='((struct %s *)packet)->'%p.name
+        c='((const struct %s *)packet)->'%p.name
         b=[]
         for x in p.fields:
             y="%s%s"%(c,x.name)
