@@ -14,20 +14,23 @@
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
-#include <assert.h>
-#include <string.h>		/* strlen */
+#include <string.h>             /* strlen */
 
-#include "city.h"
+/* utility */
 #include "fcintl.h"
-#include "game.h"
-#include "hash.h"
+#include "iterator.h"
 #include "log.h"
 #include "mem.h"
-#include "movement.h"
-#include "packets.h"
 #include "rand.h"
 #include "shared.h"
 #include "support.h"
+
+/* common */
+#include "city.h"
+#include "game.h"
+#include "movement.h"
+#include "nation.h"
+#include "packets.h"
 #include "unit.h"
 #include "unitlist.h"
 
@@ -36,12 +39,33 @@
 /* the very map */
 struct civ_map map;
 
-/* An entry in the start position table. */
-struct startpos_entry {
-  int key;
-  const struct nation_type *nation;
+struct startpos {
+  struct tile *location;
+  bool exclude;
+  struct nation_hash *nations;
 };
 
+static struct startpos *startpos_new(struct tile *ptile);
+static void startpos_destroy(struct startpos *psp);
+
+/* struct startpos_hash and related functions. */
+#define SPECHASH_TAG startpos
+#define SPECHASH_KEY_TYPE const struct tile *
+#define SPECHASH_DATA_TYPE struct startpos *
+#define SPECHASH_DATA_FREE startpos_destroy
+#include "spechash.h"
+
+/* Srart position iterator. */
+struct startpos_iter {
+  struct iterator vtable;
+  const struct startpos *psp;
+  /* 'struct nation_iter' really. See startpos_iter_sizeof(). */
+  struct iterator nation_iter;
+};
+
+#define STARTPOS_ITER(p) ((struct startpos_iter *) (p))
+
+
 /* these are initialized from the terrain ruleset */
 struct terrain_misc terrain_control;
 
@@ -63,6 +87,9 @@ struct terrain_misc terrain_control;
 const int DIR_DX[8] = { -1, 0, 1, -1, 1, -1, 0, 1 };
 const int DIR_DY[8] = { -1, -1, -1, 0, 0, 1, 1, 1 };
 
+static bool restrict_infra(const struct player *pplayer, const struct tile *t1,
+                           const struct tile *t2);
+
 /****************************************************************************
   Return a bitfield of the specials on the tile that are infrastructure.
 ****************************************************************************/
@@ -134,11 +161,13 @@ void map_init(void)
   /* The [xy]size values are set in map_init_topology.  It is initialized
    * to a non-zero value because some places erronously use these values
    * before they're initialized. */
-  map.xsize = MAP_MIN_LINEAR_SIZE;  
-  map.ysize = MAP_MIN_LINEAR_SIZE;
+  map.xsize = MAP_DEFAULT_LINEAR_SIZE;
+  map.ysize = MAP_DEFAULT_LINEAR_SIZE;
 
   if (is_server()) {
+    map.server.mapsize = MAP_DEFAULT_MAPSIZE;
     map.server.size = MAP_DEFAULT_SIZE;
+    map.server.tilesperplayer = MAP_DEFAULT_TILESPERPLAYER;
     map.server.seed = MAP_DEFAULT_SEED;
     map.server.riches = MAP_DEFAULT_RICHES;
     map.server.huts = MAP_DEFAULT_HUTS;
@@ -151,7 +180,6 @@ void map_init(void)
     map.server.separatepoles = MAP_DEFAULT_SEPARATE_POLES;
     map.server.alltemperate = MAP_DEFAULT_ALLTEMPERATE;
     map.server.temperature = MAP_DEFAULT_TEMPERATURE;
-    map.server.num_start_positions = 0;
     map.server.have_resources = FALSE;
     map.server.have_rivers_overlay = FALSE;
     map.server.have_huts = FALSE;
@@ -213,7 +241,7 @@ static void generate_map_indices(void)
 	       : (nat_center_y + map.ysize - 1));
   tiles = (nat_max_x - nat_min_x + 1) * (nat_max_y - nat_min_y + 1);
 
-  assert(map.iterate_outwards_indices == NULL);
+  fc_assert(NULL == map.iterate_outwards_indices);
   map.iterate_outwards_indices =
       fc_malloc(tiles * sizeof(*map.iterate_outwards_indices));
 
@@ -237,17 +265,17 @@ static void generate_map_indices(void)
       i++;
     }
   }
-  assert(i == tiles);
+  fc_assert(i == tiles);
 
   qsort(map.iterate_outwards_indices, tiles,
         sizeof(*map.iterate_outwards_indices), compare_iter_index);
 
 #if 0
   for (i = 0; i < tiles; i++) {
-    freelog(LOG_DEBUG, "%5d : (%3d,%3d) : %d", i,
-            map.iterate_outwards_indices[i].dx,
-            map.iterate_outwards_indices[i].dy,
-            map.iterate_outwards_indices[i].dist);
+    log_debug("%5d : (%3d,%3d) : %d", i,
+              map.iterate_outwards_indices[i].dx,
+              map.iterate_outwards_indices[i].dy,
+              map.iterate_outwards_indices[i].dist);
   }
 #endif
 
@@ -275,14 +303,14 @@ void map_init_topology(bool set_sizes)
   }
   
   /* sanity check for iso topologies*/
-  assert(!MAP_IS_ISOMETRIC || (map.ysize % 2) == 0);
+  fc_assert(!MAP_IS_ISOMETRIC || (map.ysize % 2) == 0);
 
   /* The size and ratio must satisfy the minimum and maximum *linear*
    * restrictions on width */
-  assert(MAP_WIDTH >= MAP_MIN_LINEAR_SIZE);
-  assert(MAP_HEIGHT >= MAP_MIN_LINEAR_SIZE);
-  assert(MAP_WIDTH <= MAP_MAX_LINEAR_SIZE);
-  assert(MAP_HEIGHT <= MAP_MAX_LINEAR_SIZE);
+  fc_assert(MAP_WIDTH >= MAP_MIN_LINEAR_SIZE);
+  fc_assert(MAP_HEIGHT >= MAP_MIN_LINEAR_SIZE);
+  fc_assert(MAP_WIDTH <= MAP_MAX_LINEAR_SIZE);
+  fc_assert(MAP_HEIGHT <= MAP_MAX_LINEAR_SIZE);
 
   map.num_valid_dirs = map.num_cardinal_dirs = 0;
   for (dir = 0; dir < 8; dir++) {
@@ -295,9 +323,9 @@ void map_init_topology(bool set_sizes)
       map.num_cardinal_dirs++;
     }
   }
-  assert(map.num_valid_dirs > 0 && map.num_valid_dirs <= 8);
-  assert(map.num_cardinal_dirs > 0
-	 && map.num_cardinal_dirs <= map.num_valid_dirs);
+  fc_assert(map.num_valid_dirs > 0 && map.num_valid_dirs <= 8);
+  fc_assert(map.num_cardinal_dirs > 0
+            && map.num_cardinal_dirs <= map.num_valid_dirs);
 }
 
 /***************************************************************
@@ -307,11 +335,6 @@ static void tile_init(struct tile *ptile)
 {
   ptile->continent = 0;
 
-  BV_CLR_ALL(ptile->tile_known);
-  vision_layer_iterate(v) {
-    BV_CLR_ALL(ptile->tile_seen[v]);
-  } vision_layer_iterate_end;
-
   tile_clear_all_specials (ptile);
   ptile->resource = NULL;
   ptile->terrain  = T_UNKNOWN;
@@ -418,7 +441,7 @@ struct tile *index_to_tile(int index)
 ***************************************************************/
 static void tile_free(struct tile *ptile)
 {
-  unit_list_free(ptile->units);
+  unit_list_destroy(ptile->units);
   if (ptile->spec_sprite) {
     free(ptile->spec_sprite);
     ptile->spec_sprite = NULL;
@@ -431,10 +454,10 @@ static void tile_free(struct tile *ptile)
 **************************************************************************/
 void map_allocate(void)
 {
-  freelog(LOG_DEBUG, "map_allocate (was %p) (%d,%d)",
-	  (void *)map.tiles, map.xsize, map.ysize);
+  log_debug("map_allocate (was %p) (%d,%d)",
+            (void *) map.tiles, map.xsize, map.ysize);
 
-  assert(map.tiles == NULL);
+  fc_assert_ret(NULL == map.tiles);
   map.tiles = fc_calloc(MAP_INDEX_SIZE, sizeof(*map.tiles));
 
   /* Note this use of whole_map_iterate may be a bit sketchy, since the
@@ -455,10 +478,9 @@ void map_allocate(void)
   generate_map_indices();
 
   if (map.startpos_table != NULL) {
-    hash_free(map.startpos_table);
+    startpos_hash_destroy(map.startpos_table);
   }
-  map.startpos_table = hash_new_full(hash_fval_keyval, hash_fcmp_keyval,
-                                     NULL, free);
+  map.startpos_table = startpos_hash_new();
 }
 
 /***************************************************************
@@ -477,7 +499,7 @@ void map_free(void)
     map.tiles = NULL;
 
     if (map.startpos_table) {
-      hash_free(map.startpos_table);
+      startpos_hash_destroy(map.startpos_table);
       map.startpos_table = NULL;
     }
 
@@ -681,7 +703,8 @@ bool can_channel_land(const struct tile *ptile)
   May also be used with punit==NULL, in which case punit
   tests are not done (for unit-independent results).
 ***************************************************************/
-static int tile_move_cost_ptrs(struct unit *punit,
+static int tile_move_cost_ptrs(const struct unit *punit,
+                               const struct player *pplayer,
 			       const struct tile *t1, const struct tile *t2)
 {
   bool cardinal_move;
@@ -714,7 +737,7 @@ static int tile_move_cost_ptrs(struct unit *punit,
    * leaving ships, so F_IGTER check has to be before native terrain
    * check. We want to give railroad bonus only to native units. */
   if (tile_has_special(t1, S_RAILROAD) && tile_has_special(t2, S_RAILROAD)
-      && native) {
+      && native && !restrict_infra(pplayer, t1, t2)) {
     return MOVE_COST_RAIL;
   }
   if (punit && unit_has_type_flag(punit, F_IGTER)) {
@@ -724,7 +747,8 @@ static int tile_move_cost_ptrs(struct unit *punit,
     /* Loading to transport or entering port */
     return SINGLE_MOVE;
   }
-  if (tile_has_special(t1, S_ROAD) && tile_has_special(t2, S_ROAD)) {
+  if (tile_has_special(t1, S_ROAD) && tile_has_special(t2, S_ROAD)
+      && !restrict_infra(pplayer, t1, t2)) {
     return MOVE_COST_ROAD;
   }
 
@@ -752,6 +776,29 @@ static int tile_move_cost_ptrs(struct unit *punit,
   return tile_terrain(t2)->movement_cost * SINGLE_MOVE;
 }
 
+/****************************************************************************
+  Returns TRUE if there is a restriction with regard to the infrastructure,
+  i.e. at least one of the tiles t1 and t2 is claimed by a unfriendly
+  nation. This means that one can not use of the infrastructure (road,
+  railroad) on this tile.
+****************************************************************************/
+static bool restrict_infra(const struct player *pplayer, const struct tile *t1,
+                           const struct tile *t2)
+{
+  struct player *plr1 = tile_owner(t1), *plr2 = tile_owner(t2);
+
+  if (!pplayer || !game.info.restrictinfra) {
+    return FALSE;
+  }
+
+  if ((plr1 && pplayers_at_war(plr1, pplayer))
+      || (plr2 && pplayers_at_war(plr2, pplayer))) {
+    return TRUE;
+  }
+
+  return FALSE;
+}
+
 /****************************************************************************
   map_move_cost_ai returns the move cost as
   calculated by tile_move_cost_ptrs (with no unit pointer to get
@@ -765,13 +812,14 @@ static int tile_move_cost_ptrs(struct unit *punit,
   FIXME: this function can't be used for air units because it returns
   sea<->land moves as impossible.
 ****************************************************************************/
-int map_move_cost_ai(const struct tile *tile0, const struct tile *tile1)
+int map_move_cost_ai(const struct player *pplayer, const struct tile *tile0,
+                     const struct tile *tile1)
 {
   const int maxcost = 72; /* Arbitrary. */
 
-  assert(!is_server()
-	 || (tile_terrain(tile0) != T_UNKNOWN 
-	  && tile_terrain(tile1) != T_UNKNOWN));
+  fc_assert_ret_val(!is_server()
+                    || (tile_terrain(tile0) != T_UNKNOWN
+                        && tile_terrain(tile1) != T_UNKNOWN), FC_INFINITY);
 
   /* A ship can take the step if:
    * - both tiles are ocean or
@@ -799,7 +847,7 @@ int map_move_cost_ai(const struct tile *tile0, const struct tile *tile1)
     return maxcost;
   }
 
-  return tile_move_cost_ptrs(NULL, tile0, tile1);
+  return tile_move_cost_ptrs(NULL, pplayer, tile0, tile1);
 }
 
 /***************************************************************
@@ -808,15 +856,17 @@ int map_move_cost_ai(const struct tile *tile0, const struct tile *tile1)
 ***************************************************************/
 int map_move_cost_unit(struct unit *punit, const struct tile *ptile)
 {
-  return tile_move_cost_ptrs(punit, punit->tile, ptile);
+  return tile_move_cost_ptrs(punit, unit_owner(punit),
+                             punit->tile, ptile);
 }
 
 /***************************************************************
   Move cost between two tiles
 ***************************************************************/
-int map_move_cost(const struct tile *src_tile, const struct tile *dst_tile)
+int map_move_cost(const struct player *pplayer,
+                  const struct tile *src_tile, const struct tile *dst_tile)
 {
-  return tile_move_cost_ptrs(NULL, src_tile, dst_tile);
+  return tile_move_cost_ptrs(NULL, pplayer, src_tile, dst_tile);
 }
 
 /***************************************************************
@@ -833,7 +883,7 @@ bool is_tiles_adjacent(const struct tile *tile0, const struct tile *tile1)
 ***************************************************************/
 bool same_pos(const struct tile *tile1, const struct tile *tile2)
 {
-  assert(tile1 != NULL && tile2 != NULL);
+  fc_assert_ret_val(tile1 != NULL && tile2 != NULL, FALSE);
   return (tile1 == tile2);
 }
 
@@ -986,7 +1036,7 @@ struct tile *rand_neighbour(const struct tile *ptile)
   /* This clever loop by Trent Piepho will take no more than
    * 8 tries to find a valid direction. */
   for (n = 8; n > 0; n--) {
-    enum direction8 choice = (enum direction8) myrand(n);
+    enum direction8 choice = (enum direction8) fc_rand(n);
 
     /* this neighbour's OK */
     tile1 = mapstep(ptile, dirs[choice]);
@@ -999,7 +1049,7 @@ struct tile *rand_neighbour(const struct tile *ptile)
     dirs[choice] = dirs[n - 1];
   }
 
-  assert(0);			/* Are we on a 1x1 map with no wrapping??? */
+  fc_assert(FALSE);     /* Are we on a 1x1 map with no wrapping??? */
   return NULL;
 }
 
@@ -1009,7 +1059,7 @@ struct tile *rand_neighbour(const struct tile *ptile)
 **************************************************************************/
 struct tile *rand_map_pos(void)
 {
-  int nat_x = myrand(map.xsize), nat_y = myrand(map.ysize);
+  int nat_x = fc_rand(map.xsize), nat_y = fc_rand(map.ysize);
 
   return native_pos_to_tile(nat_x, nat_y);
 }
@@ -1031,13 +1081,15 @@ struct tile *rand_map_pos_filtered(void *data,
   /* First do a few quick checks to find a spot.  The limit on number of
    * tries could use some tweaking. */
   do {
-    ptile = map.tiles + myrand(MAP_INDEX_SIZE);
+    ptile = map.tiles + fc_rand(MAP_INDEX_SIZE);
   } while (filter && !filter(ptile, data) && ++tries < max_tries);
 
   /* If that fails, count all available spots and pick one.
    * Slow but reliable. */
   if (tries == max_tries) {
-    int count = 0, positions[MAP_INDEX_SIZE];
+    int count = 0, *positions;
+
+    positions = fc_calloc(MAP_INDEX_SIZE, sizeof(*positions));
 
     whole_map_iterate(ptile) {
       if (filter(ptile, data)) {
@@ -1047,13 +1099,14 @@ struct tile *rand_map_pos_filtered(void *data,
     } whole_map_iterate_end;
 
     if (count == 0) {
-      return NULL;
+      ptile = NULL;
+    } else {
+      ptile = map.tiles + positions[fc_rand(count)];
     }
 
-    return map.tiles + positions[myrand(count)];
-  } else {
-    return ptile;
+    FC_FREE(positions);
   }
+  return ptile;
 }
 
 /**************************************************************************
@@ -1108,7 +1161,7 @@ enum direction8 dir_cw(enum direction8 dir)
   case DIR8_NORTHWEST:
     return DIR8_NORTH;
   default:
-    assert(0);
+    fc_assert(FALSE);
     return -1;
   }
 }
@@ -1137,7 +1190,7 @@ enum direction8 dir_ccw(enum direction8 dir)
   case DIR8_NORTHWEST:
     return DIR8_WEST;
   default:
-    assert(0);
+    fc_assert(FALSE);
     return -1;
   }
 }
@@ -1224,7 +1277,7 @@ int get_direction_for_step(const struct tile *start_tile,
     return dir;
   }
 
-  assert(0);
+  fc_assert(FALSE);
   return -1;
 }
 
@@ -1259,90 +1312,331 @@ bool is_singular_tile(const struct tile *ptile, int dist)
   } do_in_natural_pos_end;
 }
 
-/****************************************************************************
-  Is there start positions set for map
-****************************************************************************/
-bool map_startpositions_set(void)
-{
-  if (!map.startpos_table) {
-    return FALSE;
-  }
 
-  return hash_num_entries(map.startpos_table) != 0;
+/****************************************************************************
+  Create a new, empty start position.
+****************************************************************************/
+static struct startpos *startpos_new(struct tile *ptile)
+{
+  struct startpos *psp = fc_malloc(sizeof(*psp));
+
+  psp->location = ptile;
+  psp->exclude = FALSE;
+  psp->nations = nation_hash_new();
+
+  return psp;
 }
 
 /****************************************************************************
-  Set a start position at the given tile for the given nation. Clears any
-  existing start position at the tile.
+  Free all memory allocated by the start position.
 ****************************************************************************/
-void map_set_startpos(const struct tile *ptile,
-                      const struct nation_type *pnation)
+static void startpos_destroy(struct startpos *psp)
 {
-  struct startpos_entry *spe;
-
-  if (!map.startpos_table || !ptile) {
-    return;
-  }
-  map_clear_startpos(ptile);
-
-  spe = fc_calloc(1, sizeof(*spe));
-  spe->key = tile_index(ptile);
-  spe->nation = pnation;
-
-  hash_insert(map.startpos_table, FC_INT_TO_PTR(spe->key), spe);
+  fc_assert_ret(NULL != psp);
+  nation_hash_destroy(psp->nations);
+  free(psp);
 }
 
 /****************************************************************************
-  Returns the nation of the start position at the given tile, or NULL if
-  none exists there.
+  Returns the start position associated to the given ID.
 ****************************************************************************/
-bool map_has_startpos(const struct tile *ptile)
+struct startpos *map_startpos_by_number(int id)
 {
-  struct startpos_entry *spe;
+  return map_startpos_get(index_to_tile(id));
+}
 
-  if (!map.startpos_table || !ptile) {
-    return FALSE;
+/****************************************************************************
+  Returns the unique ID number for this start position. This is just the
+  tile index of the tile where this start position is located.
+****************************************************************************/
+int startpos_number(const struct startpos *psp)
+{
+  fc_assert_ret_val(NULL != psp, -1);
+  return tile_index(psp->location);
+}
+
+/****************************************************************************
+  Allow the nation to start at the start position.
+  NB: in "excluding" mode, this remove the nation from the excluded list.
+****************************************************************************/
+bool startpos_allow(struct startpos *psp, struct nation_type *pnation)
+{
+  fc_assert_ret_val(NULL != psp, FALSE);
+  fc_assert_ret_val(NULL != pnation, FALSE);
+
+  if (0 == nation_hash_size(psp->nations) || !psp->exclude) {
+    psp->exclude = FALSE; /* Disable "excluding" mode. */
+    return nation_hash_insert(psp->nations, pnation, NULL);
+  } else {
+    return nation_hash_remove(psp->nations, pnation);
   }
+}
 
-  spe = hash_lookup_data(map.startpos_table,
-                         FC_INT_TO_PTR(tile_index(ptile)));
-  if (!spe) {
-    return FALSE;
+/****************************************************************************
+  Disallow the nation to start at the start position.
+  NB: in "excluding" mode, this add the nation to the excluded list.
+****************************************************************************/
+bool startpos_disallow(struct startpos *psp, struct nation_type *pnation)
+{
+  fc_assert_ret_val(NULL != psp, FALSE);
+  fc_assert_ret_val(NULL != pnation, FALSE);
+
+  if (0 == nation_hash_size(psp->nations) || psp->exclude) {
+    psp->exclude = TRUE; /* Enable "excluding" mode. */
+    return nation_hash_remove(psp->nations, pnation);
+  } else {
+    return nation_hash_insert(psp->nations, pnation, NULL);
   }
+}
 
+/****************************************************************************
+  Returns the tile where this start position is located.
+****************************************************************************/
+struct tile *startpos_tile(const struct startpos *psp)
+{
+  fc_assert_ret_val(NULL != psp, NULL);
+  return psp->location;
+}
+
+/****************************************************************************
+  Returns TRUE if the given nation can start here.
+****************************************************************************/
+bool startpos_nation_allowed(const struct startpos *psp,
+                             const struct nation_type *pnation)
+{
+  fc_assert_ret_val(NULL != psp, FALSE);
+  fc_assert_ret_val(NULL != pnation, FALSE);
+  return XOR(psp->exclude, nation_hash_lookup(psp->nations, pnation, NULL));
+}
+
+/****************************************************************************
+  Returns TRUE if any nation can start here.
+****************************************************************************/
+bool startpos_allows_all(const struct startpos *psp)
+{
+  fc_assert_ret_val(NULL != psp, FALSE);
+  return (0 == nation_hash_size(psp->nations));
+}
+
+/****************************************************************************
+  Fills the packet with all of the information at this start position.
+  Returns TRUE if the packet can be sent.
+****************************************************************************/
+bool startpos_pack(const struct startpos *psp,
+                   struct packet_edit_startpos_full *packet)
+{
+  fc_assert_ret_val(NULL != psp, FALSE);
+  fc_assert_ret_val(NULL != packet, FALSE);
+
+  packet->id = startpos_number(psp);
+  packet->exclude = psp->exclude;
+  BV_CLR_ALL(packet->nations);
+
+  nation_hash_iterate(psp->nations, pnation) {
+    BV_SET(packet->nations, nation_number(pnation));
+  } nation_hash_iterate_end;
   return TRUE;
 }
 
 /****************************************************************************
-  Returns the nation of the start position at the given tile, or NULL if
-  none exists there.
+  Fills the start position with the nation information in the packet.
+  Returns TRUE if the start position was changed.
 ****************************************************************************/
-const struct nation_type *map_get_startpos(const struct tile *ptile)
+bool startpos_unpack(struct startpos *psp,
+                     const struct packet_edit_startpos_full *packet)
 {
-  struct startpos_entry *spe;
+  fc_assert_ret_val(NULL != psp, FALSE);
+  fc_assert_ret_val(NULL != packet, FALSE);
 
-  if (!map.startpos_table || !ptile) {
-    return NULL;
+  psp->exclude = packet->exclude;
+
+  nation_hash_clear(psp->nations);
+  if (!BV_ISSET_ANY(packet->nations)) {
+    return TRUE;
   }
-
-  spe = hash_lookup_data(map.startpos_table,
-                         FC_INT_TO_PTR(tile_index(ptile)));
-  if (!spe) {
-    return NULL;
-  }
-
-  return spe->nation;
+  nations_iterate(pnation) {
+    if (BV_ISSET(packet->nations, nation_number(pnation))) {
+      nation_hash_insert(psp->nations, pnation, NULL);
+    }
+  } nations_iterate_end;
+  return TRUE;
 }
 
 /****************************************************************************
-  Remove a start position at the given tile.
+  Returns TRUE if the nations returned by startpos_raw_nations()
+  are actually excluded from the nations allowed to start at this position.
+
+  FIXME: This function exposes the internal implementation and should be
+  removed when no longer needed by the property editor system.
 ****************************************************************************/
-void map_clear_startpos(const struct tile *ptile)
+bool startpos_is_excluding(const struct startpos *psp)
 {
-  if (!map.startpos_table || !ptile) {
-    return;
+  fc_assert_ret_val(NULL != psp, FALSE);
+  return psp->exclude;
+}
+
+/****************************************************************************
+  Return a the nations hash, used for the property editor.
+
+  FIXME: This function exposes the internal implementation and should be
+  removed when no longer needed by the property editor system.
+****************************************************************************/
+const struct nation_hash *startpos_raw_nations(const struct startpos *psp)
+{
+  fc_assert_ret_val(NULL != psp, FALSE);
+  return psp->nations;
+}
+
+/****************************************************************************
+  Implementation of iterator 'sizeof' function.
+
+  struct startpos_iter can be either a 'struct nation_hash_iter', a 'struct
+  nation_iter' or 'struct startpos_iter'.
+****************************************************************************/
+size_t startpos_iter_sizeof(void)
+{
+  return MAX(sizeof(struct startpos_iter) + nation_iter_sizeof()
+             - sizeof(struct iterator), nation_hash_iter_sizeof());
+}
+
+/****************************************************************************
+  Implementation of iterator 'next' function.
+
+  NB: This is only used for the case where 'exclude' is set in the start
+  position.
+****************************************************************************/
+static void startpos_exclude_iter_next(struct iterator *startpos_iter)
+{
+  struct startpos_iter *iter = STARTPOS_ITER(startpos_iter);
+
+  do {
+    iterator_next(&iter->nation_iter);
+  } while (iterator_valid(&iter->nation_iter)
+           || !nation_hash_lookup(iter->psp->nations,
+                                  iterator_get(&iter->nation_iter), NULL));
+}
+
+/****************************************************************************
+  Implementation of iterator 'get' function. Returns a struct nation_type
+  pointer.
+
+  NB: This is only used for the case where 'exclude' is set in the start
+  position.
+****************************************************************************/
+static void *startpos_exclude_iter_get(const struct iterator *startpos_iter)
+{
+  struct startpos_iter *iter = STARTPOS_ITER(startpos_iter);
+  return iterator_get(&iter->nation_iter);
+}
+
+/****************************************************************************
+  Implementation of iterator 'valid' function.
+****************************************************************************/
+static bool startpos_exclude_iter_valid(const struct iterator *startpos_iter)
+{
+  struct startpos_iter *iter = STARTPOS_ITER(startpos_iter);
+  return iterator_valid(&iter->nation_iter);
+}
+
+/****************************************************************************
+  Initialize and return an iterator for the nations at this start position.
+****************************************************************************/
+struct iterator *startpos_iter_init(struct startpos_iter *iter,
+                                    const struct startpos *psp)
+{
+  if (!psp) {
+    return invalid_iter_init(ITERATOR(iter));
   }
 
-  hash_delete_entry(map.startpos_table,
-                    FC_INT_TO_PTR(tile_index(ptile)));
+  if (startpos_allows_all(psp)) {
+    return nation_iter_init((struct nation_iter *) iter);
+  }
+
+  if (!psp->exclude) {
+    return nation_hash_key_iter_init((struct nation_hash_iter *) iter,
+                                     psp->nations);
+  }
+
+  iter->vtable.next = startpos_exclude_iter_next;
+  iter->vtable.get = startpos_exclude_iter_get;
+  iter->vtable.valid = startpos_exclude_iter_valid;
+  iter->psp = psp;
+  (void) nation_iter_init((struct nation_iter *) &iter->nation_iter);
+
+  return ITERATOR(iter);
+}
+
+
+/****************************************************************************
+  Is there start positions set for map
+****************************************************************************/
+int map_startpos_count(void)
+{
+  if (NULL != map.startpos_table) {
+    return startpos_hash_size(map.startpos_table);
+  } else {
+    return 0;
+  }
+}
+
+/****************************************************************************
+  Create a new start position at the given tile and return it. If a start
+  position already exists there, it is first removed.
+****************************************************************************/
+struct startpos *map_startpos_new(struct tile *ptile)
+{
+  struct startpos *psp;
+
+  fc_assert_ret_val(NULL != ptile, NULL);
+  fc_assert_ret_val(NULL != map.startpos_table, NULL);
+
+  psp = startpos_new(ptile);
+  startpos_hash_replace(map.startpos_table, ptile, psp);
+  return psp;
+}
+
+/****************************************************************************
+  Returns the start position at the given tile, or NULL if none exists
+  there.
+****************************************************************************/
+struct startpos *map_startpos_get(const struct tile *ptile)
+{
+  struct startpos *psp;
+
+  fc_assert_ret_val(NULL != ptile, NULL);
+  fc_assert_ret_val(NULL != map.startpos_table, NULL);
+
+  startpos_hash_lookup(map.startpos_table, ptile, &psp);
+  return psp;
+}
+
+/****************************************************************************
+  Remove the start position at the given tile. Returns TRUE if the start
+  position was removed.
+****************************************************************************/
+bool map_startpos_remove(struct tile *ptile)
+{
+  fc_assert_ret_val(NULL != ptile, FALSE);
+  fc_assert_ret_val(NULL != map.startpos_table, FALSE);
+
+  return startpos_hash_remove(map.startpos_table, ptile);
+}
+
+/****************************************************************************
+  Implementation of iterator sizeof function.
+  NB: map_sp_iter just wraps startpos_hash_iter.
+****************************************************************************/
+size_t map_startpos_iter_sizeof(void)
+{
+  return startpos_hash_iter_sizeof();
+}
+
+/****************************************************************************
+  Implementation of iterator init function.
+  NB: map_sp_iter just wraps startpos_hash_iter.
+****************************************************************************/
+struct iterator *map_startpos_iter_init(struct map_startpos_iter *iter)
+{
+  return startpos_hash_value_iter_init((struct startpos_hash_iter *) iter,
+                                       map.startpos_table);
 }
