@@ -15,9 +15,8 @@
 #include <config.h>
 #endif
 
-#include <assert.h>
-
 /* utility */
+#include "bitvector.h"
 #include "fcintl.h"
 #include "log.h"
 #include "shared.h"
@@ -36,7 +35,6 @@
 
 #include "movement.h"
 
-
 static const char *move_type_names[] = {
   "Land", "Sea", "Both"
 };
@@ -155,15 +153,15 @@ bool is_ground_unittype(const struct unit_type *punittype)
 static bool is_city_channel_tile(const struct unit_class *punitclass,
                                  const struct tile *ptile)
 {
-  bool tile_processed[map_num_tiles()];
+  struct dbv tile_processed = { 0, NULL };
   struct tile_list *process_queue = tile_list_new();
   bool found = FALSE;
 
-  memset(tile_processed, 0, sizeof(tile_processed));
+  dbv_init(&tile_processed, map_num_tiles());
   for (;;) {
-    tile_processed[tile_index(ptile)] = TRUE;
+    dbv_set(&tile_processed, tile_index(ptile));
     adjc_iterate(ptile, piter) {
-      if (tile_processed[tile_index(piter)]) {
+      if (dbv_isset(&tile_processed, tile_index(piter))) {
         continue;
       } else if (is_native_to_class(punitclass, tile_terrain(piter),
                                     piter->special, piter->bases)) {
@@ -172,19 +170,20 @@ static bool is_city_channel_tile(const struct unit_class *punitclass,
       } else if (NULL != tile_city(piter)) {
         tile_list_append(process_queue, piter);
       } else {
-        tile_processed[tile_index(piter)] = TRUE;
+        dbv_set(&tile_processed, tile_index(piter));
       }
     } adjc_iterate_end;
 
     if (0 == tile_list_size(process_queue)) {
       break; /* No more tile to process. */
     } else {
-      ptile = tile_list_get(process_queue, 0);
-      tile_list_unlink(process_queue, (struct tile *) ptile);
+      ptile = tile_list_front(process_queue);
+      tile_list_pop_front(process_queue);
     }
   }
 
-  tile_list_free(process_queue);
+  dbv_free(&tile_processed);
+  tile_list_destroy(process_queue);
   return found;
 }
 
@@ -422,19 +421,18 @@ bool zoc_ok_move(const struct unit *punit, const struct tile *dst_tile)
   Returns whether the unit can move from its current tile to the destination
   tile.
 
-  See test_unit_move_to_tile().
+  See unit_move_to_tile_test().
 ****************************************************************************/
-bool can_unit_move_to_tile(const struct unit *punit,
+bool unit_can_move_to_tile(const struct unit *punit,
                            const struct tile *dst_tile,
                            bool igzoc)
 {
-  return MR_OK == test_unit_move_to_tile(unit_type(punit), unit_owner(punit),
-					 punit->activity,
-					 punit->tile, dst_tile,
-					 igzoc);
+  return (MR_OK == unit_move_to_tile_test(unit_type(punit),
+                                          unit_owner(punit),
+                                          punit->activity, unit_tile(punit),
+                                          dst_tile, igzoc));
 }
 
-
 /**************************************************************************
   Returns whether the unit can move from its current tile to the
   destination tile.  An enumerated value is returned indication the error
@@ -444,21 +442,22 @@ bool can_unit_move_to_tile(const struct unit *punit,
     1) The unit is idle or on server goto.
     2) The target location is next to the unit.
     3) There are no non-allied units on the target tile.
-    4) Unit can move to non-native tile if there is city
-       or free transport capacity.
-    5) Marines are the only land units that can attack from a ocean square.
-    6) There are no peaceful but un-allied units on the target tile.
-    7) There is not a peaceful but un-allied city on the target tile.
-    8) There is no non-allied unit blocking (zoc) [or igzoc is true].
-    9) Triremes cannot move out of sight from land.
-   10) It is not the territory of a player we are at peace with.
+    4) Unit can move to a tile where it can't survive on its own if there
+       is city or free transport capacity.
+    5) Some units cannot take over a city.
+    6) Marines are the only land units that can attack from a ocean square.
+    7) There are no peaceful but un-allied units on the target tile.
+    8) There is not a peaceful but un-allied city on the target tile.
+    9) There is no non-allied unit blocking (zoc) [or igzoc is true].
+   10) Triremes cannot move out of sight from land.
+   11) It is not the territory of a player we are at peace with.
 **************************************************************************/
-enum unit_move_result test_unit_move_to_tile(const struct unit_type *punittype,
-					     const struct player *unit_owner,
-					     enum unit_activity activity,
-					     const struct tile *src_tile,
-					     const struct tile *dst_tile,
-					     bool igzoc)
+enum unit_move_result
+unit_move_to_tile_test(const struct unit_type *punittype,
+                       const struct player *unit_owner,
+                       enum unit_activity activity,
+                       const struct tile *src_tile,
+                       const struct tile *dst_tile, bool igzoc)
 {
   bool zoc;
   struct city *pcity;
@@ -484,28 +483,31 @@ enum unit_move_result test_unit_move_to_tile(const struct unit_type *punittype,
   }
 
   /* 4) */
-  if (!is_native_tile(punittype, dst_tile)
+  if (!can_exist_at_tile(punittype, dst_tile)
       && !is_allied_city_tile(dst_tile, unit_owner)
       && unit_class_transporter_capacity(dst_tile, unit_owner, utype_class(punittype)) <= 0) {
     return MR_NO_TRANSPORTER_CAPACITY;
   }
 
-  if (utype_move_type(punittype) == LAND_MOVING) {
+  pcity = is_enemy_city_tile(dst_tile, unit_owner);
+  if (NULL != pcity) {
+    /* 5) */
+    if (!utype_can_take_over(punittype)) {
+      return MR_BAD_TYPE_FOR_CITY_TAKE_OVER;
+    }
 
-    /* Moving from ocean */
-    if (is_ocean_tile(src_tile)) {
-      /* 5) */
-      if (!utype_has_flag(punittype, F_MARINES)
-	  && is_enemy_city_tile(dst_tile, unit_owner)) {
-	/* Most ground units can't move into cities from ships.  (Note this
-	 * check is only for movement, not attacking: most ground units
-	 * can't attack from ship at *any* units on land.) */
-	return MR_BAD_TYPE_FOR_CITY_TAKE_OVER;
-      }
+    /* 6) */
+    if (utype_move_type(punittype) == LAND_MOVING
+        && is_ocean_tile(src_tile)      /* Moving from ocean */
+        && !utype_has_flag(punittype, F_MARINES)) {
+      /* Most ground units can't move into cities from ships. (Note this
+       * check is only for movement, not attacking: most ground units
+       * can't attack from ship at *any* units on land.) */
+      return MR_BAD_TYPE_FOR_CITY_TAKE_OVER_FROM_SEA;
     }
   }
 
-  /* 6) */
+  /* 7) */
   if (is_non_attack_unit_tile(dst_tile, unit_owner)) {
     /* You can't move into a non-allied tile.
      *
@@ -514,7 +516,7 @@ enum unit_move_result test_unit_move_to_tile(const struct unit_type *punittype,
     return MR_NO_WAR;
   }
 
-  /* 7) */
+  /* 8) */
   pcity = tile_city(dst_tile);
   if (pcity && pplayers_non_attack(city_owner(pcity), unit_owner)) {
     /* You can't move into an empty city of a civilization you're at
@@ -522,7 +524,7 @@ enum unit_move_result test_unit_move_to_tile(const struct unit_type *punittype,
     return MR_NO_WAR;
   }
 
-  /* 8) */
+  /* 9) */
   zoc = igzoc
     || can_step_taken_wrt_to_zoc(punittype, unit_owner, src_tile, dst_tile);
   if (!zoc) {
@@ -530,12 +532,12 @@ enum unit_move_result test_unit_move_to_tile(const struct unit_type *punittype,
     return MR_ZOC;
   }
 
-  /* 9) */
+  /* 10) */
   if (utype_has_flag(punittype, F_TRIREME) && !is_safe_ocean(dst_tile)) {
     return MR_TRIREME;
   }
 
-  /* 9) */
+  /* 11) */
   if (!utype_has_flag(punittype, F_CIVILIAN)
       && !player_can_invade_tile(unit_owner, dst_tile)) {
     return MR_PEACE;
@@ -550,6 +552,9 @@ enum unit_move_result test_unit_move_to_tile(const struct unit_type *punittype,
 bool can_unit_transport(const struct unit *transporter,
                         const struct unit *transported)
 {
+  fc_assert_ret_val(transporter != NULL, FALSE);
+  fc_assert_ret_val(transported != NULL, FALSE);
+
   return can_unit_type_transport(unit_type(transporter), unit_class(transported));
 }
 
@@ -575,7 +580,7 @@ enum unit_move_type move_type_from_str(const char *s)
   int i;
 
   for (i = 0; i < MOVETYPE_LAST; i++) {
-    if (mystrcasecmp(move_type_names[i], s)==0) {
+    if (fc_strcasecmp(move_type_names[i], s)==0) {
       return i;
     }
   }
@@ -586,7 +591,7 @@ enum unit_move_type move_type_from_str(const char *s)
   Search transport suitable for given unit from tile. It has to have
   free space in it.
 **************************************************************************/
-struct unit *find_transport_from_tile(struct unit *punit, struct tile *ptile)
+struct unit *transport_from_tile(struct unit *punit, struct tile *ptile)
 {
   unit_list_iterate(ptile->units, ptransport) {
     if (could_unit_load(punit, ptransport)) {
