@@ -13,10 +13,13 @@
 #ifndef FC__PLAYER_H
 #define FC__PLAYER_H
 
-#include "fc_types.h"
+/* utility */
+#include "bitvector.h"
 
+/* common */
 #include "city.h"
 #include "connection.h"
+#include "fc_types.h"
 #include "nation.h"
 #include "shared.h"
 #include "spaceship.h"
@@ -31,6 +34,8 @@
 #define ANON_PLAYER_NAME "noname"
 #define ANON_USER_NAME "Unassigned"
 
+struct player_slot;
+
 enum handicap_type {
   H_DIPLOMAT = 0,     /* Can't build offensive diplomats */
   H_AWAY,             /* Away mode */
@@ -50,8 +55,6 @@ enum handicap_type {
   H_LAST
 };
 
-BV_DEFINE(bv_player, MAX_NUM_PLAYERS + MAX_NUM_BARBARIANS);
-
 BV_DEFINE(bv_handicap, H_LAST);
 
 struct player_economic {
@@ -61,6 +64,21 @@ struct player_economic {
   int luxury;
 };
 
+#define SPECENUM_NAME player_status
+/* 'normal' status */
+#define SPECENUM_VALUE0      PSTATUS_NORMAL
+/* set once the player is in the process of dying */
+#define SPECENUM_VALUE1      PSTATUS_DYING
+/* this player is winner in scenario game */
+#define SPECENUM_VALUE2      PSTATUS_WINNER
+/* has indicated willingness to surrender */
+#define SPECENUM_VALUE3      PSTATUS_SURRENDER
+/* keep this last */
+#define SPECENUM_COUNT       PSTATUS_COUNT
+#include "specenum_gen.h"
+
+BV_DEFINE(bv_pstatus, PSTATUS_COUNT);
+
 struct player_score {
   int happy;
   int content;
@@ -80,20 +98,15 @@ struct player_score {
   int bnp;
   int mfg;
   int spaceship;
-  int game;             /* total score you get in player dialog */
+  int units_built;      /* Number of units this player produced. */
+  int units_killed;     /* Number of enemy units killed. */
+  int units_lost;       /* Number of own units that died,
+                         * by combat or otherwise. */
+  int game;             /* Total score you get in player dialog. */
 };
 
 struct player_ai {
-  bool control;
-
-  /* 
-   * Valid values for tech_goal are:
-   *  - any existing tech but not A_NONE or
-   *  - A_UNSET.
-   */
-  int prev_gold;
   int maxbuycost;
-  int est_upkeep; /* estimated upkeep of buildings in cities */
   /* The units of tech_want seem to be shields */
   int tech_want[A_LAST+1];
   bv_handicap handicaps;        /* sum of enum handicap_type */
@@ -105,7 +118,7 @@ struct player_ai {
   int warmth, frost; /* threat of global warming / nuclear winter */
   enum barbarian_type barbarian_type;
 
-  int love[MAX_NUM_PLAYERS + MAX_NUM_BARBARIANS];
+  int love[MAX_NUM_PLAYER_SLOTS];
 };
 
 /* Diplomatic states (how one player views another).
@@ -125,17 +138,18 @@ enum diplstate_type {
 };
 
 enum dipl_reason {
-  DIPL_OK, DIPL_ERROR, DIPL_SENATE_BLOCKING, DIPL_ALLIANCE_PROBLEM
+  DIPL_OK, DIPL_ERROR, DIPL_SENATE_BLOCKING,
+  DIPL_ALLIANCE_PROBLEM_US, DIPL_ALLIANCE_PROBLEM_THEM
 };
 
 /* the following are for "pacts" */
 struct player_diplstate {
-  enum diplstate_type type;	/* this player's disposition towards other */
+  enum diplstate_type type; /* this player's disposition towards other */
   enum diplstate_type max_state; /* maximum treaty level ever had */
   int first_contact_turn; /* turn we had first contact with this player */
-  int turns_left;		/* until pact (e.g., cease-fire) ends */
-  int has_reason_to_cancel;	/* 0: no, 1: this turn, 2: this or next turn */
-  int contact_turns_left;	/* until contact ends */
+  int turns_left; /* until pact (e.g., cease-fire) ends */
+  int has_reason_to_cancel; /* 0: no, 1: this turn, 2: this or next turn */
+  int contact_turns_left; /* until contact ends */
 };
 
 /***************************************************************************
@@ -156,9 +170,10 @@ struct attribute_block_s {
 };
 
 struct ai_type;
+struct ai_data;
 
 struct player {
-  bool used;
+  struct player_slot *slot;
   char name[MAX_LEN_NAME];
   char username[MAX_LEN_NAME];
   char ranked_username[MAX_LEN_NAME]; /* the user who will be ranked */
@@ -172,16 +187,12 @@ struct player {
   bool phase_done;
   int nturns_idle;
   bool is_alive;
-  bool is_dying; /* set once the player is in the process of dying */
-  bool is_winner; /* This player is winner in scenario game */
-  bool surrendered; /* has indicated willingness to surrender */
 
   /* Turn in which the player's revolution is over; see update_revolution. */
   int revolution_finishes;
 
-  bool capital; /* used to give player init_buildings in first city. */
   bv_player real_embassy;
-  struct player_diplstate diplstates[MAX_NUM_PLAYERS + MAX_NUM_BARBARIANS];
+  const struct player_diplstate **diplstates;
   int city_style;
   struct city_list *cities;
   struct unit_list *units;
@@ -190,45 +201,82 @@ struct player {
 
   int bulbs_last_turn;    /* # bulbs researched last turn only */
   struct player_spaceship spaceship;
-  struct player_ai ai_data;
+
+  bool ai_controlled; /* 0: not automated; 1: automated */
+  struct player_ai ai_common;
   struct ai_type *ai;
+
   bool was_created;                    /* if the player was /created */
   bool is_connected;
   struct connection *current_conn;     /* non-null while handling packet */
   struct conn_list *connections;       /* will replace conn */
-  struct player_tile *private_map;
-  unsigned int gives_shared_vision; /* bitvector those that give you shared vision */
-  unsigned int really_gives_vision; /* takes into account that p3 may see what p1 has via p2 */
+  bv_player gives_shared_vision;       /* bitvector those that give you
+                                        * shared vision */
   int wonders[B_LAST];              /* contains city id's or WONDER_NOT_BUILT */
   struct attribute_block_s attribute_block;
   struct attribute_block_s attribute_block_buffer;
-  bv_debug debug;
+
+  struct dbv tile_known;
+
+  union {
+    struct {
+      /* Only used in the server (./ai/ and ./server/). */
+      bv_pstatus status;
+
+      bool capital; /* used to give player init_buildings in first city. */
+
+      struct player_tile *private_map;
+
+      bv_player really_gives_vision; /* takes into account that p3 may see
+                                      * what p1 has via p2 */
+
+      bv_debug debug;
+
+      struct ai_data *aidata;
+    } server;
+
+    struct {
+      /* Only used at the client (the server is omniscient; ./client/). */
+
+      /* Corresponds to the result of
+         (player:server:private_map[tile_index]:seen_count[vlayer] != 0). */
+      struct dbv tile_vision[V_COUNT];
+    } client;
+  };
 };
 
-/* A slot is a possibly uninitialized player. */
+/* Initialization and iteration */
+void player_slots_init(void);
+bool player_slots_initialised(void);
+void player_slots_free(void);
+
+struct player_slot *player_slot_first(void);
+struct player_slot *player_slot_next(struct player_slot *pslot);
+
+/* A player slot contains a possibly uninitialized player. */
 int player_slot_count(void);
-bool player_slot_is_used(const struct player *pslot);
-void player_slot_set_used(struct player *pslot, bool used);
-struct player *player_slot_by_number(int player_id);
+int player_slot_index(const struct player_slot *pslot);
+struct player *player_slot_get_player(const struct player_slot *pslot);
+bool player_slot_is_used(const struct player_slot *pslot);
+struct player_slot *player_slot_by_number(int player_id);
 
 /* General player accessor functions. */
-int player_count(void);
-void set_player_count(int count);
+struct player *player_new(struct player_slot *pslot);
+void player_clear(struct player *pplayer, bool full);
+void player_destroy(struct player *pplayer);
 
+int player_count(void);
 int player_index(const struct player *pplayer);
 int player_number(const struct player *pplayer);
-
 struct player *player_by_number(const int player_id);
-struct player *valid_player_by_number(const int player_id);
 
 const char *player_name(const struct player *pplayer);
-struct player *find_player_by_name(const char *name);
-struct player *find_player_by_name_prefix(const char *name,
-					  enum m_pre_result *result);
-struct player *find_player_by_user(const char *name);
+struct player *player_by_name(const char *name);
+struct player *player_by_name_prefix(const char *name,
+                                     enum m_pre_result *result);
+struct player *player_by_user(const char *name);
 
 bool player_set_nation(struct player *pplayer, struct nation_type *pnation);
-void player_set_unit_focus_status(struct player *pplayer);
 
 bool player_has_embassy(const struct player *pplayer,
                         const struct player *pplayer2);
@@ -253,22 +301,20 @@ bool player_owns_city(const struct player *pplayer,
 bool player_can_invade_tile(const struct player *pplayer,
                             const struct tile *ptile);
 
-struct city *player_find_city_by_id(const struct player *pplayer,
-				    int city_id);
-struct unit *player_find_unit_by_id(const struct player *pplayer,
-				    int unit_id);
+struct city *player_city_by_number(const struct player *pplayer,
+                                   int city_id);
+struct unit *player_unit_by_number(const struct player *pplayer,
+                                    int unit_id);
 
-bool player_in_city_radius(const struct player *pplayer,
-			   const struct tile *ptile);
+bool player_in_city_map(const struct player *pplayer,
+                        const struct tile *ptile);
 bool player_knows_techs_with_flag(const struct player *pplayer,
 				  enum tech_flag_id flag);
 int num_known_tech_with_flag(const struct player *pplayer,
 			     enum tech_flag_id flag);
 int player_get_expected_income(const struct player *pplayer);
 
-struct player_economic player_limit_to_max_rates(struct player *pplayer);
-
-struct city *find_palace(const struct player *pplayer);
+struct city *player_palace(const struct player *pplayer);
 
 bool ai_handicap(const struct player *pplayer, enum handicap_type htype);
 bool ai_fuzzy(const struct player *pplayer, bool normal_decision);
@@ -276,10 +322,8 @@ bool ai_fuzzy(const struct player *pplayer, bool normal_decision);
 const char *diplstate_text(const enum diplstate_type type);
 const char *love_text(const int love);
 
-const struct player_diplstate *pplayer_get_diplstate(const struct player
-						     *pplayer,
-						     const struct player
-						     *pplayer2);
+struct player_diplstate *player_diplstate_get(const struct player *plr1,
+                                              const struct player *plr2);
 bool are_diplstates_equal(const struct player_diplstate *pds1,
 			  const struct player_diplstate *pds2);
 enum dipl_reason pplayer_can_make_treaty(const struct player *p1,
@@ -306,37 +350,34 @@ bool is_barbarian(const struct player *pplayer);
 
 bool gives_shared_vision(const struct player *me, const struct player *them);
 
-struct player_research *get_player_research(const struct player *p1);
+/* iterate over all player slots */
+#define player_slots_iterate(_pslot)                                        \
+  if (player_slots_initialised()) {                                         \
+    struct player_slot *_pslot = player_slot_first();                       \
+    for (; NULL != _pslot; _pslot = player_slot_next(_pslot)) {
+#define player_slots_iterate_end                                            \
+    }                                                                       \
+  }
 
-void player_set_winner(struct player *plr);
+/* iterate over all players, which are used at the moment */
+#define players_iterate(_pplayer)                                           \
+  player_slots_iterate(_pslot) {                                            \
+    if (!player_slot_is_used(_pslot)) {                                     \
+      continue;                                                             \
+    }                                                                       \
+    struct player *_pplayer = player_slot_get_player(_pslot);
+#define players_iterate_end                                                 \
+  } player_slots_iterate_end;
 
-/* Initialization and iteration */
-void player_init(struct player *plr);
+/* get 'struct player_list' and related functions: */
+#define SPECLIST_TAG player
+#define SPECLIST_TYPE struct player
+#include "speclist.h"
 
-#define player_slots_iterate(NAME_pslot)\
-do {\
-  struct player *NAME_pslot;\
-  int MY_i;\
-  for (MY_i = 0; MY_i < player_slot_count(); MY_i++) {\
-    NAME_pslot = player_slot_by_number(MY_i);\
-    if (NAME_pslot != NULL) {\
-
-#define player_slots_iterate_end\
-    }\
-  }\
-} while (0)
-
-/* NB!!! This will only iterate over players for
- * which player_slot_is_used() returns TRUE. */
-#define players_iterate(NAME_pplayer)\
-do {\
-  player_slots_iterate(NAME_pplayer) {\
-    if (player_slot_is_used(NAME_pplayer)) {
-
-#define players_iterate_end\
-    }\
-  } player_slots_iterate_end;\
-} while (0)
+#define player_list_iterate(playerlist, pplayer)                            \
+  TYPED_LIST_ITERATE(struct player, playerlist, pplayer)
+#define player_list_iterate_end                                             \
+  LIST_ITERATE_END
 
 /* ai love values should be in range [-MAX_AI_LOVE..MAX_AI_LOVE] */
 #define MAX_AI_LOVE 1000
@@ -345,7 +386,7 @@ do {\
 /* User functions. */
 bool is_valid_username(const char *name);
 
-enum ai_level find_ai_level_by_name(const char *name);
+enum ai_level ai_level_by_name(const char *name);
 const char *ai_level_name(enum ai_level level);
 const char *ai_level_cmd(enum ai_level level);
 bool is_settable_ai_level(enum ai_level level);
