@@ -15,14 +15,17 @@
 #include <config.h>
 #endif
 
-#include <assert.h>
-
 /* utility */
+#include "bitvector.h"
 #include "log.h"
 #include "support.h"
 
 /* common */
+#include "fc_interface.h"
 #include "game.h"
+#include "movement.h"
+#include "unit.h"
+#include "unitlist.h"
 
 #include "tile.h"
 
@@ -62,7 +65,7 @@ struct tile *tile_claimer(const struct tile *ptile)
 void tile_set_owner(struct tile *ptile, struct player *pplayer,
                     struct tile *claimer)
 {
-  if (0 < game.info.borders) {
+  if (BORDERS_DISABLED != game.info.borders) {
     ptile->owner = pplayer;
     ptile->claimer = claimer;
   }
@@ -114,15 +117,14 @@ struct terrain *tile_terrain(const struct tile *ptile)
 ****************************************************************************/
 void tile_set_terrain(struct tile *ptile, struct terrain *pterrain)
 {
-  if (NULL != pterrain
-      && terrain_has_flag(pterrain, TER_NO_CITIES)
-      && NULL != tile_city(ptile)) {
-    freelog(LOG_ERROR, "At (%d, %d), the terrain \"%s\" (nb %d) doesn't "
-            "support cities, whereas \"%s\" (nb %d) is built there.",
-            TILE_XY(ptile), terrain_rule_name(pterrain),
-            terrain_number(pterrain), city_name(tile_city(ptile)),
-            tile_city(ptile)->id);
-  }
+  fc_assert_msg(NULL == pterrain
+                || !terrain_has_flag(pterrain, TER_NO_CITIES)
+                || NULL == tile_city(ptile),
+                "At (%d, %d), the terrain \"%s\" (nb %d) doesn't "
+                "support cities, whereas \"%s\" (nb %d) is built there.",
+                TILE_XY(ptile), terrain_rule_name(pterrain),
+                terrain_number(pterrain), city_name(tile_city(ptile)),
+                tile_city(ptile)->id);
 
   ptile->terrain = pterrain;
   if (NULL != pterrain
@@ -381,11 +383,11 @@ void tile_set_continent(struct tile *ptile, Continent_id val)
   Note that the client only has known data about its own player.
 ****************************************************************************/
 enum known_type tile_get_known(const struct tile *ptile,
-			       const struct player *pplayer)
+                               const struct player *pplayer)
 {
-  if (!BV_ISSET(ptile->tile_known, player_index(pplayer))) {
+  if (!dbv_isset(&pplayer->tile_known, tile_index(ptile))) {
     return TILE_UNKNOWN;
-  } else if (!BV_ISSET(ptile->tile_seen[V_MAIN], player_index(pplayer))) {
+  } else if (!fc_funcs->player_tile_vision_get(ptile, pplayer, V_MAIN)) {
     return TILE_KNOWN_UNSEEN;
   } else {
     return TILE_KNOWN_SEEN;
@@ -400,10 +402,11 @@ int tile_activity_time(enum unit_activity activity, const struct tile *ptile)
   struct terrain *pterrain = tile_terrain(ptile);
 
   /* Make sure nobody uses old activities */
-  assert(activity != ACTIVITY_FORTRESS && activity != ACTIVITY_AIRBASE);
+  fc_assert_ret_val(activity != ACTIVITY_FORTRESS
+                    && activity != ACTIVITY_AIRBASE, FC_INFINITY);
 
   /* ACTIVITY_BASE not handled here */
-  assert(activity != ACTIVITY_BASE);
+  fc_assert_ret_val(activity != ACTIVITY_BASE, FC_INFINITY);
 
   switch (activity) {
   case ACTIVITY_POLLUTION:
@@ -477,7 +480,6 @@ void tile_change_terrain(struct tile *ptile, struct terrain *pterrain)
 
   /* Clear mining/irrigation if resulting terrain type cannot support
    * that feature. */
-  
   if (pterrain->mining_result != pterrain) {
     tile_clear_special(ptile, S_MINE);
   }
@@ -502,7 +504,7 @@ void tile_change_terrain(struct tile *ptile, struct terrain *pterrain)
 ****************************************************************************/
 void tile_add_special(struct tile *ptile, enum tile_special_type special)
 {
-  assert(special != S_OLD_FORTRESS && special != S_OLD_AIRBASE);
+  fc_assert_ret(special != S_OLD_FORTRESS && special != S_OLD_AIRBASE);
 
   tile_set_special(ptile, special);
 
@@ -537,7 +539,7 @@ void tile_add_special(struct tile *ptile, enum tile_special_type special)
 ****************************************************************************/
 void tile_remove_special(struct tile *ptile, enum tile_special_type special)
 {
-  assert(special != S_OLD_FORTRESS && special != S_OLD_AIRBASE);
+  fc_assert_ret(special != S_OLD_FORTRESS && special != S_OLD_AIRBASE);
 
   tile_clear_special(ptile, special);
 
@@ -674,7 +676,7 @@ bool tile_apply_activity(struct tile *ptile, Activity_type_id act)
        on terrain type or tile specials */
     return FALSE;
   }
-  assert(0);
+  fc_assert(FALSE);
   return FALSE;
 }
 
@@ -690,15 +692,15 @@ static bool tile_info_pollution(char *buf, int bufsz,
   if (tile_has_special(ptile, special)) {
     if (!prevp) {
       if (linebreak) {
-        mystrlcat(buf, "\n[", bufsz);
+        fc_strlcat(buf, "\n[", bufsz);
       } else {
-        mystrlcat(buf, " [", bufsz);
+        fc_strlcat(buf, " [", bufsz);
       }
     } else {
-      mystrlcat(buf, "/", bufsz);
+      fc_strlcat(buf, "/", bufsz);
     }
 
-    mystrlcat(buf, special_name_translation(special), bufsz);
+    fc_strlcat(buf, special_name_translation(special), bufsz);
 
     return TRUE;
   }
@@ -786,16 +788,56 @@ bool tile_has_any_bases(const struct tile *ptile)
 }
 
 /****************************************************************************
-  Returns a completely blank virtual tile (except for the unit list
-  vtile->units, which is created for you). Be sure to call virtual_tile_free
+  Returns a virtual tile. If ptile is given, the properties of this tile are
+  copied, else it is completely blank (except for the unit list
+  vtile->units, which is created for you). Be sure to call tile_virtual_free
   on it when it is no longer needed.
 ****************************************************************************/
-struct tile *create_tile_virtual(void)
+struct tile *tile_virtual_new(const struct tile *ptile)
 {
   struct tile *vtile;
 
   vtile = fc_calloc(1, sizeof(*vtile));
+
+  /* initialise some values */
+  vtile->x = -1;
+  vtile->y = -1;
+  vtile->nat_x = -1;
+  vtile->nat_y = -1;
+  vtile->index = -1;
+  vtile->continent = -1;
+
+  BV_CLR_ALL(vtile->special);
+  BV_CLR_ALL(vtile->bases);
+  vtile->resource = NULL;
+  vtile->terrain = NULL;
   vtile->units = unit_list_new();
+  vtile->worked = NULL;
+  vtile->owner = NULL;
+  vtile->claimer = NULL;
+  vtile->spec_sprite = NULL;
+
+  if (ptile) {
+    /* Copy all but the unit list. */
+    tile_special_type_iterate(spe) {
+      if (BV_ISSET(ptile->special, spe)) {
+        BV_SET(vtile->special, spe);
+      }
+    } tile_special_type_iterate_end;
+
+    base_type_iterate(pbase) {
+      if (BV_ISSET(ptile->bases, base_number(pbase))) {
+        BV_SET(vtile->bases, base_number(pbase));
+      }
+    } base_type_iterate_end;
+
+    vtile->resource = ptile->resource;
+    vtile->terrain = ptile->terrain;
+    vtile->worked = ptile->worked;
+    vtile->owner = ptile->owner;
+    vtile->claimer = ptile->claimer;
+    vtile->spec_sprite = NULL;
+  }
 
   return vtile;
 }
@@ -807,7 +849,7 @@ struct tile *create_tile_virtual(void)
 
   NB: Do not call this on real tiles!
 ****************************************************************************/
-void destroy_tile_virtual(struct tile *vtile)
+void tile_virtual_destroy(struct tile *vtile)
 {
   struct city *vcity;
 
@@ -821,7 +863,7 @@ void destroy_tile_virtual(struct tile *vtile)
         destroy_unit_virtual(vunit);
       }
     } unit_list_iterate_end;
-    unit_list_free(vtile->units);
+    unit_list_destroy(vtile->units);
     vtile->units = NULL;
   }
 
