@@ -43,7 +43,7 @@
 	 _unit##_index < _unit##_size;					\
 	 _unit##_index++) {						\
       struct unit *_unit =						\
-	game_find_unit_by_number(_unit##_numbers[_unit##_index]);	\
+	game_unit_by_number(_unit##_numbers[_unit##_index]);		\
 									\
       if (NULL != _unit) {
 
@@ -76,4 +76,7 @@ bool units_can_unload(const struct unit_list *punits);
 bool units_have_activity_on_tile(const struct unit_list *punits,
 				 enum unit_activity activity);
 
+bool units_can_upgrade(const struct unit_list *punits);
+bool units_can_convert(const struct unit_list *punits);
+
 #endif  /* FC__UNITLIST_H */
