@@ -29,8 +29,8 @@
 
 struct command {
   const char *name;       /* name - will be matched by unique prefix   */
-  enum cmdlevel_id level; /* access level required to use the command  */
-  const char *synopsis;	  /* one or few-line summary of usage */
+  enum cmdlevel level;    /* access level required to use the command  */
+  const char *synopsis;   /* one or few-line summary of usage */
   const char *short_help; /* one line (about 70 chars) description */
   const char *extra_help; /* extra help information; will be line-wrapped */
   enum cmd_echo echo;     /* Who will be notified when used. */
@@ -44,14 +44,14 @@ static struct command commands[] = {
    /* no translatable parameters */
    SYN_ORIG_("start"),
    N_("Start the game, or restart after loading a savegame."),
-   N_("This command starts the game.  When starting a new game, "
+   N_("This command starts the game. When starting a new game, "
       "it should be used after all human players have connected, and "
       "AI players have been created (if required), and any desired "
-      "changes to initial server options have been made.  "
+      "changes to initial server options have been made. "
       "After 'start', each human player will be able to "
-      "choose their nation, and then the game will begin.  "
+      "choose their nation, and then the game will begin. "
       "This command is also required after loading a savegame "
-      "for the game to recommence.  Once the game is running this command "
+      "for the game to recommence. Once the game is running this command "
       "is no longer available, since it would have no effect."),
    CMD_ECHO_NONE, VCF_NONE, 0
   },
@@ -64,12 +64,12 @@ static struct command commands[] = {
       "help <command-name>\n"
       "help <option-name>"),
    N_("Show help about server commands and server options."),
-   N_("With no arguments gives some introductory help.  "
+   N_("With no arguments gives some introductory help. "
       "With argument \"commands\" or \"options\" gives respectively "
-      "a list of all commands or all options.  "
+      "a list of all commands or all options. "
       "Otherwise the argument is taken as a command name or option name, "
-      "and help is given for that command or option.  For options, the help "
-      "information includes the current and default values for that option.  "
+      "and help is given for that command or option. For options, the help "
+      "information includes the current and default values for that option. "
       "The argument may be abbreviated where unambiguous."),
    CMD_ECHO_NONE, VCF_NONE, 0
   },
@@ -77,14 +77,17 @@ static struct command commands[] = {
   {"list",	ALLOW_INFO,
    /* no translatable parameters */
    SYN_ORIG_("list\n"
-             "list players\n"
-             "list teams\n"
              "list connections\n"
-             "list scenarios"),
-   N_("Show a list of players, teams, connections, or scenarios."),
-   N_("Show a list of players in the game, teams of players, connections to "
-      "the server, or available scenarios.  The argument may be abbreviated,"
-      " and defaults to 'players' if absent."),
+             "list ignored users\n"
+             "list players\n"
+             "list scenarios\n"
+             "list teams\n"
+             "list votes\n"),
+   N_("Show a list of various things."),
+   N_("Show a list of connections to the server, your ignore list, "
+      "the list of the players in the game, the available scenarios, "
+      "the teams of players, or the running votes. The argument may be "
+      "abbreviated, and defaults to 'players' if absent."),
    CMD_ECHO_NONE, VCF_NONE, 0
   },
   {"quit",	ALLOW_HACK,
@@ -98,8 +101,8 @@ static struct command commands[] = {
    N_("cut <connection-name>"),
    N_("Cut a client's connection to server."),
    N_("Cut specified client's connection to the server, removing that client "
-      "from the game.  If the game has not yet started that client's player "
-      "is removed from the game, otherwise there is no effect on the player.  "
+      "from the game. If the game has not yet started that client's player "
+      "is removed from the game, otherwise there is no effect on the player. "
       "Note that this command now takes connection names, not player names."),
    CMD_ECHO_ALL, VCF_NONE, 50
   },
@@ -109,7 +112,7 @@ static struct command commands[] = {
       "explain <option-name>"),
    N_("Explain server options."),
    N_("The 'explain' command gives a subset of the functionality of 'help', "
-      "and is included for backward compatibility.  With no arguments it "
+      "and is included for backward compatibility. With no arguments it "
       "gives a list of options (like 'help options'), and with an argument "
       "it gives help for a particular option (like 'help <option-name>')."),
    CMD_ECHO_NONE, VCF_NONE, 0
@@ -123,15 +126,19 @@ static struct command commands[] = {
       "show vital\n"
       "show situational\n"
       "show rare\n"
-      "show changed"),
+      "show changed\n"
+      "show locked\n"
+      "show rulesetdir"),
    N_("Show server options."),
    N_("With no arguments, shows vital server options (or available options, "
       "when used by clients). With an option name argument, show only the "
       "named option, or options with that prefix. With \"all\", it shows "
       "all options. With \"vital\", \"situational\" or \"rare\", a set of "
       "options with this level. With \"changed\", it shows only the options "
-      "which have been modified."),
-   CMD_ECHO_NONE, VCF_NONE, 0
+      "which have been modified, while with \"locked\" all settings locked "
+      "by the ruleset will be listed. With \"ruleset\", it will show the "
+      "current ruleset directory name."),
+    CMD_ECHO_NONE, VCF_NONE, 0
   },
   {"wall",	ALLOW_ADMIN,
    /* TRANS: translate text between <> only */
@@ -156,29 +163,42 @@ static struct command commands[] = {
    N_("Cast a vote."),
       /* xgettext:no-c-format */
    N_("A player with basic level access issuing a control level command "
-      "starts a new vote for the command.  The /vote command followed by "
+      "starts a new vote for the command. The /vote command followed by "
       "\"yes\", \"no\", or \"abstain\", and optionally a vote number, "
-      "gives your vote.  If you do not add a vote number, your vote applies "
-      "to the latest vote.  You can only suggest one vote at a time.  "
+      "gives your vote. If you do not add a vote number, your vote applies "
+      "to the latest vote. You can only suggest one vote at a time. "
       "The vote will pass immediately if more than half of the voters "
       "who have not abstained vote for it, or fail immediately if at "
       "least half of the voters who have not abstained vote against it."),
    CMD_ECHO_NONE, VCF_NONE, 0
   },
   {"debug",	ALLOW_CTRL,
-   /* no translatable parameters */
-   SYN_ORIG_("debug [ diplomacy | ferries | player <player> | tech <player>"
-             " | city <x> <y> | units <x> <y> | unit <id> "
-             " | timing | info ]"),
+   /* TRANS: translate text between <> only */
+   N_("debug diplomacy <player>\n"
+      "debug ferries\n"
+      "debug player <player>\n"
+      "debug tech <player>\n"
+      "debug city <x> <y>\n"
+      "debug units <x> <y>\n"
+      "debug unit <id>\n"
+      "debug timing\n"
+      "debug info"),
    N_("Turn on or off AI debugging of given entity."),
-   N_("Print AI debug information about given entity and turn continous "
+   N_("Print AI debug information about given entity and turn continuous "
       "debugging output for this entity on or off."),
    CMD_ECHO_ADMINS, VCF_NONE, 0
   },
   {"set",	ALLOW_CTRL,
    /* TRANS: translate text between <> only */
    N_("set <option-name> <value>"),
-   N_("Set server option."), NULL,
+   N_("Set server option."),
+   /* TRANS: don't translate text in '' */
+   N_("Set an option on the server. The syntax and legal values depend "
+      "on the option; see the help for each option. Some options are "
+      "\"bitwise\", in that they consist of a choice from a set of values; "
+      "separate these with |, for instance, '/set topology wrapx|iso'. For "
+      "these options, use syntax like '/set topology \"\"' to set no "
+      "values."),
    CMD_ECHO_ALL, VCF_NONE, 50
   },
   {"team",	ALLOW_CTRL,
@@ -196,18 +216,16 @@ static struct command commands[] = {
    /* TRANS: translate text between <> only */
    N_("rulesetdir <directory>"),
    N_("Choose new ruleset directory or modpack."),
-   N_("Choose new ruleset directory or modpack. Calling this\n "
-      "without any arguments will show you the currently selected "
-      "ruleset."),
+   N_("Choose new ruleset directory or modpack."),
    CMD_ECHO_ALL, VCF_NONE, 50
   },
   {"metamessage", ALLOW_CTRL,
    /* TRANS: translate text between <> only */
    N_("metainfo <meta-line>"),
    N_("Set metaserver info line."),
-   N_("Set user defined metaserver info line. If parameter is omitted,\n"
-      "previously set metamessage will be removed. For most of the time\n"
-      "user defined metamessage will be used instead of automatically\n"
+   N_("Set user defined metaserver info line. If parameter is omitted, "
+      "previously set metamessage will be removed. For most of the time "
+      "user defined metamessage will be used instead of automatically "
       "generated messages, if it is available."),
    CMD_ECHO_ADMINS, VCF_NONE, 50
   },
@@ -223,8 +241,8 @@ static struct command commands[] = {
              "metaconnection d|down\n"
              "metaconnection ?"),
    N_("Control metaserver connection."),
-   N_("'metaconnection ?' reports on the status of the connection to metaserver.\n"
-      "'metaconnection down' or 'metac d' brings the metaserver connection down.\n"
+   N_("'metaconnection ?' reports on the status of the connection to metaserver. "
+      "'metaconnection down' or 'metac d' brings the metaserver connection down. "
       "'metaconnection up' or 'metac u' brings the metaserver connection up."),
    CMD_ECHO_ADMINS, VCF_NONE, 0
   },
@@ -246,7 +264,7 @@ static struct command commands[] = {
    N_("Take over a player's place in the game."),
    N_("Only the console and connections with cmdlevel 'hack' can force "
       "other connections to take over a player. If you're not one of these, "
-      "only the <player-name> argument is allowed.  If '-' is given for the "
+      "only the <player-name> argument is allowed. If '-' is given for the "
       "player name and the connection does not already control a player, one "
       "is created and assigned to the connection."),
    CMD_ECHO_ADMINS, VCF_NONE, 0
@@ -265,7 +283,7 @@ static struct command commands[] = {
   {"detach",    ALLOW_INFO,
    /* TRANS: translate text between <> only */
    N_("detach <connection-name>"),
-   N_("detach from a player."),
+   N_("Detach from a player."),
    N_("Only the console and connections with cmdlevel 'hack' can force "
       "other connections to detach from a player."),
    CMD_ECHO_ADMINS, VCF_NONE, 0
@@ -274,8 +292,11 @@ static struct command commands[] = {
    /* TRANS: translate text between <> only */
    N_("create <player-name>"),
    N_("Create an AI player with a given name."),
-   N_("The 'create' command is only available before the game has "
-      "been started."),
+   N_("With the 'create' command a new player with the given name is "
+      "created.\n"
+      "If the game was started, the command checks for free player slots "
+      "and, if no free slots are available, it tries to reuse the slots of "
+      "dead players. The new player has no units or cities."),
    CMD_ECHO_ALL, VCF_NONE, 50
   },
   {"away",	ALLOW_BASIC,
@@ -291,7 +312,7 @@ static struct command commands[] = {
       "novice <player-name>"),
    N_("Set one or all AI players to 'novice'."),
    N_("With no arguments, sets all AI players to skill level 'novice', and "
-      "sets the default level for any new AI players to 'novice'.  With an "
+      "sets the default level for any new AI players to 'novice'. With an "
       "argument, sets the skill level for that player only."),
    CMD_ECHO_ALL, VCF_NONE, 50
   },
@@ -301,7 +322,7 @@ static struct command commands[] = {
       "easy <player-name>"),
    N_("Set one or all AI players to 'easy'."),
    N_("With no arguments, sets all AI players to skill level 'easy', and "
-      "sets the default level for any new AI players to 'easy'.  With an "
+      "sets the default level for any new AI players to 'easy'. With an "
       "argument, sets the skill level for that player only."),
    CMD_ECHO_ALL, VCF_NONE, 50
   },
@@ -311,7 +332,7 @@ static struct command commands[] = {
       "normal <player-name>"),
    N_("Set one or all AI players to 'normal'."),
    N_("With no arguments, sets all AI players to skill level 'normal', and "
-      "sets the default level for any new AI players to 'normal'.  With an "
+      "sets the default level for any new AI players to 'normal'. With an "
       "argument, sets the skill level for that player only."),
    CMD_ECHO_ALL, VCF_NONE, 50
   },
@@ -321,7 +342,7 @@ static struct command commands[] = {
       "hard <player-name>"),
    N_("Set one or all AI players to 'hard'."),
    N_("With no arguments, sets all AI players to skill level 'hard', and "
-      "sets the default level for any new AI players to 'hard'.  With an "
+      "sets the default level for any new AI players to 'hard'. With an "
       "argument, sets the skill level for that player only."),
    CMD_ECHO_ALL, VCF_NONE, 50
   },
@@ -331,7 +352,7 @@ static struct command commands[] = {
       "cheating <player-name>"),
    N_("Set one or all AI players to 'cheating'."),
    N_("With no arguments, sets all AI players to skill level 'cheating', and "
-      "sets the default level for any new AI players to 'cheating'.  With an "
+      "sets the default level for any new AI players to 'cheating'. With an "
       "argument, sets the skill level for that player only."),
    CMD_ECHO_ALL, VCF_NONE, 50
   },
@@ -341,7 +362,7 @@ static struct command commands[] = {
       "experimental <player-name>"),
    N_("Set one or all AI players to 'experimental'."),
    N_("With no arguments, sets all AI players to skill 'experimental', and "
-      "sets the default level for any new AI players to this.  With an "
+      "sets the default level for any new AI players to this. With an "
       "argument, sets the skill level for that player only. THIS IS ONLY "
       "FOR TESTING OF NEW AI FEATURES! For ordinary servers, this option "
       "has no effect."),
@@ -355,25 +376,25 @@ static struct command commands[] = {
       "cmdlevel <level> first\n"
       "cmdlevel <level> <connection-name>"),
    N_("Query or set command access level access."),
-   N_("The command access level controls which server commands are available\n"
-      "to users via the client chatline.  The available levels are:\n"
+   N_("The command access level controls which server commands are available "
+      "to users via the client chatline. The available levels are:\n"
       "    none  -  no commands\n"
       "    info  -  informational or observer commands only\n"
       "    basic -  commands available to players in the game\n"
       "    ctrl  -  commands that affect the game and users\n"
       "    admin -  commands that affect server operation\n"
       "    hack  -  *all* commands - dangerous!\n"
-      "With no arguments, the current command access levels are reported.\n"
+      "With no arguments, the current command access levels are reported. "
       "With a single argument, the level is set for all existing "
-      "connections,\nand the default is set for future connections.\n"
-      "If 'new' is specified, the level is set for newly connecting clients.\n"
-      "If 'first come' is specified, the 'first come' level is set; it will be\n"
-      "granted to the first client to connect, or if there are connections\n"
-      "already, the first client to issue the 'first' command.\n"
+      "connections, and the default is set for future connections. "
+      "If 'new' is specified, the level is set for newly connecting clients. "
+      "If 'first come' is specified, the 'first come' level is set; it will be "
+      "granted to the first client to connect, or if there are connections "
+      "already, the first client to issue the 'first' command. "
       "If a connection name is specified, the level is set for that "
       "connection only.\n"
       "Command access levels do not persist if a client disconnects, "
-      "because some untrusted person could reconnect with the same name.  "
+      "because some untrusted person could reconnect with the same name. "
       "Note that this command now takes connection names, not player names."
       ),
    CMD_ECHO_ADMINS, VCF_NONE, 50
@@ -390,7 +411,7 @@ static struct command commands[] = {
    N_("timeoutincrease <turn> <turninc> <value> <valuemult>"), 
    N_("See \"help timeoutincrease\"."),
    N_("Every <turn> turns, add <value> to timeout timer, then add <turninc> "
-      "to <turn> and multiply <value> by <valuemult>.  Use this command in "
+      "to <turn> and multiply <value> by <valuemult>. Use this command in "
       "concert with the option \"timeout\". Defaults are 0 0 0 1"),
    CMD_ECHO_ALL, VCF_NONE, 50
   },
@@ -402,11 +423,37 @@ static struct command commands[] = {
    /* TRANS: "vote" as a process */
    N_("Cancel a running vote.\n"),
    /* TRANS: "vote" as a process */
-   N_("With no arguments this command removes your own vote.  If you have "
+   N_("With no arguments this command removes your own vote. If you have "
       "an admin access level, you can cancel any vote by vote number, or "
       "all votes with the \'all\' argument."),
    CMD_ECHO_ADMINS, VCF_NONE, 0
   },
+  {"ignore", ALLOW_INFO,
+   /* TRANS: translate text between <> and [] only */
+   N_("ignore [type=]<pattern>"),
+   N_("Block all messages from users matching the pattern."),
+   N_("The given pattern will be added to your ignore list; you will not "
+      "receive any messages from this users matching this pattern. The type "
+      "may be either \"user\", \"host\", or \"ip\". The default type "
+      "(if ommited) is to match against the username. The pattern supports "
+      "unix glob style wildcards, i.e. * matches zero or more character, ? "
+      "exactly one character, [abc] exactly one of 'a' 'b' or 'c', etc. "
+      "To access your current ignore list, issue \"/list ignore\"."),
+   CMD_ECHO_NONE, VCF_NONE, 0
+  },
+  {"unignore", ALLOW_INFO,
+   /* TRANS: translate text between <> */
+   N_("unignore <range>"),
+   N_("Remove ignore list entries."),
+   N_("The ignore list entries in the given range will be removed; "
+      "you will be able to receive messages from the respective users. "
+      "The range argument may be a single number or a pair of numbers "
+      "separated by a dash '-'. If the first number is ommitted, it is "
+      "assumed to be 1, if the last is ommitted, it is assumed to be "
+      "the last valid ignore list index. To access your current ignore "
+      "list, issue \"/list ignore\"."),
+   CMD_ECHO_NONE, VCF_NONE, 0
+  },
   {"endgame",	ALLOW_ADMIN,
    /* no translatable parameters */
    SYN_ORIG_("endgame"),
@@ -427,7 +474,7 @@ static struct command commands[] = {
    N_("remove <player-name>"),
    N_("Fully remove player from game."),
    N_("This *completely* removes a player from the game, including "
-      "all cities and units etc.  Use with care!"),
+      "all cities and units etc. Use with care!"),
    CMD_ECHO_ALL, VCF_NONE, 50
   },
   {"save",	ALLOW_ADMIN,
@@ -435,11 +482,11 @@ static struct command commands[] = {
    N_("save\n"
       "save <file-name>"),
    N_("Save game to file."),
-   N_("Save the current game to file <file-name>.  If no file-name "
-      "argument is given saves to \"<auto-save name prefix><year>m.sav[.gz]\".\n"
+   N_("Save the current game to file <file-name>. If no file-name "
+      "argument is given saves to \"<auto-save name prefix><year>m.sav[.gz]\". "
       "To reload a savegame created by 'save', start the server with "
       "the command-line argument:\n"
-      "    --file <filename>\n"
+      "    '--file <filename>' or '-f <filename>'\n"
       "and use the 'start' command once players have reconnected."),
    CMD_ECHO_ADMINS, VCF_NONE, 0
   },
@@ -449,7 +496,7 @@ static struct command commands[] = {
       "load <file-name>"),
    N_("Load game from file."),
    N_("Load a game from <file-name>. Any current data including players, "
-      "rulesets and server options are lost.\n"),
+      "rulesets and server options are lost."),
    CMD_ECHO_ADMINS, VCF_NONE, 50
   },
   {"read",	ALLOW_CTRL,
@@ -468,10 +515,29 @@ static struct command commands[] = {
    /* no translatable parameters */
    SYN_ORIG_("reset"),
    N_("Reset all server settings."),
-   N_("Reset all settings and re-read the server start script, "
-      "if there was one given with the --read server argument. "),
+   N_("Reset all settings if it is possible. The following levels are "
+      "supported:\n"
+      "  game     - using the values defined at the game start\n"
+      "  ruleset  - using the values defined in the ruleset\n"
+      "  script   - using default values and rereading the start script\n"
+      "  default  - using default values\n"),
    CMD_ECHO_ALL, VCF_NONE, 50
   },
+  {"lua", ALLOW_ADMIN,
+   /* TRANS: translate text between <> only */
+   N_("lua <script>"),
+   N_("Evaluate a line of freeciv script in the current game."), NULL,
+   CMD_ECHO_ADMINS, VCF_NONE, 0
+  },
+  {"kick", ALLOW_CTRL,
+   /* TRANS: translate text between <> */
+    N_("kick <user>"),
+    N_("Cut a connection and disallow reconnect."),
+    N_("The connection given by the 'user' argument will be cut from the "
+       "server and not allowed to reconnect. The time the user wouldn't be "
+       "able to reconnect is controlled by the 'kicktime' setting."),
+   CMD_ECHO_ADMINS, VCF_NOPASSALONE, 50
+  },
   {"rfcstyle",	ALLOW_HACK,
    /* no translatable parameters */
    SYN_ORIG_("rfcstyle"),
@@ -492,7 +558,7 @@ static struct command commands[] = {
 **************************************************************************/
 const struct command *command_by_number(int i)
 {
-  assert(i >= 0 && i < CMD_NUM);
+  fc_assert_ret_val(i >= 0 && i < CMD_NUM, NULL);
   return &commands[i];
 }
 
@@ -539,7 +605,7 @@ const char *command_extra_help(const struct command *pcommand)
 /**************************************************************************
   ...
 **************************************************************************/
-enum cmdlevel_id command_level(const struct command *pcommand)
+enum cmdlevel command_level(const struct command *pcommand)
 {
   return pcommand->level;
 }
