@@ -25,7 +25,7 @@
   pointing to some data if the token was found, or NULL otherwise.
   The data pointed to should not be modified.  The retuned pointer
   is valid _only_ until another inputfile is performed.  (So should
-  be used immediately, or mystrdup-ed etc.)
+  be used immediately, or fc_strdup-ed etc.)
   
   The tokens recognised are as follows:
   (Single quotes are delimiters used here, but are not part of the
@@ -67,7 +67,7 @@
 #include <config.h>
 #endif
 
-#include <assert.h>
+#include <stdarg.h>
 #include <stdio.h>
 #include <string.h>
 
@@ -90,10 +90,7 @@ struct inputfile {
   char *filename;		/* filename as passed to fopen */
   fz_FILE *fp;			/* read from this */
   bool at_eof;			/* flag for end-of-file */
-  struct astring cur_line;	/* data from current line, or .n==0 if
-				   have not yet read in the current line */
-  struct astring copy_line;	/* original cur_line (sometimes insert nulls
-				   in cur_line for processing) */
+  struct astring cur_line;	/* data from current line */
   int cur_line_pos;		/* position in current line */
   int line_num;			/* line number from file in cur_line */
   struct astring token;		/* data returned to user */
@@ -140,7 +137,14 @@ tok_tab[INF_TOK_LAST] =
 };
 
 static bool read_a_line(struct inputfile *inf);
-static void inf_warn(struct inputfile *inf, const char *message);
+
+#define inf_log(inf, level, message, ...)                                   \
+  if (log_do_output_for_level(level)) {                                     \
+    do_log(__FILE__, __FUNCTION__, __LINE__, FALSE, level, "%s",            \
+           inf_log_str(inf, message, ## __VA_ARGS__));                      \
+  }
+#define inf_warn(inf, message)                                              \
+  inf_log(inf, LOG_NORMAL, "%s", message);
 
 /********************************************************************** 
   Return true if c is a 'comment' character: '#' or ';'
@@ -156,7 +160,7 @@ static bool is_comment(int c)
 ***********************************************************************/
 static void init_zeros(struct inputfile *inf)
 {
-  assert(inf != NULL);
+  fc_assert_ret(NULL != inf);
   inf->magic = INF_MAGIC;
   inf->filename = NULL;
   inf->fp = NULL;
@@ -166,7 +170,6 @@ static void init_zeros(struct inputfile *inf)
   inf->at_eof = inf->in_string = FALSE;
   inf->string_start_line = 0;
   astr_init(&inf->cur_line);
-  astr_init(&inf->copy_line);
   astr_init(&inf->token);
   astr_init(&inf->partial);
 }
@@ -174,29 +177,24 @@ static void init_zeros(struct inputfile *inf)
 /********************************************************************** 
   Check sensible values for an opened inputfile.
 ***********************************************************************/
-static void assert_sanity(struct inputfile *inf)
+static bool inf_sanity_check(struct inputfile *inf)
 {
-  assert(inf != NULL);
-  assert(inf->magic==INF_MAGIC);
-  assert(inf->fp != NULL);
-  assert(inf->line_num >= 0);
-  assert(inf->cur_line_pos >= 0);
-  assert(inf->at_eof == FALSE || inf->at_eof == TRUE);
-  assert(inf->in_string == FALSE || inf->in_string == TRUE);
+  fc_assert_ret_val(NULL != inf, FALSE);
+  fc_assert_ret_val(INF_MAGIC == inf->magic, FALSE);
+  fc_assert_ret_val(NULL != inf->fp, FALSE);
+  fc_assert_ret_val(0 <= inf->line_num, FALSE);
+  fc_assert_ret_val(0 <= inf->cur_line_pos, FALSE);
+  fc_assert_ret_val(FALSE == inf->at_eof
+                    || TRUE == inf->at_eof, FALSE);
+  fc_assert_ret_val(FALSE == inf->in_string
+                    || TRUE == inf->in_string, FALSE);
 #ifdef DEBUG
-  assert(inf->string_start_line >= 0);
-  assert(inf->cur_line.n >= 0);
-  assert(inf->copy_line.n >= 0);
-  assert(inf->token.n >= 0);
-  assert(inf->partial.n >= 0);
-  assert(inf->cur_line.n_alloc >= 0);
-  assert(inf->copy_line.n_alloc >= 0);
-  assert(inf->token.n_alloc >= 0);
-  assert(inf->partial.n_alloc >= 0);
-  if(inf->included_from) {
-    assert_sanity(inf->included_from);
+  fc_assert_ret_val(0 <= inf->string_start_line, FALSE);
+  if (inf->included_from && !inf_sanity_check(inf->included_from)) {
+    return FALSE;
   }
 #endif
+  return TRUE;
 }
 
 /**************************************************************************
@@ -217,20 +215,20 @@ static const char *inf_filename(struct inputfile *inf)
   Returns NULL if the file could not be opened.
 ***********************************************************************/
 struct inputfile *inf_from_file(const char *filename,
-				datafilename_fn_t datafn)
+                                datafilename_fn_t datafn)
 {
   struct inputfile *inf;
   fz_FILE *fp;
 
-  assert(filename != NULL);
-  assert(strlen(filename) > 0);
-  fp = fz_from_file(filename, "r", FZ_NOT_USED, FZ_NOT_USED);
+  fc_assert_ret_val(NULL != filename, NULL);
+  fc_assert_ret_val(0 < strlen(filename), NULL);
+  fp = fz_from_file(filename, "r", -1, 0);
   if (!fp) {
     return NULL;
   }
-  freelog(LOG_DEBUG, "inputfile: opened \"%s\" ok", filename);
+  log_debug("inputfile: opened \"%s\" ok", filename);
   inf = inf_from_stream(fp, datafn);
-  inf->filename = mystrdup(filename);
+  inf->filename = fc_strdup(filename);
   return inf;
 }
 
@@ -242,7 +240,7 @@ struct inputfile *inf_from_stream(fz_FILE * stream, datafilename_fn_t datafn)
 {
   struct inputfile *inf;
 
-  assert(stream != NULL);
+  fc_assert_ret_val(NULL != stream, NULL);
   inf = fc_malloc(sizeof(*inf));
   init_zeros(inf);
   
@@ -250,7 +248,7 @@ struct inputfile *inf_from_stream(fz_FILE * stream, datafilename_fn_t datafn)
   inf->fp = stream;
   inf->datafn = datafn;
 
-  freelog(LOG_DEBUG, "inputfile: opened \"%s\" ok", inf_filename(inf));
+  log_debug("inputfile: opened \"%s\" ok", inf_filename(inf));
   return inf;
 }
 
@@ -263,25 +261,24 @@ struct inputfile *inf_from_stream(fz_FILE * stream, datafilename_fn_t datafn)
 ***********************************************************************/
 static void inf_close_partial(struct inputfile *inf)
 {
-  assert_sanity(inf);
+  fc_assert_ret(inf_sanity_check(inf));
 
-  freelog(LOG_DEBUG, "inputfile: sub-closing \"%s\"", inf_filename(inf));
+  log_debug("inputfile: sub-closing \"%s\"", inf_filename(inf));
 
   if (fz_ferror(inf->fp) != 0) {
-    freelog(LOG_ERROR, "Error before closing %s: %s", inf_filename(inf),
-	    fz_strerror(inf->fp));
+    log_error("Error before closing %s: %s", inf_filename(inf),
+              fz_strerror(inf->fp));
     fz_fclose(inf->fp);
     inf->fp = NULL;
   }
   else if (fz_fclose(inf->fp) != 0) {
-    freelog(LOG_ERROR, "Error closing %s", inf_filename(inf));
+    log_error("Error closing %s", inf_filename(inf));
   }
   if (inf->filename) {
     free(inf->filename);
   }
   inf->filename = NULL;
   astr_free(&inf->cur_line);
-  astr_free(&inf->copy_line);
   astr_free(&inf->token);
   astr_free(&inf->partial);
 
@@ -289,7 +286,7 @@ static void inf_close_partial(struct inputfile *inf)
   init_zeros(inf);
   inf->magic = ~INF_MAGIC;
 
-  freelog(LOG_DEBUG, "inputfile: sub-closed ok");
+  log_debug("inputfile: sub-closed ok");
 }
 
 /********************************************************************** 
@@ -300,42 +297,42 @@ static void inf_close_partial(struct inputfile *inf)
 ***********************************************************************/
 void inf_close(struct inputfile *inf)
 {
-  assert_sanity(inf);
+  fc_assert_ret(inf_sanity_check(inf));
 
-  freelog(LOG_DEBUG, "inputfile: closing \"%s\"", inf_filename(inf));
+  log_debug("inputfile: closing \"%s\"", inf_filename(inf));
   if (inf->included_from) {
     inf_close(inf->included_from);
   }
   inf_close_partial(inf);
   free(inf);
-  freelog(LOG_DEBUG, "inputfile: closed ok");
+  log_debug("inputfile: closed ok");
 }
 
 /********************************************************************** 
-  Return 1 if have data for current line.
+  Return TRUE if have data for current line.
 ***********************************************************************/
 static bool have_line(struct inputfile *inf)
 {
-  assert_sanity(inf);
-  return (inf->cur_line.n > 0);
+  fc_assert_ret_val(inf_sanity_check(inf), FALSE);
+  return !astr_empty(&inf->cur_line);
 }
 
 /********************************************************************** 
-  Return 1 if current pos is at end of current line.
+  Return TRUE if current pos is at end of current line.
 ***********************************************************************/
 static bool at_eol(struct inputfile *inf)
 {
-  assert_sanity(inf);
-  assert(inf->cur_line_pos <= inf->cur_line.n);
-  return (inf->cur_line_pos >= inf->cur_line.n - 1);
+  fc_assert_ret_val(inf_sanity_check(inf), TRUE);
+  fc_assert_ret_val(inf->cur_line_pos <= astr_len(&inf->cur_line), TRUE);
+  return (inf->cur_line_pos >= astr_len(&inf->cur_line));
 }
 
 /********************************************************************** 
-  ...
+  Return TRUE if current pos is at end of file.
 ***********************************************************************/
 bool inf_at_eof(struct inputfile *inf)
 {
-  assert_sanity(inf);
+  fc_assert_ret_val(inf_sanity_check(inf), TRUE);
   return inf->at_eof;
 }
 
@@ -351,26 +348,31 @@ static bool check_include(struct inputfile *inf)
 {
   const char *include_prefix = "*include";
   static size_t len = 0;
-  char *bare_name, *full_name, *c;
+  size_t bare_name_len;
+  char *bare_name;
+  const char *c, *bare_name_start, *full_name;
   struct inputfile *new_inf, temp;
 
   if (len==0) {
     len = strlen(include_prefix);
   }
-  assert_sanity(inf);
-  if (inf->in_string || inf->cur_line.n <= len || inf->cur_line_pos > 0) {
+  fc_assert_ret_val(inf_sanity_check(inf), FALSE);
+  if (inf->in_string || astr_len(&inf->cur_line) <= len
+      || inf->cur_line_pos > 0) {
     return FALSE;
   }
-  if (strncmp(inf->cur_line.str, include_prefix, len)!=0) {
+  if (strncmp(astr_str(&inf->cur_line), include_prefix, len) != 0) {
     return FALSE;
   }
-  /* from here, the include-line must be well formed or we die */
+  /* from here, the include-line must be well formed */
   /* keep inf->cur_line_pos accurate just so error messages are useful */
 
   /* skip any whitespace: */
   inf->cur_line_pos = len;
-  c = inf->cur_line.str + len;
-  while (*c != '\0' && my_isspace(*c)) c++;
+  c = astr_str(&inf->cur_line) + len;
+  while (*c != '\0' && fc_isspace(*c)){
+    c++;
+  }
 
   if (*c != '\"') {
     inf_log(inf, LOG_ERROR, 
@@ -378,45 +380,52 @@ static bool check_include(struct inputfile *inf)
     return FALSE;
   }
   c++;
-  inf->cur_line_pos = c - inf->cur_line.str;
+  inf->cur_line_pos = c - astr_str(&inf->cur_line);
 
-  bare_name = c;
+  bare_name_start = c;
   while (*c != '\0' && *c != '\"') c++;
   if (*c != '\"') {
     inf_log(inf, LOG_ERROR, 
             "Did not find closing doublequote for '*include' line");
     return FALSE;
   }
-  *c++ = '\0';
-  inf->cur_line_pos = c - inf->cur_line.str;
+  c++;
+  bare_name_len = c - bare_name_start;
+  bare_name = fc_malloc(bare_name_len);
+  strncpy(bare_name, bare_name_start, bare_name_len - 1);
+  bare_name[bare_name_len - 1] = '\0';
+  inf->cur_line_pos = c - astr_str(&inf->cur_line);
 
   /* check rest of line is well-formed: */
-  while (*c != '\0' && my_isspace(*c) && !is_comment(*c)) c++;
-  if (!(*c=='\0' || is_comment(*c))) {
+  while (*c != '\0' && fc_isspace(*c) && !is_comment(*c)) {
+    c++;
+  }
+  if (!(*c == '\0' || is_comment(*c))) {
     inf_log(inf, LOG_ERROR, "Junk after filename for '*include' line");
     return FALSE;
   }
-  inf->cur_line_pos = inf->cur_line.n-1;
+  inf->cur_line_pos = astr_len(&inf->cur_line) - 1;
 
   full_name = inf->datafn(bare_name);
   if (!full_name) {
-    freelog(LOG_ERROR, "Could not find included file \"%s\"", bare_name);
+    log_error("Could not find included file \"%s\"", bare_name);
+    free(bare_name);
     return FALSE;
   }
+  free(bare_name);
 
   /* avoid recursion: (first filename may not have the same path,
-     but will at least stop infinite recursion) */
+   * but will at least stop infinite recursion) */
   {
     struct inputfile *inc = inf;
     do {
-      if (inc->filename && strcmp(full_name, inc->filename)==0) {
-        freelog(LOG_ERROR, 
-                "Recursion trap on '*include' for \"%s\"", full_name);
+      if (inc->filename && strcmp(full_name, inc->filename) == 0) {
+        log_error("Recursion trap on '*include' for \"%s\"", full_name);
         return FALSE;
       }
-    } while((inc=inc->included_from));
+    } while ((inc = inc->included_from));
   }
-  
+
   new_inf = inf_from_file(full_name, inf->datafn);
 
   /* Swap things around so that memory pointed to by inf (user pointer,
@@ -432,7 +441,7 @@ static bool check_include(struct inputfile *inf)
 }
 
 /********************************************************************** 
-  Read a new line into cur_line; also copy to copy_line.
+  Read a new line into cur_line.
   Increments line_num and cur_line_pos.
   Returns 0 if didn't read or other problem: treat as EOF.
   Strips newline from input.
@@ -442,29 +451,29 @@ static bool read_a_line(struct inputfile *inf)
   struct astring *line;
   char *ret;
   int pos;
-  
-  assert_sanity(inf);
 
-  if (inf->at_eof)
+  fc_assert_ret_val(inf_sanity_check(inf), FALSE);
+
+  if (inf->at_eof) {
     return FALSE;
-  
+  }
+
   /* abbreviation: */
   line = &inf->cur_line;
-  
+
   /* minimum initial line length: */
-  astr_minsize(line, 80);
+  astr_reserve(line, 80);
+  astr_clear(line);
   pos = 0;
 
-  /* don't print "orig line" in warnings until we have it: */
-  inf->copy_line.n = 0;
-  
   /* Read until we get a full line:
    * At start of this loop, pos is index to trailing null
    * (or first position) in line.
    */
   for(;;) {
-    ret = fz_fgets(line->str + pos, line->n_alloc - pos, inf->fp);
-    
+    ret = fz_fgets((char *) astr_str(line) + pos,
+                   astr_capacity(line) - pos, inf->fp);
+
     if (!ret) {
       /* fgets failed */
       inf->at_eof = TRUE;
@@ -476,36 +485,26 @@ static bool read_a_line(struct inputfile *inf)
       }
       break;
     }
-    
-    pos += strlen(line->str + pos);
-    line->n = pos + 1;
-    
-    if (line->str[pos-1] == '\n') {
-      line->str[pos-1] = '\0';
-      line->n--;
+
+    pos = astr_len(line);
+
+    if (0 < pos && astr_str(line)[pos - 1] == '\n') {
+      *((char *) astr_str(line) + pos - 1) = '\0';
       break;
     }
-    if (line->n != line->n_alloc) {
-      freelog(LOG_VERBOSE, "inputfile: expect missing newline at EOF");
-    }
-    astr_minsize(line, line->n*2);
+    astr_reserve(line, pos * 2);
   }
 
   if (!inf->at_eof) {
     inf->line_num++;
     inf->cur_line_pos = 0;
 
-    astr_minsize(&inf->copy_line,
-                 inf->cur_line.n + ((inf->cur_line.n == 0) ? 1 : 0));
-    strcpy(inf->copy_line.str, inf->cur_line.str);
-
     if (check_include(inf)) {
       return read_a_line(inf);
     }
     return TRUE;
   } else {
-    line->str[0] = '\0';
-    line->n = 0;
+    astr_clear(line);
     if (inf->included_from) {
       /* Pop the include, and get next line from file above instead. */
       struct inputfile *inc = inf->included_from;
@@ -519,59 +518,46 @@ static bool read_a_line(struct inputfile *inf)
   }
 }
 
-/********************************************************************** 
-  Set "flag" token when we don't really want to return anything,
-  except non-null.
+/**********************************************************************
+  Return a detailed log message, including information on current line
+  number etc. Message can be NULL: then just logs information on where
+  we are in the file.
 ***********************************************************************/
-static void assign_flag_token(struct astring *astr, char val)
+char *inf_log_str(struct inputfile *inf, const char *message, ...)
 {
-  static char flag_token[2];
+  va_list args;
+  static char str[512];
 
-  assert(astr != NULL);
-  flag_token[0] = val;
-  flag_token[1] = '\0';
-  astr_minsize(astr, 2);
-  strcpy(astr->str, flag_token);
-}
-
-/********************************************************************** 
-  Give a detailed log message, including information on
-  current line number etc.  Message can be NULL: then just logs
-  information on where we are in the file.
-***********************************************************************/
-void inf_log(struct inputfile *inf, int loglevel, const char *message)
-{
-  assert_sanity(inf);
+  fc_assert_ret_val(inf_sanity_check(inf), NULL);
 
   if (message) {
-    freelog(loglevel, "%s", message);
+    va_start(args, message);
+    fc_vsnprintf(str, sizeof(str), message, args);
+    va_end(args);
+    sz_strlcat(str, "\n");
+  } else {
+    str[0] = '\0';
   }
-  freelog(loglevel, "  file \"%s\", line %d, pos %d%s",
-	  inf_filename(inf), inf->line_num, inf->cur_line_pos,
-	  (inf->at_eof ? ", EOF" : ""));
-  if (inf->cur_line.str && inf->cur_line.n > 0) {
-    freelog(loglevel, "  looking at: '%s'",
-	    inf->cur_line.str+inf->cur_line_pos);
-  }
-  if (inf->copy_line.str && inf->copy_line.n > 0) {
-    freelog(loglevel, "  original line: '%s'", inf->copy_line.str);
+
+  cat_snprintf(str, sizeof(str), "  file \"%s\", line %d, pos %d%s",
+               inf_filename(inf), inf->line_num, inf->cur_line_pos,
+               (inf->at_eof ? ", EOF" : ""));
+
+  if (!astr_empty(&inf->cur_line)) {
+    cat_snprintf(str, sizeof(str), "\n  looking at: '%s'",
+                 astr_str(&inf->cur_line) + inf->cur_line_pos);
   }
   if (inf->in_string) {
-    freelog(loglevel, "  processing string starting at line %d",
-	    inf->string_start_line);
+    cat_snprintf(str, sizeof(str),
+                 "\n  processing string starting at line %d",
+                 inf->string_start_line);
   }
-  while ((inf=inf->included_from)) {    /* local pointer assignment */
-    freelog(loglevel, "  included from file \"%s\", line %d",
-	    inf_filename(inf), inf->line_num);
+  while ((inf = inf->included_from)) {  /* local pointer assignment */
+    cat_snprintf(str, sizeof(str), "\n  included from file \"%s\", line %d",
+                 inf_filename(inf), inf->line_num);
   }
-}
 
-/********************************************************************** 
-  ...
-***********************************************************************/
-static void inf_warn(struct inputfile *inf, const char *message)
-{
-  inf_log(inf, LOG_NORMAL, message);
+  return str;
 }
 
 /********************************************************************** 
@@ -582,19 +568,20 @@ const char *inf_token(struct inputfile *inf, enum inf_token_type type)
   const char *c;
   const char *name;
   get_token_fn_t func;
-  
-  assert_sanity(inf);
-  assert(type>=INF_TOK_FIRST && type<INF_TOK_LAST);
+
+  fc_assert_ret_val(inf_sanity_check(inf), NULL);
+  fc_assert_ret_val(INF_TOK_FIRST <= type && INF_TOK_LAST > type, NULL);
 
   name = tok_tab[type].name ? tok_tab[type].name : "(unnamed)";
   func = tok_tab[type].func;
   
   if (!func) {
-    freelog(LOG_ERROR, "token type %d (%s) not supported yet", type, name);
+    log_error("token type %d (%s) not supported yet", type, name);
     c = NULL;
   } else {
-    if (!have_line(inf))
-      (void) read_a_line(inf);
+    while (!have_line(inf) && read_a_line(inf)) {
+      /* Nothing. */
+    }
     if (!have_line(inf)) {
       c = NULL;
     } else {
@@ -602,7 +589,7 @@ const char *inf_token(struct inputfile *inf, enum inf_token_type type)
     }
   }
   if (c && INF_DEBUG_FOUND) {
-    freelog(LOG_DEBUG, "inputfile: found %s '%s'", name, inf->token.str);
+    log_debug("inputfile: found %s '%s'", name, astr_str(&inf->token));
   }
   return c;
 }
@@ -625,24 +612,26 @@ int inf_discard_tokens(struct inputfile *inf, enum inf_token_type type)
 ***********************************************************************/
 static const char *get_token_section_name(struct inputfile *inf)
 {
-  char *c, *start;
-  
-  assert(have_line(inf));
+  const char *c, *start;
 
-  c = inf->cur_line.str + inf->cur_line_pos;
-  if (*c++ != '[')
+  fc_assert_ret_val(have_line(inf), NULL);
+
+  c = astr_str(&inf->cur_line) + inf->cur_line_pos;
+  if (*c++ != '[') {
     return NULL;
+  }
   start = c;
   while (*c != '\0' && *c != ']') {
     c++;
   }
-  if (*c != ']')
+  if (*c != ']') {
     return NULL;
-  *c++ = '\0';
-  inf->cur_line_pos = c - inf->cur_line.str;
-  astr_minsize(&inf->token, strlen(start)+1);
-  strcpy(inf->token.str, start);
-  return inf->token.str;
+  }
+  *((char *) c) = '\0'; /* Tricky. */
+  astr_set(&inf->token, "%s", start);
+  *((char *) c) = ']';  /* Revert. */
+  inf->cur_line_pos = c + 1 - astr_str(&inf->cur_line);
+  return astr_str(&inf->token);
 }
 
 /********************************************************************** 
@@ -650,22 +639,25 @@ static const char *get_token_section_name(struct inputfile *inf)
 ***********************************************************************/
 static const char *get_token_entry_name(struct inputfile *inf)
 {
-  char *c, *start, *end;
-  
-  assert(have_line(inf));
+  const char *c, *start, *end;
+  char trailing;
 
-  c = inf->cur_line.str + inf->cur_line_pos;
-  while(*c != '\0' && my_isspace(*c)) {
+  fc_assert_ret_val(have_line(inf), NULL);
+
+  c = astr_str(&inf->cur_line) + inf->cur_line_pos;
+  while (*c != '\0' && fc_isspace(*c)) {
     c++;
   }
-  if (*c == '\0')
+  if (*c == '\0') {
     return NULL;
+  }
   start = c;
-  while (*c != '\0' && !my_isspace(*c) && *c != '=' && !is_comment(*c)) {
+  while (*c != '\0' && !fc_isspace(*c) && *c != '=' && !is_comment(*c)) {
     c++;
   }
-  if (!(*c != '\0' && (my_isspace(*c) || *c == '='))) 
+  if (!(*c != '\0' && (fc_isspace(*c) || *c == '='))) {
     return NULL;
+  }
   end = c;
   while (*c != '\0' && *c != '=' && !is_comment(*c)) {
     c++;
@@ -673,11 +665,12 @@ static const char *get_token_entry_name(struct inputfile *inf)
   if (*c != '=') {
     return NULL;
   }
-  *end = '\0';
-  inf->cur_line_pos = c + 1 - inf->cur_line.str;
-  astr_minsize(&inf->token, strlen(start)+1);
-  strcpy(inf->token.str, start);
-  return inf->token.str;
+  trailing = *end;
+  *((char *) end) = '\0';       /* Tricky. */
+  astr_set(&inf->token, "%s", start);
+  *((char *) end) = trailing;   /* Revert. */
+  inf->cur_line_pos = c + 1 - astr_str(&inf->cur_line);
+  return astr_str(&inf->token);
 }
 
 /********************************************************************** 
@@ -685,25 +678,26 @@ static const char *get_token_entry_name(struct inputfile *inf)
 ***********************************************************************/
 static const char *get_token_eol(struct inputfile *inf)
 {
-  char *c;
-  
-  assert(have_line(inf));
+  const char *c;
+
+  fc_assert_ret_val(have_line(inf), NULL);
 
   if (!at_eol(inf)) {
-    c = inf->cur_line.str + inf->cur_line_pos;
-    while(*c != '\0' && my_isspace(*c)) {
+    c = astr_str(&inf->cur_line) + inf->cur_line_pos;
+    while (*c != '\0' && fc_isspace(*c)) {
       c++;
     }
-    if (*c != '\0' && !is_comment(*c))
+    if (*c != '\0' && !is_comment(*c)) {
       return NULL;
+    }
   }
 
   /* finished with this line: say that we don't have it any more: */
-  inf->cur_line.n = 0;
+  astr_clear(&inf->cur_line);
   inf->cur_line_pos = 0;
-  
-  assign_flag_token(&inf->token, ' ');
-  return inf->token.str;
+
+  astr_set(&inf->token, " ");
+  return astr_str(&inf->token);
 }
 
 /********************************************************************** 
@@ -711,21 +705,22 @@ static const char *get_token_eol(struct inputfile *inf)
   preceeding whitespace.
 ***********************************************************************/
 static const char *get_token_white_char(struct inputfile *inf,
-					char target)
+                                        char target)
 {
-  char *c;
-  
-  assert(have_line(inf));
+  const char *c;
 
-  c = inf->cur_line.str + inf->cur_line_pos;
-  while(*c != '\0' && my_isspace(*c)) {
+  fc_assert_ret_val(have_line(inf), NULL);
+
+  c = astr_str(&inf->cur_line) + inf->cur_line_pos;
+  while (*c != '\0' && fc_isspace(*c)) {
     c++;
   }
-  if (*c != target)
+  if (*c != target) {
     return NULL;
-  inf->cur_line_pos = c + 1 - inf->cur_line.str;
-  assign_flag_token(&inf->token, target);
-  return inf->token.str;
+  }
+  inf->cur_line_pos = c + 1 - astr_str(&inf->cur_line);
+  astr_set(&inf->token, "%c", target);
+  return astr_str(&inf->token);
 }
 
 /********************************************************************** 
@@ -758,52 +753,53 @@ static const char *get_token_comma(struct inputfile *inf)
 static const char *get_token_value(struct inputfile *inf)
 {
   struct astring *partial;
-  char *c, *start;
+  const char *c, *start;
   char trailing;
   bool has_i18n_marking = FALSE;
   char border_character = '\"';
-  
-  assert(have_line(inf));
 
-  c = inf->cur_line.str + inf->cur_line_pos;
-  while(*c != '\0' && my_isspace(*c)) {
+  fc_assert_ret_val(have_line(inf), NULL);
+
+  c = astr_str(&inf->cur_line) + inf->cur_line_pos;
+  while (*c != '\0' && fc_isspace(*c)) {
     c++;
   }
-  if (*c == '\0')
+  if (*c == '\0') {
     return NULL;
+  }
 
-  if (*c == '-' || my_isdigit(*c)) {
+  if (*c == '-' || fc_isdigit(*c)) {
     /* a number: */
     start = c++;
-    while(*c != '\0' && my_isdigit(*c)) {
+    while(*c != '\0' && fc_isdigit(*c)) {
       c++;
     }
     /* check that the trailing stuff is ok: */
-    if (!(*c == '\0' || *c == ',' || my_isspace(*c) || is_comment(*c))) {
+    if (!(*c == '\0' || *c == ',' || fc_isspace(*c) || is_comment(*c))) {
       return NULL;
     }
     /* If its a comma, we don't want to obliterate it permanently,
      * so rememeber it: */
     trailing = *c;
-    *c = '\0';
-    
-    inf->cur_line_pos = c - inf->cur_line.str;
-    astr_minsize(&inf->token, strlen(start)+1);
-    strcpy(inf->token.str, start);
-    
-    *c = trailing;
-    return inf->token.str;
+    *((char *) c) = '\0';       /* Tricky. */
+
+    inf->cur_line_pos = c - astr_str(&inf->cur_line);
+    astr_set(&inf->token, "%s", start);
+
+    *((char *) c) = trailing;   /* Revert. */
+    return astr_str(&inf->token);
   }
 
   /* allow gettext marker: */
-  if (*c == '_' && *(c+1) == '(') {
+  if (*c == '_' && *(c + 1) == '(') {
     has_i18n_marking = TRUE;
     c += 2;
-    while(*c != '\0' && my_isspace(*c)) {
+    while (*c != '\0' && fc_isspace(*c)) {
       c++;
     }
-    if (*c == '\0')
+    if (*c == '\0') {
       return NULL;
+    }
   }
 
   border_character = *c;
@@ -811,7 +807,25 @@ static const char *get_token_value(struct inputfile *inf)
   if (border_character != '\"'
       && border_character != '\''
       && border_character != '$') {
-    return NULL;
+    /* A one-word string: maybe FALSE or TRUE. */
+    start = c;
+    while (fc_isalnum(*c)) {
+      c++;
+    }
+    /* check that the trailing stuff is ok: */
+    if (!(*c == '\0' || *c == ',' || fc_isspace(*c) || is_comment(*c))) {
+      return NULL;
+    }
+    /* If its a comma, we don't want to obliterate it permanently,
+     * so rememeber it: */
+    trailing = *c;
+    *((char *) c) = '\0';       /* Tricky. */
+
+    inf->cur_line_pos = c - astr_str(&inf->cur_line);
+    astr_set(&inf->token, "%s", start);
+
+    *((char *) c) = trailing;   /* Revert. */
+    return astr_str(&inf->token);
   }
 
   /* From here, we know we have a string, we just have to find the
@@ -831,20 +845,17 @@ static const char *get_token_value(struct inputfile *inf)
   inf->string_start_line = inf->line_num;
   inf->in_string = TRUE;
 
-  partial = &inf->partial;	/* abbreviation */
-  astr_minsize(partial, 1);
-  partial->str[0] = '\0';
-  
-  start = c++;			/* start includes the initial \", to
-				   distinguish from a number */
-  for(;;) {
-    int pos;
-    
-    while(*c != '\0' && *c != border_character) {
+  partial = &inf->partial;      /* abbreviation */
+  astr_clear(partial);
+
+  start = c++;                  /* start includes the initial \", to
+                                 * distinguish from a number */
+  for (;;) {
+    while (*c != '\0' && *c != border_character) {
       /* skip over escaped chars, including backslash-doublequote,
-	 and backslash-backslash: */
-      if (*c == '\\' && *(c+1) != '\0') {  
-	c++;
+       * and backslash-backslash: */
+      if (*c == '\\' && *(c + 1) != '\0') {
+        c++;
       }
       c++;
     }
@@ -854,30 +865,25 @@ static const char *get_token_value(struct inputfile *inf)
       break;
     }
 
-    /* Accumulate to partial string and try more lines;
-     * note partial->n must be _exactly_ the right size, so we
-     * can strcpy instead of strcat-ing all the way to the end
-     * each time. */
-    pos = partial->n - 1;
-    astr_minsize(partial, partial->n + c - start + 1);
-    strcpy(partial->str + pos, start);
-    strcpy(partial->str + partial->n - 2, "\n");
-    
+    astr_add(partial, "%s\n", start);
+
     if (!read_a_line(inf)) {
       /* shouldn't happen */
       inf_log(inf, LOG_ERROR, 
               "Bad return for multi-line string from read_a_line");
       return NULL;
     }
-    c = start = inf->cur_line.str;
+    c = start = astr_str(&inf->cur_line);
   }
 
   /* found end of string */
-  *c = '\0';
-  inf->cur_line_pos = c + 1 - inf->cur_line.str;
-  astr_minsize(&inf->token, partial->n + strlen(start));
-  strcpy(inf->token.str, partial->str);
-  strcpy(inf->token.str + partial->n - 1, start);
+  trailing = *c;
+  *((char *) c) = '\0';         /* Tricky. */
+
+  inf->cur_line_pos = c + 1 - astr_str(&inf->cur_line);
+  astr_set(&inf->token, "%s%s", astr_str(partial), start);
+
+  *((char *) c) = trailing;     /* Revert. */
 
   /* check gettext tag at end: */
   if (has_i18n_marking) {
@@ -888,5 +894,5 @@ static const char *get_token_value(struct inputfile *inf)
     }
   }
   inf->in_string = FALSE;
-  return inf->token.str;
+  return astr_str(&inf->token);
 }
