@@ -52,7 +52,6 @@
 
 #include "fcintl.h"
 #include "log.h"
-#include "shared.h"		/* TRUE, FALSE */
 #include "support.h"
 
 #include "netintf.h"
@@ -79,12 +78,11 @@ static void set_socket_errno(void)
       errno = err;
       return;
     default:
-      freelog(LOG_ERROR,
-              "Missing errno mapping for Winsock error #%d.", err);
-      freelog(LOG_ERROR,
-              /* TRANS: No full stop after the URL, could cause confusion. */
-              _("Please report this message at %s"),
-              BUG_URL);
+      log_error("Missing errno mapping for Winsock error #%d.", err);
+      /* TRANS: No full stop after the URL, could cause confusion. */
+      log_error(_("Please report this message at %s"), BUG_URL);
+
+      errno = 0;
   }
 }
 #endif /* HAVE_WINSOCK */
@@ -189,7 +187,7 @@ void fc_init_network(void)
   WSADATA wsa;
 
   if (WSAStartup(MAKEWORD(1, 1), &wsa) != 0) {
-    freelog(LOG_ERROR, "no usable WINSOCK.DLL: %s", fc_strerror(fc_get_errno()));
+    log_error("no usable WINSOCK.DLL: %s", fc_strerror(fc_get_errno()));
   }
 #endif
 
@@ -222,27 +220,27 @@ void fc_nonblock(int sockfd)
 #ifdef HAVE_FCNTL
   int f_set;
 
-  if ((f_set=fcntl(sockfd, F_GETFL)) == -1) {
-    freelog(LOG_ERROR, "fcntl F_GETFL failed: %s", fc_strerror(fc_get_errno()));
+  if ((f_set = fcntl(sockfd, F_GETFL)) == -1) {
+    log_error("fcntl F_GETFL failed: %s", fc_strerror(fc_get_errno()));
   }
 
   f_set |= O_NONBLOCK;
 
   if (fcntl(sockfd, F_SETFL, f_set) == -1) {
-    freelog(LOG_ERROR, "fcntl F_SETFL failed: %s", fc_strerror(fc_get_errno()));
+    log_error("fcntl F_SETFL failed: %s", fc_strerror(fc_get_errno()));
   }
 #else
 #ifdef HAVE_IOCTL
   long value=1;
 
   if (ioctl(sockfd, FIONBIO, (char*)&value) == -1) {
-    freelog(LOG_ERROR, "ioctl failed: %s", fc_strerror(fc_get_errno()));
+    log_error("ioctl failed: %s", fc_strerror(fc_get_errno()));
   }
 #endif
 #endif
 #endif
 #else
-  freelog(LOG_DEBUG, "NONBLOCKING_SOCKETS not available");
+  log_debug("NONBLOCKING_SOCKETS not available");
 #endif
 }
 
@@ -256,20 +254,20 @@ void sockaddr_debug(union fc_sockaddr *addr)
 
   if (addr->saddr.sa_family == AF_INET6) { 
     inet_ntop(AF_INET6, &addr->saddr_in6.sin6_addr, buf, INET6_ADDRSTRLEN);
-    freelog(LOG_DEBUG, "Host: %s, Port: %d (IPv6)",
-            buf, ntohs(addr->saddr_in6.sin6_port));
+    log_debug("Host: %s, Port: %d (IPv6)",
+              buf, ntohs(addr->saddr_in6.sin6_port));
   } else {
     inet_ntop(AF_INET, &addr->saddr_in4.sin_addr, buf, INET_ADDRSTRLEN);
-    freelog(LOG_DEBUG, "Host: %s, Port: %d (IPv4)",
-            buf, ntohs(addr->saddr_in4.sin_port));
+    log_debug("Host: %s, Port: %d (IPv4)",
+              buf, ntohs(addr->saddr_in4.sin_port));
   }
 #else  /* IPv6 support */
   char *buf;
 
   buf = inet_ntoa(addr->saddr_in4.sin_addr);
 
-  freelog(LOG_DEBUG, "Host: %s, Port: %d",
-          buf, ntohs(addr->saddr_in4.sin_port));
+  log_debug("Host: %s, Port: %d",
+            buf, ntohs(addr->saddr_in4.sin_port));
 #endif /* IPv6 support */
 }
 
@@ -321,46 +319,43 @@ bool net_lookup_service(const char *name, int port, union fc_sockaddr *addr,
 #ifdef IPV6_SUPPORT
   sock6 = &addr->saddr_in6;
 
-    if (!force_ipv4) {
-    addr->saddr.sa_family = AF_INET6;
-    sock6->sin6_port = htons(port);
+  if (!force_ipv4) {
+    struct addrinfo hints, *res0;
+    int error;
+    char servname[8];
 
-    if (!name) {
-      sock6->sin6_addr = in6addr_any;
-      return TRUE;
-    }
+    /* Convert port to string for getaddrinfo() */
+    fc_snprintf(servname, sizeof(servname), "%d", port);
 
-    if (inet_pton(AF_INET6, name, &sock6->sin6_addr)) {
+    /* Use getaddrinfo() to lookup IPv6 addresses */
+    memset(&hints, 0, sizeof(hints));
+    hints.ai_family = AF_INET6;     /* request only IPv6 */
+    hints.ai_socktype = SOCK_DGRAM; /* any type that uses sin6_port */
+    hints.ai_flags = AI_PASSIVE | AI_NUMERICSERV;
+    error = getaddrinfo(name, servname, &hints, &res0);
+
+    if (error == 0) {
+      /* getaddrinfo() provides a linked list of addresses, but
+       * net_lookup_service() wants only one address.
+       *
+       * We handle this by copying the first address from the list,
+       * then freeing the list. */
+      memcpy(sock6, res0->ai_addr, MIN(sizeof(*sock6), res0->ai_addrlen));
+      freeaddrinfo(res0);
       return TRUE;
-    }
-    /* TODO: Replace gethostbyname2() with getaddrinfo() */
-    hp = gethostbyname2(name, AF_INET6);
-  } else
+    } /* else continue to IPv4 */
+  }
 #endif /* IPv6 support */
-  {
-    addr->saddr.sa_family = AF_INET;
-    sock4->sin_port = htons(port);
 
-    if (!name) {
-      sock4->sin_addr.s_addr = htonl(INADDR_ANY);
-      return TRUE;
-    }
+  /* IPv4 resolution */
+  addr->saddr.sa_family = AF_INET;
+  sock4->sin_port = htons(port);
+
+  if (!name) {
+    sock4->sin_addr.s_addr = htonl(INADDR_ANY);
+    return TRUE;
   }
 
-#ifdef IPV6_SUPPORT
-  if (force_ipv4 || !hp || hp->h_addrtype != AF_INET6) {
-    /* Try to fallback to IPv4 resolution */
-    if (!force_ipv4) {
-      freelog(LOG_DEBUG, "Falling back to IPv4");
-    }
-    hp = gethostbyname2(name, AF_INET);
-    if (!hp || hp->h_addrtype != AF_INET) {
-      return FALSE;
-    }
-    addr->saddr.sa_family = AF_INET;
-    sock4->sin_port = htons(port);
-  }
-#else  /* IPV6 support */
 #if defined(HAVE_INET_ATON)
   if (inet_aton(name, &sock4->sin_addr) != 0) {
     return TRUE;
@@ -374,17 +369,8 @@ bool net_lookup_service(const char *name, int port, union fc_sockaddr *addr,
   if (!hp || hp->h_addrtype != AF_INET) {
     return FALSE;
   }
-#endif /* IPv6 support */
-
-#ifdef IPV6_SUPPORT
-  if (addr->saddr.sa_family == AF_INET6) {
-    memcpy(&sock6->sin6_addr, hp->h_addr, hp->h_length);
-  } else
-#endif /* IPv6 support */
-  {
-    memcpy(&sock4->sin_addr, hp->h_addr, hp->h_length);
-  }
 
+  memcpy(&sock4->sin_addr, hp->h_addr, hp->h_length);
   return TRUE;
 }
 
@@ -399,7 +385,7 @@ fz_FILE *fc_querysocket(int sock, void *buf, size_t size)
 #ifdef HAVE_FDOPEN
   fp = fdopen(sock, "r+b");
   if (fwrite(buf, 1, size, fp) != size) {
-    die("socket %d: write error", sock);
+    log_error("socket %d: write error", sock);
   }
   fflush(fp);
 
@@ -435,7 +421,7 @@ fz_FILE *fc_querysocket(int sock, void *buf, size_t size)
 
     while ((n = fc_readsocket(sock, tmp, sizeof(tmp))) > 0) {
       if (fwrite(tmp, 1, n, fp) != n) {
-	die("socket %d: write error", sock);
+        log_error("socket %d: write error", sock);
       }
     }
     fflush(fp);
@@ -503,7 +489,7 @@ const char *fc_lookup_httpd(char *server, int *port, const char *url)
   }
 
   /* snarf port. */
-  if (!pport || sscanf(pport+1, "%d", port) != 1) {
+  if (NULL == pport || !str_to_int(pport + 1, port)) {
     *port = 80;
   }
 
